@@ -39,7 +39,7 @@ class PoissonFactory
 {
   public:
 
-  static std::shared_ptr<Form> a(const Mesh& mesh)
+  static std::shared_ptr<Form> a(std::shared_ptr<const Mesh> mesh)
   {
     // Create function space
     std::shared_ptr<FunctionSpace> _V(new Poisson::FunctionSpace(mesh));
@@ -53,7 +53,7 @@ class NavierStokesFactory
 {
   public:
 
-  static std::shared_ptr<Form> a(const Mesh& mesh)
+  static std::shared_ptr<Form> a(std::shared_ptr<const Mesh> mesh)
   {
     std::shared_ptr<FunctionSpace> _V(new NavierStokes::FunctionSpace(mesh));
 
@@ -97,7 +97,6 @@ double bench(std::string form, std::shared_ptr<const Form> a)
 
   // Assemble once to initialize matrix
   Assembler assembler;
-  assembler.reset_sparsity = false;
   assemble(A, *a);
 
   // Run timing
@@ -106,8 +105,9 @@ double bench(std::string form, std::shared_ptr<const Form> a)
     assemble(A, *a);
   const double t = timer.stop();
 
-  // Write summary
-  summary(true);
+  // Report timings
+  list_timings(TimingClear::clear,
+               { TimingType::wall, TimingType::user, TimingType::system });
 
   info("");
 
@@ -116,16 +116,15 @@ double bench(std::string form, std::shared_ptr<const Form> a)
 
 int main(int argc, char* argv[])
 {
+  info("Runtime of threaded assembly benchmark");
+
   // Parse command-line arguments
   parameters.parse(argc, argv);
 
-  // Set backend
-  //parameters["linear_algebra_backend"] = "Epetra";
-
   // Create mesh
-  UnitCubeMesh old_mesh(SIZE, SIZE, SIZE);
-  old_mesh.color("vertex");
-  Mesh mesh = old_mesh.renumber_by_color();
+  auto old_mesh = std::make_shared<UnitCubeMesh>(SIZE, SIZE, SIZE);
+  old_mesh->color("vertex");
+  auto mesh = std::make_shared<Mesh>(old_mesh->renumber_by_color());
 
   // Test cases
   std::vector<std::pair<std::string, std::shared_ptr<const Form> > > forms;
