@@ -66,18 +66,18 @@ int main()
   parameters["ghost_mode"] = "shared_facet";
 
   // Create mesh
-  UnitSquareMesh mesh(32, 32);
+  auto mesh = std::make_shared<UnitSquareMesh>(32, 32);
 
   // Create functions
-  Source f;
-  Constant alpha(8.0);
+  auto f = std::make_shared<Source>();
+  auto alpha = std::make_shared<Constant>(8.0);
 
   // Create function space
-  Biharmonic::FunctionSpace V(mesh);
+  auto V = std::make_shared<Biharmonic::FunctionSpace>(mesh);
 
   // Define boundary condition
-  Constant u0(0.0);
-  DirichletBoundary boundary;
+  auto u0 = std::make_shared<Constant>(0.0);
+  auto boundary = std::make_shared<DirichletBoundary>();
   DirichletBC bc(V, u0, boundary);
 
   // Define variational problem
