@@ -71,8 +71,8 @@ bottom.mark(boundaries, 4)
 # Define input data
 a0 = Constant(1.0)
 a1 = Constant(0.01)
-g_L = Expression("- 10*exp(- pow(x[1] - 0.5, 2))")
-g_R = Constant("1.0")
+g_L = Expression("- 10*exp(- pow(x[1] - 0.5, 2))", degree=2)
+g_R = Constant(1.0)
 f = Constant(1.0)
 
 # Define function space and basis functions
@@ -86,8 +86,8 @@ bcs = [DirichletBC(V, 5.0, boundaries, 2),
 
 # Define new measures associated with the interior domains and
 # exterior boundaries
-dx = Measure("dx")[domains]
-ds = Measure("ds")[boundaries]
+dx = Measure('dx', domain=mesh, subdomain_data=domains)
+ds = Measure('ds', domain=mesh, subdomain_data=boundaries)
 
 # Define variational form
 F = (inner(a0*grad(u), grad(v))*dx(0) + inner(a1*grad(u), grad(v))*dx(1)
