@@ -24,8 +24,8 @@
 #ifndef __MPI_DOLFIN_WRAPPER_H
 #define __MPI_DOLFIN_WRAPPER_H
 
+#include <cstdint>
 #include <iostream>
-
 #include <numeric>
 #include <type_traits>
 #include <utility>
@@ -36,7 +36,8 @@
 #include <mpi.h>
 #endif
 
-#include <dolfin/log/dolfin_log.h>
+#include <dolfin/log/log.h>
+#include <dolfin/log/Table.h>
 
 #ifndef HAS_MPI
 typedef int MPI_Comm;
@@ -71,103 +72,106 @@ namespace dolfin
   public:
 
     /// Return process rank for the communicator
-    static unsigned int rank(const MPI_Comm comm);
+    static unsigned int rank(MPI_Comm comm);
 
     /// Return size of the group (number of processes) associated with
     /// the communicator
-    static unsigned int size(const MPI_Comm comm);
+    static unsigned int size(MPI_Comm comm);
 
     /// Determine whether we should broadcast (based on current
     /// parallel policy)
-    static bool is_broadcaster(const MPI_Comm comm);
+    static bool is_broadcaster(MPI_Comm comm);
 
     /// Determine whether we should receive (based on current parallel
     /// policy)
-    static bool is_receiver(const MPI_Comm comm);
+    static bool is_receiver(MPI_Comm comm);
 
     /// Set a barrier (synchronization point)
-    static void barrier(const MPI_Comm comm);
+    static void barrier(MPI_Comm comm);
 
     /// Send in_values[p0] to process p0 and receive values from
     /// process p1 in out_values[p1]
     template<typename T>
-      static void all_to_all(const MPI_Comm comm,
+      static void all_to_all(MPI_Comm comm,
                              std::vector<std::vector<T> >& in_values,
                              std::vector<std::vector<T> >& out_values);
 
     /// Broadcast vector of value from broadcaster to all processes
     template<typename T>
-      static void broadcast(const MPI_Comm comm, std::vector<T>& value,
+      static void broadcast(MPI_Comm comm, std::vector<T>& value,
                             unsigned int broadcaster=0);
 
     /// Broadcast single primitive from broadcaster to all processes
     template<typename T>
-      static void broadcast(const MPI_Comm comm, T& value,
+      static void broadcast(MPI_Comm comm, T& value,
                             unsigned int broadcaster=0);
 
     /// Scatter vector in_values[i] to process i
     template<typename T>
-      static void scatter(const MPI_Comm comm,
+      static void scatter(MPI_Comm comm,
                           const std::vector<std::vector<T> >& in_values,
                           std::vector<T>& out_value,
                           unsigned int sending_process=0);
 
     /// Scatter primitive in_values[i] to process i
     template<typename T>
-      static void scatter(const MPI_Comm comm,
+      static void scatter(MPI_Comm comm,
                           const std::vector<T>& in_values,
                           T& out_value, unsigned int sending_process=0);
 
     /// Gather values on one process
     template<typename T>
-      static void gather(const MPI_Comm comm, const std::vector<T>& in_values,
+      static void gather(MPI_Comm comm, const std::vector<T>& in_values,
                          std::vector<T>& out_values,
                          unsigned int receiving_process=0);
 
     /// Gather strings on one process
-    static void gather(const MPI_Comm comm, const std::string& in_values,
+    static void gather(MPI_Comm comm, const std::string& in_values,
                        std::vector<std::string>& out_values,
                        unsigned int receiving_process=0);
 
     /// Gather values from all processes. Same data count from each
     /// process (wrapper for MPI_Allgather)
     template<typename T>
-      static void all_gather(const MPI_Comm comm,
+      static void all_gather(MPI_Comm comm,
                              const std::vector<T>& in_values,
                              std::vector<T>& out_values);
 
     /// Gather values from each process (variable count per process)
     template<typename T>
-      static void all_gather(const MPI_Comm comm,
+      static void all_gather(MPI_Comm comm,
                              const std::vector<T>& in_values,
                              std::vector<std::vector<T> >& out_values);
 
     /// Gather values, one primitive from each process (MPI_Allgather)
     template<typename T>
-      static void all_gather(const MPI_Comm comm, const T& in_value,
+      static void all_gather(MPI_Comm comm, const T in_value,
                              std::vector<T>& out_values);
 
     /// Return global max value
-    template<typename T> static T max(const MPI_Comm comm, const T& value);
+    template<typename T> static T max(MPI_Comm comm, const T& value);
 
     /// Return global min value
-    template<typename T> static T min(const MPI_Comm comm, const T& value);
+    template<typename T> static T min(MPI_Comm comm, const T& value);
 
     /// Sum values and return sum
-    template<typename T> static T sum(const MPI_Comm comm, const T& value);
+    template<typename T> static T sum(MPI_Comm comm, const T& value);
+
+    /// Return average across comm; implemented only for T == Table
+    template<typename T> static T avg(MPI_Comm comm, const T& value);
 
     /// All reduce
     template<typename T, typename X> static
-      T all_reduce(const MPI_Comm comm, const T& value, X op);
+      T all_reduce(MPI_Comm comm, const T& value, X op);
 
     /// Find global offset (index) (wrapper for MPI_(Ex)Scan with
     /// MPI_SUM as reduction op)
-    static std::size_t global_offset(const MPI_Comm comm,
+    static std::size_t global_offset(MPI_Comm comm,
                                      std::size_t range, bool exclusive);
 
     /// Send-receive data between processes (blocking)
     template<typename T>
-      static void send_recv(const MPI_Comm comm,
+      static void send_recv(MPI_Comm comm,
                             const std::vector<T>& send_value,
                             unsigned int dest, int send_tag,
                             std::vector<T>& recv_value,
@@ -175,31 +179,35 @@ namespace dolfin
 
     /// Send-receive data between processes
     template<typename T>
-      static void send_recv(const MPI_Comm comm,
+      static void send_recv(MPI_Comm comm,
                             const std::vector<T>& send_value, unsigned int dest,
                             std::vector<T>& recv_value, unsigned int source);
 
     /// Return local range for local process, splitting [0, N - 1] into
     /// size() portions of almost equal size
-    static std::pair<std::size_t, std::size_t>
-      local_range(const MPI_Comm comm, std::size_t N);
+    static std::pair<std::int64_t, std::int64_t>
+      local_range(MPI_Comm comm, std::int64_t N);
 
     /// Return local range for given process, splitting [0, N - 1] into
     /// size() portions of almost equal size
-    static std::pair<std::size_t, std::size_t>
-      local_range(const MPI_Comm comm, unsigned int process,
-                  std::size_t N);
+    static std::pair<std::int64_t, std::int64_t>
+      local_range(MPI_Comm comm, int process, std::int64_t N);
 
     /// Return local range for given process, splitting [0, N - 1] into
     /// size() portions of almost equal size
-    static std::pair<std::size_t, std::size_t>
-      compute_local_range(unsigned int process, std::size_t N,
-                          unsigned int size);
+    static std::pair<std::int64_t, std::int64_t>
+      compute_local_range(int process, std::int64_t N, int size);
 
     /// Return which process owns index (inverse of local_range)
-    static unsigned int index_owner(const MPI_Comm comm,
+    static unsigned int index_owner(MPI_Comm comm,
                                     std::size_t index, std::size_t N);
 
+    #ifdef HAS_MPI
+    /// Return average reduction operation; recognized by
+    /// all_reduce(MPI_Comm, Table&, MPI_Op)
+    static MPI_Op MPI_AVG();
+    #endif
+
   private:
 
     #ifndef HAS_MPI
@@ -225,26 +233,27 @@ namespace dolfin
     }
     #endif
 
+    #ifdef HAS_MPI
+    // Maps some MPI_Op values to string
+    static std::map<MPI_Op, std::string> operation_map;
+    #endif
+
   };
 
   #ifdef HAS_MPI
   // Specialisations for MPI_Datatypes
   template<> inline MPI_Datatype MPI::mpi_type<float>() { return MPI_FLOAT; }
   template<> inline MPI_Datatype MPI::mpi_type<double>() { return MPI_DOUBLE; }
-  template<> inline MPI_Datatype MPI::mpi_type<short int>()
-  { return MPI_SHORT; }
+  template<> inline MPI_Datatype MPI::mpi_type<short int>() { return MPI_SHORT; }
   template<> inline MPI_Datatype MPI::mpi_type<int>() { return MPI_INT; }
   template<> inline MPI_Datatype MPI::mpi_type<long int>() { return MPI_LONG; }
-  template<> inline MPI_Datatype MPI::mpi_type<unsigned int>()
-  { return MPI_UNSIGNED; }
-  template<> inline MPI_Datatype MPI::mpi_type<unsigned long int>()
-  { return MPI_UNSIGNED_LONG; }
-  template<> inline MPI_Datatype MPI::mpi_type<long long>()
-  { return MPI_LONG_LONG; }
+  template<> inline MPI_Datatype MPI::mpi_type<unsigned int>() { return MPI_UNSIGNED; }
+  template<> inline MPI_Datatype MPI::mpi_type<unsigned long int>() { return MPI_UNSIGNED_LONG; }
+  template<> inline MPI_Datatype MPI::mpi_type<long long>() { return MPI_LONG_LONG; }
   #endif
   //---------------------------------------------------------------------------
   template<typename T>
-    void dolfin::MPI::broadcast(const MPI_Comm comm, std::vector<T>& value,
+    void dolfin::MPI::broadcast(MPI_Comm comm, std::vector<T>& value,
                                 unsigned int broadcaster)
   {
     #ifdef HAS_MPI
@@ -260,7 +269,7 @@ namespace dolfin
   }
   //---------------------------------------------------------------------------
   template<typename T>
-    void dolfin::MPI::broadcast(const MPI_Comm comm, T& value,
+    void dolfin::MPI::broadcast(MPI_Comm comm, T& value,
                                 unsigned int broadcaster)
   {
     #ifdef HAS_MPI
@@ -269,7 +278,7 @@ namespace dolfin
   }
   //---------------------------------------------------------------------------
   template<typename T>
-    void dolfin::MPI::all_to_all(const MPI_Comm comm,
+    void dolfin::MPI::all_to_all(MPI_Comm comm,
                                  std::vector<std::vector<T> >& in_values,
                                  std::vector<std::vector<T> >& out_values)
   {
@@ -325,7 +334,7 @@ namespace dolfin
   }
   //---------------------------------------------------------------------------
   template<> inline
-    void dolfin::MPI::all_to_all(const MPI_Comm comm,
+    void dolfin::MPI::all_to_all(MPI_Comm comm,
                                  std::vector<std::vector<bool> >& in_values,
                                  std::vector<std::vector<bool> >& out_values)
   {
@@ -350,7 +359,7 @@ namespace dolfin
   }
   //---------------------------------------------------------------------------
   template<typename T>
-    void dolfin::MPI::scatter(const MPI_Comm comm,
+    void dolfin::MPI::scatter(MPI_Comm comm,
                               const std::vector<std::vector<T> >& in_values,
                               std::vector<T>& out_value,
                               unsigned int sending_process)
@@ -405,7 +414,7 @@ namespace dolfin
   }
   //---------------------------------------------------------------------------
   template<> inline
-    void dolfin::MPI::scatter(const MPI_Comm comm,
+    void dolfin::MPI::scatter(MPI_Comm comm,
                               const std::vector<std::vector<bool> >& in_values,
                               std::vector<bool>& out_value,
                               unsigned int sending_process)
@@ -429,7 +438,7 @@ namespace dolfin
   }
   //---------------------------------------------------------------------------
   template<typename T>
-    void dolfin::MPI::scatter(const MPI_Comm comm,
+    void dolfin::MPI::scatter(MPI_Comm comm,
                               const std::vector<T>& in_values,
                               T& out_value, unsigned int sending_process)
   {
@@ -447,7 +456,7 @@ namespace dolfin
   }
   //---------------------------------------------------------------------------
   template<typename T>
-  void dolfin::MPI::gather(const MPI_Comm comm,
+  void dolfin::MPI::gather(MPI_Comm comm,
                            const std::vector<T>& in_values,
                            std::vector<T>& out_values,
                            unsigned int receiving_process)
@@ -478,7 +487,7 @@ namespace dolfin
     #endif
   }
   //---------------------------------------------------------------------------
-  inline void dolfin::MPI::gather(const MPI_Comm comm,
+  inline void dolfin::MPI::gather(MPI_Comm comm,
                                   const std::string& in_values,
                                   std::vector<std::string>& out_values,
                                   unsigned int receiving_process)
@@ -520,7 +529,7 @@ namespace dolfin
   }
   //-------------------------------------------------------------------------
   template<typename T>
-    void dolfin::MPI::all_gather(const MPI_Comm comm,
+    void dolfin::MPI::all_gather(MPI_Comm comm,
                                  const std::vector<T>& in_values,
                                  std::vector<T>& out_values)
   {
@@ -536,7 +545,7 @@ namespace dolfin
   }
   //---------------------------------------------------------------------------
   template<typename T>
-    void dolfin::MPI::all_gather(const MPI_Comm comm,
+    void dolfin::MPI::all_gather(MPI_Comm comm,
                                  const std::vector<T>& in_values,
                                  std::vector<std::vector<T> >& out_values)
   {
@@ -577,7 +586,7 @@ namespace dolfin
   }
   //---------------------------------------------------------------------------
   template<typename T>
-    void dolfin::MPI::all_gather(const MPI_Comm comm, const T& in_value,
+    void dolfin::MPI::all_gather(MPI_Comm comm, const T in_value,
                                  std::vector<T>& out_values)
   {
     #ifdef HAS_MPI
@@ -591,49 +600,66 @@ namespace dolfin
   }
   //---------------------------------------------------------------------------
   template<typename T, typename X>
-    T dolfin::MPI::all_reduce(const MPI_Comm comm, const T& value, X op)
+    T dolfin::MPI::all_reduce(MPI_Comm comm, const T& value, X op)
   {
     #ifdef HAS_MPI
     T out;
     MPI_Allreduce(const_cast<T*>(&value), &out, 1, mpi_type<T>(), op, comm);
     return out;
     #else
+    return value;
+    #endif
+  }
+  //---------------------------------------------------------------------------
+  template<typename T> T dolfin::MPI::max(MPI_Comm comm, const T& value)
+  {
+    #ifdef HAS_MPI
+    // Enforce cast to MPI_Op; this is needed because template dispatch may
+    // not recognize this is possible, e.g. C-enum to unsigned int in SGI MPT
+    MPI_Op op = static_cast<MPI_Op>(MPI_MAX);
+    return all_reduce(comm, value, op);
+    #else
+    return value;
+    #endif
+  }
+  //---------------------------------------------------------------------------
+  template<typename T> T dolfin::MPI::min(MPI_Comm comm, const T& value)
+  {
+    #ifdef HAS_MPI
+    // Enforce cast to MPI_Op; this is needed because template dispatch may
+    // not recognize this is possible, e.g. C-enum to unsigned int in SGI MPT
+    MPI_Op op = static_cast<MPI_Op>(MPI_MIN);
+    return all_reduce(comm, value, op);
+    #else
+    return value;
+    #endif
+  }
+  //---------------------------------------------------------------------------
+  template<typename T> T dolfin::MPI::sum(MPI_Comm comm, const T& value)
+  {
+    #ifdef HAS_MPI
+    // Enforce cast to MPI_Op; this is needed because template dispatch may
+    // not recognize this is possible, e.g. C-enum to unsigned int in SGI MPT
+    MPI_Op op = static_cast<MPI_Op>(MPI_SUM);
+    return all_reduce(comm, value, op);
+    #else
+    return value;
+    #endif
+  }
+  //---------------------------------------------------------------------------
+  template<typename T> T dolfin::MPI::avg(MPI_Comm comm, const T& value)
+  {
+    #ifdef HAS_MPI
     dolfin_error("MPI.h",
-      "call MPI::all_reduce",
-      "DOLFIN has been configured without MPI support");
-    return T(0);
-    #endif
-  }
-  //---------------------------------------------------------------------------
-  template<typename T> T dolfin::MPI::max(const MPI_Comm comm, const T& value)
-  {
-    #ifdef HAS_MPI
-    return all_reduce(comm, value, MPI_MAX);
-    #else
-    return value;
-    #endif
-  }
-  //---------------------------------------------------------------------------
-  template<typename T> T dolfin::MPI::min(const MPI_Comm comm, const T& value)
-  {
-    #ifdef HAS_MPI
-    return all_reduce(comm, value, MPI_MIN);
-    #else
-    return value;
-    #endif
-  }
-  //---------------------------------------------------------------------------
-  template<typename T> T dolfin::MPI::sum(const MPI_Comm comm, const T& value)
-  {
-    #ifdef HAS_MPI
-    return all_reduce(comm, value, MPI_SUM);
+                 "perform average reduction",
+                 "Not implemented for this type");
     #else
     return value;
     #endif
   }
   //---------------------------------------------------------------------------
   template<typename T>
-    void dolfin::MPI::send_recv(const MPI_Comm comm,
+    void dolfin::MPI::send_recv(MPI_Comm comm,
                                 const std::vector<T>& send_value,
                                 unsigned int dest, int send_tag,
                                 std::vector<T>& recv_value,
@@ -660,7 +686,7 @@ namespace dolfin
   }
   //---------------------------------------------------------------------------
   template<typename T>
-    void dolfin::MPI::send_recv(const MPI_Comm comm,
+    void dolfin::MPI::send_recv(MPI_Comm comm,
                                 const std::vector<T>& send_value,
                                 unsigned int dest,
                                 std::vector<T>& recv_value,
@@ -669,6 +695,20 @@ namespace dolfin
     MPI::send_recv(comm, send_value, dest, 0, recv_value, source, 0);
   }
   //---------------------------------------------------------------------------
+  // Specialization for dolfin::log::Table class
+  // NOTE: This function is not trully "all_reduce", it reduces to rank 0
+  //       and returns zero Table on other ranks.
+  #ifdef HAS_MPI
+  template<>
+    Table dolfin::MPI::all_reduce(MPI_Comm, const Table&, MPI_Op);
+  #endif
+  //---------------------------------------------------------------------------
+  // Specialization for dolfin::log::Table class
+  #ifdef HAS_MPI
+  template<>
+    Table dolfin::MPI::avg(MPI_Comm, const Table&);
+  #endif
+  //---------------------------------------------------------------------------
 }
 
 #endif
