@@ -26,8 +26,8 @@
 
 #include <cstring>
 #include <limits>
-#include <string>
 #include <sstream>
+#include <string>
 #include <vector>
 #include <boost/functional/hash.hpp>
 #include <dolfin/common/MPI.h>
@@ -38,12 +38,20 @@ namespace dolfin
   /// Indent string block
   std::string indent(std::string block);
 
-  /// Return string representation of int
-  template <class T>
-  std::string to_string(T x)
+  /// Return string representation of given container of ints, floats,
+  /// etc.
+  template<typename T>
+    std::string container_to_string(const T& x, std::string delimiter,
+                                    int precision)
   {
     std::stringstream s;
-    s << x;
+    s.precision(precision);
+    if (!x.empty())
+    {
+      s << *x.begin();
+      for (auto it = x.begin() + 1; it != x.end(); ++it)
+        s << delimiter << *it;
+    }
     return s.str();
   }
 
@@ -62,41 +70,25 @@ namespace dolfin
   /// on each process, and the hash of the std::vector of all local hash
   /// keys is returned. This function is collective.
   template <class T>
-  std::size_t hash_global(const T& x)
+    std::size_t hash_global(const MPI_Comm mpi_comm, const T& x)
   {
     // Compute local hash
     std::size_t local_hash = hash_local(x);
 
     // Gather hash keys on root process
     std::vector<std::size_t> all_hashes;
-    MPI::gather(local_hash, all_hashes);
+    std::vector<std::size_t> local_hash_tmp(1, local_hash);
+    MPI::gather(mpi_comm, local_hash_tmp, all_hashes);
 
     // Hash the received hash keys
-    boost::hash<std::vector<std::size_t> > hash;
+    boost::hash<std::vector<std::size_t>> hash;
     std::size_t global_hash = hash(all_hashes);
 
     // Broadcast hash key to all processes
-    MPI::broadcast(global_hash);
+    MPI::broadcast(mpi_comm, global_hash);
     return global_hash;
   }
 
-  /// Fast zero-fill of numeric vectors/blocks.
-  template <class T>
-  void zerofill(T* arr, std::size_t n)
-  {
-    if (std::numeric_limits<T>::is_integer || std::numeric_limits<T>::is_iec559)
-      std::memset(arr, 0, n*sizeof(T));
-    else
-    {
-      // should never happen in practice
-      std::fill_n(arr, n, T(0));
-    }
-  }
-
-  template <class T>
-  void zerofill(std::vector<T>& vec)
-  { zerofill(&vec[0], vec.size()); }
-
 }
 
 #endif
