@@ -1,4 +1,4 @@
-// Copyright (C) 2007-2013 Anders Logg and Garth N. Wells
+// Copyright (C) 2007-2016 Anders Logg and Garth N. Wells
 //
 // This file is part of DOLFIN.
 //
@@ -15,16 +15,13 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
 //
-// Modified by Martin Alnes, 2008, 2013
-// Modified by Kent-Andre Mardal, 2009
-// Modified by Ola Skavhaug, 2009
-// Modified by Niclas Jansson, 2009
-// Modified by Joachim B Haga, 2012
-// Modified by Mikael Mortensen, 2012
-// Modified by Jan Blechta, 2013
-//
-// First added:  2007-03-01
-// Last changed: 2014-09-08
+// Modified by Martin Alnes 2008-2015
+// Modified by Kent-Andre Mardal 2009
+// Modified by Ola Skavhaug 2009
+// Modified by Niclas Jansson 2009
+// Modified by Joachim B Haga 2012
+// Modified by Mikael Mortensen 2012
+// Modified by Jan Blechta 2013
 
 #include <unordered_map>
 
@@ -33,6 +30,7 @@
 #include <dolfin/common/types.h>
 #include <dolfin/la/GenericVector.h>
 #include <dolfin/log/LogStream.h>
+#include <dolfin/mesh/MeshEntityIterator.h>
 #include <dolfin/mesh/PeriodicBoundaryComputation.h>
 #include <dolfin/mesh/Vertex.h>
 #include "DofMapBuilder.h"
@@ -43,8 +41,9 @@ using namespace dolfin;
 //-----------------------------------------------------------------------------
 DofMap::DofMap(std::shared_ptr<const ufc::dofmap> ufc_dofmap,
                const Mesh& mesh)
-  : _ufc_dofmap(ufc_dofmap), _is_view(false), _global_dimension(0),
-    _ufc_offset(0), _global_offset(0)
+  : _cell_dimension(0), _ufc_dofmap(ufc_dofmap), _is_view(false),
+    _global_dimension(0), _ufc_offset(0), _multimesh_offset(0),
+    _index_map(new IndexMap(mesh.mpi_comm()))
 {
   dolfin_assert(_ufc_dofmap);
 
@@ -55,8 +54,9 @@ DofMap::DofMap(std::shared_ptr<const ufc::dofmap> ufc_dofmap,
 DofMap::DofMap(std::shared_ptr<const ufc::dofmap> ufc_dofmap,
                const Mesh& mesh,
                std::shared_ptr<const SubDomain> constrained_domain)
-  : _ufc_dofmap(ufc_dofmap), _is_view(false), _global_dimension(0),
-    _ufc_offset(0), _global_offset(0)
+  : _cell_dimension(0), _ufc_dofmap(ufc_dofmap), _is_view(false),
+    _global_dimension(0), _ufc_offset(0), _multimesh_offset(0),
+    _index_map(new IndexMap(mesh.mpi_comm()))
 {
   dolfin_assert(_ufc_dofmap);
 
@@ -68,10 +68,10 @@ DofMap::DofMap(std::shared_ptr<const ufc::dofmap> ufc_dofmap,
 }
 //-----------------------------------------------------------------------------
 DofMap::DofMap(const DofMap& parent_dofmap,
-  const std::vector<std::size_t>& component, const Mesh& mesh)
-  : _is_view(true), _global_dimension(0), _ufc_offset(0),
-    _global_offset(parent_dofmap._global_offset),
-    _local_ownership_size(parent_dofmap._local_ownership_size)
+               const std::vector<std::size_t>& component, const Mesh& mesh)
+  : _cell_dimension(0), _ufc_dofmap(0), _is_view(true),
+    _global_dimension(0), _ufc_offset(0), _multimesh_offset(0),
+    _index_map(parent_dofmap._index_map)
 {
   // Build sub-dofmap
   DofMapBuilder::build_sub_map_view(*this, parent_dofmap, component, mesh);
@@ -79,9 +79,9 @@ DofMap::DofMap(const DofMap& parent_dofmap,
 //-----------------------------------------------------------------------------
 DofMap::DofMap(std::unordered_map<std::size_t, std::size_t>& collapsed_map,
                const DofMap& dofmap_view, const Mesh& mesh)
-  :  _ufc_dofmap(dofmap_view._ufc_dofmap), _is_view(false),
-     _global_dimension(0), _ufc_offset(0), _global_offset(0),
-     _local_ownership_size(0)
+  : _cell_dimension(0), _ufc_dofmap(dofmap_view._ufc_dofmap), _is_view(false),
+    _global_dimension(0), _ufc_offset(0), _multimesh_offset(0),
+    _index_map(new IndexMap(mesh.mpi_comm()))
 {
   dolfin_assert(_ufc_dofmap);
 
@@ -104,9 +104,10 @@ DofMap::DofMap(std::unordered_map<std::size_t, std::size_t>& collapsed_map,
   DofMapBuilder::build(*this, mesh, constrained_domain);
 
   // Dimension sanity checks
-  dolfin_assert(dofmap_view._dofmap.size() == mesh.num_cells());
+  dolfin_assert(dofmap_view._dofmap.size()
+                == mesh.num_cells()*dofmap_view._cell_dimension);
   dolfin_assert(global_dimension() == dofmap_view.global_dimension());
-  dolfin_assert(_dofmap.size() == mesh.num_cells());
+  dolfin_assert(_dofmap.size() == mesh.num_cells()*_cell_dimension);
 
   // FIXME: Could we use a std::vector instead of std::map if the
   //        collapsed dof map is contiguous (0, . . . , n)?
@@ -115,9 +116,9 @@ DofMap::DofMap(std::unordered_map<std::size_t, std::size_t>& collapsed_map,
   collapsed_map.clear();
   for (std::size_t i = 0; i < mesh.num_cells(); ++i)
   {
-    const std::vector<dolfin::la_index>& view_cell_dofs
-      = dofmap_view._dofmap[i];
-    const std::vector<dolfin::la_index>& cell_dofs = _dofmap[i];
+    const ArrayView<const dolfin::la_index> view_cell_dofs
+      = dofmap_view.cell_dofs(i);
+    const ArrayView<const dolfin::la_index> cell_dofs = this->cell_dofs(i);
     dolfin_assert(view_cell_dofs.size() == cell_dofs.size());
 
     for (std::size_t j = 0; j < view_cell_dofs.size(); ++j)
@@ -125,18 +126,18 @@ DofMap::DofMap(std::unordered_map<std::size_t, std::size_t>& collapsed_map,
   }
 }
 //-----------------------------------------------------------------------------
-DofMap::DofMap(const DofMap& dofmap)
+DofMap::DofMap(const DofMap& dofmap) : _index_map(dofmap._index_map)
 {
   // Copy data
   _dofmap = dofmap._dofmap;
+  _cell_dimension = dofmap._cell_dimension;
   _ufc_dofmap = dofmap._ufc_dofmap;
-  _global_offset = dofmap._global_offset;
-  _local_ownership_size = dofmap._local_ownership_size;
+  _num_mesh_entities_global = dofmap._num_mesh_entities_global;
   _ufc_local_to_local= dofmap._ufc_local_to_local;
   _is_view = dofmap._is_view;
   _global_dimension = dofmap._global_dimension;
   _ufc_offset = dofmap._ufc_offset;
-  _off_process_owner = dofmap._off_process_owner;
+  _multimesh_offset = dofmap._multimesh_offset;
   _shared_nodes = dofmap._shared_nodes;
   _neighbours = dofmap._neighbours;
   constrained_domain = dofmap.constrained_domain;
@@ -154,32 +155,34 @@ std::size_t DofMap::global_dimension() const
 //-----------------------------------------------------------------------------
 std::size_t DofMap::local_dimension(std::string type) const
 {
+  deprecation("DofMap::local_dimension", "2016.1",
+              "Please use dofmap::index_map()->size() instead.");
+
   if (type == "owned")
-    return _local_ownership_size;
+    return _index_map->size(IndexMap::MapSize::OWNED);
   else if (type == "unowned")
-    return block_size*_local_to_global_unowned.size();
+    return _index_map->size(IndexMap::MapSize::UNOWNED);
   else if (type == "all")
-    return _local_ownership_size + block_size*_local_to_global_unowned.size();
+    return _index_map->size(IndexMap::MapSize::ALL);
   else
   {
-    dolfin_error("DofMap.h",
-                 "report DofMap local dimension",
-                 "unknown dof type given. Use either \"owned\", "
-                 "\"unowned\", or \"all\"");
-    return 0;
+    dolfin_error("DofMap.cpp",
+                 "get local dimension",
+                 "Unknown type %s", type.c_str());
   }
+
+  return 0;
 }
 //-----------------------------------------------------------------------------
-std::size_t DofMap::cell_dimension(std::size_t cell_index) const
+std::size_t DofMap::num_element_dofs(std::size_t cell_index) const
 {
-  dolfin_assert(cell_index < _dofmap.size());
-  return _dofmap[cell_index].size();
+  return _cell_dimension;
 }
 //-----------------------------------------------------------------------------
-std::size_t DofMap::max_cell_dimension() const
+std::size_t DofMap::max_element_dofs() const
 {
   dolfin_assert(_ufc_dofmap);
-  return _ufc_dofmap->local_dimension();
+  return _ufc_dofmap->num_element_dofs();
 }
 //-----------------------------------------------------------------------------
 std::size_t DofMap::num_entity_dofs(std::size_t dim) const
@@ -188,14 +191,6 @@ std::size_t DofMap::num_entity_dofs(std::size_t dim) const
   return _ufc_dofmap->num_entity_dofs(dim);
 }
 //-----------------------------------------------------------------------------
-std::size_t DofMap::geometric_dimension() const
-{
-  deprecation("DofMap::geometric_dimension()", "1.5", "1.6",
-            "DofMap::geometric_dimension() is deprecated and will be removed");
-  dolfin_assert(_ufc_dofmap);
-  return _ufc_dofmap->geometric_dimension();
-}
-//-----------------------------------------------------------------------------
 std::size_t DofMap::num_facet_dofs() const
 {
   dolfin_assert(_ufc_dofmap);
@@ -204,13 +199,7 @@ std::size_t DofMap::num_facet_dofs() const
 //-----------------------------------------------------------------------------
 std::pair<std::size_t, std::size_t> DofMap::ownership_range() const
 {
-  return std::make_pair(_global_offset,
-                        _local_ownership_size + _global_offset);
-}
-//-----------------------------------------------------------------------------
-const std::vector<int>& DofMap::off_process_owner() const
-{
-  return _off_process_owner;
+  return _index_map->local_range();
 }
 //-----------------------------------------------------------------------------
 const std::unordered_map<int, std::vector<int>>& DofMap::shared_nodes() const
@@ -243,81 +232,6 @@ void DofMap::tabulate_facet_dofs(std::vector<std::size_t>& dofs,
   _ufc_dofmap->tabulate_facet_dofs(dofs.data(), local_facet);
 }
 //-----------------------------------------------------------------------------
-void
-DofMap::tabulate_coordinates(boost::multi_array<double, 2>& coordinates,
-                             const std::vector<double>& vertex_coordinates,
-                             const Cell& cell) const
-{
-  dolfin_assert(_ufc_dofmap);
-
-  // Check dimensions
-  if (coordinates.shape()[0] != cell_dimension(cell.index()) ||
-      coordinates.shape()[1] != _ufc_dofmap->geometric_dimension())
-  {
-    boost::multi_array<double, 2>::extent_gen extents;
-    const std::size_t cell_dim = cell_dimension(cell.index());
-    coordinates.resize(extents[cell_dim][_ufc_dofmap->geometric_dimension()]);
-  }
-
-  // Tabulate coordinates
-  _ufc_dofmap->tabulate_coordinates(coordinates.data(),
-                                    vertex_coordinates.data());
-}
-//-----------------------------------------------------------------------------
-std::vector<double> DofMap::tabulate_all_coordinates(const Mesh& mesh) const
-{
-  // Geometric dimension
-  const std::size_t gdim = _ufc_dofmap->geometric_dimension();
-  dolfin_assert(gdim == mesh.geometry().dim());
-
-  if (_is_view)
-  {
-    dolfin_error("DofMap.cpp",
-                 "tabulate_all_coordinates",
-                 "Cannot tabulate coordinates for a DofMap that is a view.");
-  }
-  //const std::size_t offset = ownership_range().first;
-
-  // Number of local dofs (dofs owned by this process)
-  //const std::size_t local_size
-  //  = ownership_range().second - ownership_range().first;
-
-  // Vector to hold coordinates and return
-  std::vector<double> x(gdim*_local_ownership_size);
-
-  // Loop over cells and tabulate dofs
-  boost::multi_array<double, 2> coordinates;
-  std::vector<double> vertex_coordinates;
-  for (CellIterator cell(mesh); !cell.end(); ++cell)
-  {
-    // Update UFC cell
-    cell->get_vertex_coordinates(vertex_coordinates);
-
-    // Get local-to-global map
-    const std::vector<dolfin::la_index>& dofs = cell_dofs(cell->index());
-
-    // Tabulate dof coordinates on cell
-    tabulate_coordinates(coordinates, vertex_coordinates, *cell);
-
-    // Copy dof coordinates into vector
-    for (std::size_t i = 0; i < dofs.size(); ++i)
-    {
-      const dolfin::la_index dof = dofs[i];
-      if (dof < (dolfin::la_index) _local_ownership_size)
-      {
-        const dolfin::la_index local_index = dof;
-        for (std::size_t j = 0; j < gdim; ++j)
-        {
-          dolfin_assert(gdim*local_index + j < x.size());
-          x[gdim*local_index + j] = coordinates[i][j];
-        }
-      }
-    }
-  }
-
-  return x;
-}
-//-----------------------------------------------------------------------------
 std::shared_ptr<GenericDofMap> DofMap::copy() const
 {
   return std::shared_ptr<GenericDofMap>(new DofMap(*this));
@@ -346,22 +260,91 @@ std::shared_ptr<GenericDofMap>
                                                      *this, mesh));
 }
 //-----------------------------------------------------------------------------
+std::vector<dolfin::la_index> DofMap::dofs(const Mesh& mesh,
+                                           std::size_t dim) const
+{
+  // FIXME: This function requires a special case when dim ==
+  // mesh.topology().dim() because of they way DOLFIN handles d-d
+  // connectivity. Change DOLFIN behaviour.
+
+  // Check number of dofs per entity (on cell cell)
+  const std::size_t num_dofs_per_entity = num_entity_dofs(dim);
+
+  // Return empty vector if not dofs on requested entity
+  if (num_dofs_per_entity == 0)
+    return std::vector<dolfin::la_index>();
+
+  // Vector to hold list of dofs
+  std::vector<dolfin::la_index>
+    dof_list(mesh.num_entities(dim)*num_dofs_per_entity);
+
+  // Iterate over cells
+  if (dim < mesh.topology().dim())
+  {
+    std::vector<std::size_t> entity_dofs_local;
+    for (CellIterator c(mesh); !c.end(); ++c)
+    {
+      // Get local-to-global dofmap for cell
+      const auto cell_dof_list = cell_dofs(c->index());
+
+      // Loop over all entities of dimension dim
+      for (MeshEntityIterator e(*c, dim); !e.end(); ++e)
+      {
+        // Tabulate cell-wise index of all dofs on entity
+        const std::size_t local_index = e.pos();
+        tabulate_entity_dofs(entity_dofs_local, dim, local_index);
+
+        // Get dof index and add to list
+        for (std::size_t i = 0; i < entity_dofs_local.size(); ++i)
+        {
+          const std::size_t entity_dof_local = entity_dofs_local[i];
+          const dolfin::la_index dof_index = cell_dof_list[entity_dof_local];
+          dolfin_assert(e->index()*num_dofs_per_entity + i < dof_list.size());
+          dof_list[e->index()*num_dofs_per_entity + i] = dof_index;
+        }
+      }
+    }
+  }
+  else
+  {
+    std::vector<std::size_t> entity_dofs_local;
+    for (CellIterator c(mesh); !c.end(); ++c)
+    {
+      // Get local-to-global dofmap for cell
+      const auto cell_dof_list = cell_dofs(c->index());
+
+      // Tabulate cell-wise index of all dofs on entity
+      tabulate_entity_dofs(entity_dofs_local, dim, 0);
+
+      // Get dof index and add to list
+      for (std::size_t i = 0; i < entity_dofs_local.size(); ++i)
+      {
+        const std::size_t entity_dof_local = entity_dofs_local[i];
+        const dolfin::la_index dof_index = cell_dof_list[entity_dof_local];
+        dolfin_assert(c->index()*num_dofs_per_entity + i < dof_list.size());
+        dof_list[c->index()*num_dofs_per_entity + i] = dof_index;
+      }
+    }
+  }
+
+  return dof_list;
+}
+//-----------------------------------------------------------------------------
 std::vector<dolfin::la_index> DofMap::dofs() const
 {
   // Create vector to hold dofs
   std::vector<la_index> _dofs;
-  _dofs.reserve(_dofmap.size()*max_cell_dimension());
+  _dofs.reserve(_dofmap.size()*max_element_dofs());
+
+  const dolfin::la_index local_ownership_size
+    = _index_map->size(IndexMap::MapSize::OWNED);
+  const std::size_t global_offset = _index_map->local_range().first;
 
   // Insert all dofs into a vector (will contain duplicates)
-  std::vector<std::vector<dolfin::la_index>>::const_iterator cell_dofs;
-  for (cell_dofs = _dofmap.begin(); cell_dofs != _dofmap.end(); ++cell_dofs)
+  for (auto dof : _dofmap)
   {
-    for (std::size_t i = 0; i < cell_dofs->size(); ++i)
-    {
-      const la_index dof = (*cell_dofs)[i];
-      if (dof >= 0 && dof < _local_ownership_size)
-        _dofs.push_back(dof + _global_offset);
-    }
+    if (dof >= 0 && dof < local_ownership_size)
+      _dofs.push_back(dof + global_offset);
   }
 
   // Sort dofs (required to later remove duplicates)
@@ -375,59 +358,38 @@ std::vector<dolfin::la_index> DofMap::dofs() const
 //-----------------------------------------------------------------------------
 void DofMap::set(GenericVector& x, double value) const
 {
-  std::vector<double> _value;
-  std::vector<std::vector<dolfin::la_index>>::const_iterator cell_dofs;
-  for (cell_dofs = _dofmap.begin(); cell_dofs != _dofmap.end(); ++cell_dofs)
+  dolfin_assert(_dofmap.size() % _cell_dimension == 0);
+  const std::size_t num_cells = _dofmap.size() / _cell_dimension;
+
+  std::vector<double> _value(_cell_dimension, value);
+  for (std::size_t i = 0; i < num_cells; ++i)
   {
-    _value.resize(cell_dofs->size(), value);
-    x.set_local(_value.data(), cell_dofs->size(), cell_dofs->data());
+    const ArrayView<const la_index> dofs = cell_dofs(i);
+    x.set_local(_value.data(), dofs.size(), dofs.data());
   }
   x.apply("insert");
 }
 //-----------------------------------------------------------------------------
-void DofMap::set_x(GenericVector& x, double value, std::size_t component,
-                   const Mesh& mesh) const
-{
-  std::vector<double> x_values;
-  boost::multi_array<double, 2> coordinates;
-  std::vector<double> vertex_coordinates;
-  for (CellIterator cell(mesh); !cell.end(); ++cell)
-  {
-    // Update UFC cell
-    cell->get_vertex_coordinates(vertex_coordinates);
-
-    // Get cell local-to-global map
-    const std::vector<dolfin::la_index>& dofs = cell_dofs(cell->index());
-
-    // Tabulate dof coordinates
-    tabulate_coordinates(coordinates, vertex_coordinates, *cell);
-    dolfin_assert(coordinates.shape()[0] == dofs.size());
-    dolfin_assert(component < coordinates.shape()[1]);
-
-    // Copy coordinate (it may be possible to avoid this)
-    x_values.resize(dofs.size());
-    for (std::size_t i = 0; i < coordinates.shape()[0]; ++i)
-      x_values[i] = value*coordinates[i][component];
-
-    // Set x[component] values in vector
-    x.set_local(x_values.data(), dofs.size(), dofs.data());
-  }
-}
-//-----------------------------------------------------------------------------
 void DofMap::tabulate_local_to_global_dofs(std::vector<std::size_t>& local_to_global_map) const
 {
-  const int size = _local_ownership_size
-    + block_size*_local_to_global_unowned.size();
-  local_to_global_map.resize(size);
-  for (int i = 0; i < _local_ownership_size; ++i)
-    local_to_global_map[i] = i + _global_offset;
+  const std::size_t bs = _index_map->block_size();
+  const std::vector<std::size_t>& local_to_global_unowned
+    = _index_map->local_to_global_unowned();
+  const std::size_t local_ownership_size
+    = _index_map->size(IndexMap::MapSize::OWNED);
+  local_to_global_map.resize(_index_map->size(IndexMap::MapSize::ALL));
 
-  for (std::size_t node = 0; node < _local_to_global_unowned.size(); ++node)
+  const std::size_t global_offset = _index_map->local_range().first;
+  for (std::size_t i = 0; i < local_ownership_size; ++i)
+    local_to_global_map[i] = i + global_offset;
+
+  for (std::size_t node = 0;
+       node < _index_map->local_to_global_unowned().size(); ++node)
   {
-    for (std::size_t component = 0; component < block_size; ++component)
+    for (std::size_t component = 0; component < bs; ++component)
     {
-      local_to_global_map[block_size*node + component + _local_ownership_size]
-        =  block_size*_local_to_global_unowned[node] + component;
+      local_to_global_map[bs*node + component + local_ownership_size]
+        = bs*local_to_global_unowned[node] + component;
     }
   }
 }
@@ -455,16 +417,19 @@ std::string DofMap::str(bool verbose) const
   if (verbose)
   {
     // Cell loop
-    for (std::size_t i = 0; i < _dofmap.size(); ++i)
+    dolfin_assert(_dofmap.size()%_cell_dimension == 0);
+    const std::size_t ncells = _dofmap.size() / _cell_dimension;
+
+    for (std::size_t i = 0; i < ncells; ++i)
     {
       s << "Local cell index, cell dofmap dimension: " << i
-        << ", " << _dofmap[i].size() << std::endl;
+        << ", " << _cell_dimension << std::endl;
 
       // Local dof loop
-      for (std::size_t j = 0; j < _dofmap[i].size(); ++j)
+      for (std::size_t j = 0; j < _cell_dimension; ++j)
       {
         s <<  "  " << "Local, global dof indices: " << j
-          << ", " << _dofmap[i][j] << std::endl;
+          << ", " << _dofmap[i*_cell_dimension + j] << std::endl;
       }
     }
   }
@@ -475,16 +440,6 @@ std::string DofMap::str(bool verbose) const
 void DofMap::check_dimensional_consistency(const ufc::dofmap& dofmap,
                                             const Mesh& mesh)
 {
-  // Check geometric dimension
-  if (dofmap.geometric_dimension() != mesh.geometry().dim())
-  {
-    dolfin_error("DofMap.cpp",
-                 "create mapping of degrees of freedom",
-                 "Geometric dimension of the UFC dofmap (dim = %d) and the mesh (dim = %d) do not match",
-                 dofmap.geometric_dimension(),
-                 mesh.geometry().dim());
-  }
-
   // Check topological dimension
   if (dofmap.topological_dimension() != mesh.topology().dim())
   {
