@@ -1,4 +1,4 @@
-// Copyright (C) 2008-2014 Anders Logg, Ola Skavhaug and Garth N. Wells
+// Copyright (C) 2008-2015 Anders Logg, Ola Skavhaug and Garth N. Wells
 //
 // This file is part of DOLFIN.
 //
@@ -19,15 +19,13 @@
 // Modified by Garth N. Wells 2010-2012
 // Modified by Mikael Mortensen, 2012.
 // Modified by Joachim B Haga, 2012
-// Modified by Martin Alnaes, 2013
+// Modified by Martin Alnaes, 2013-2015
 // Modified by Chris Richardson, 2014
-//
-// First added:  2008-08-12
-// Last changed: 2013-01-08
 
 #include <cstdlib>
 #include <random>
 #include <utility>
+#include <memory>
 #include <ufc.h>
 
 #include <dolfin/common/Timer.h>
@@ -50,6 +48,7 @@
 
 using namespace dolfin;
 
+
 //-----------------------------------------------------------------------------
 void DofMapBuilder::build(DofMap& dofmap, const Mesh& mesh,
                           std::shared_ptr<const SubDomain> constrained_domain)
@@ -79,14 +78,17 @@ void DofMapBuilder::build(DofMap& dofmap, const Mesh& mesh,
   // Sanity checks on UFC dofmap
   const std::size_t D = mesh.topology().dim();
   dolfin_assert(dofmap._ufc_dofmap);
-  dolfin_assert(dofmap._ufc_dofmap->geometric_dimension()
-                == mesh.geometry().dim());
   dolfin_assert(dofmap._ufc_dofmap->topological_dimension() == D);
 
+  // Extract needs_entities as vector
+  std::vector<bool> needs_entities(D + 1);
+  for (std::size_t d = 0; d <= D; ++d)
+    needs_entities[d] = dofmap._ufc_dofmap->needs_mesh_entities(d);
+
   // For mesh entities required by UFC dofmap, compute number of
   // entities on this process
   const std::vector<std::size_t> num_mesh_entities_local
-    = compute_num_mesh_entities_local(mesh, *dofmap._ufc_dofmap);
+    = compute_num_mesh_entities_local(mesh, needs_entities);
 
   // NOTE: We test for global dofs here because the the function
   // DofMapBuilder::compute_block size cannot distinguish between
@@ -103,7 +105,6 @@ void DofMapBuilder::build(DofMap& dofmap, const Mesh& mesh,
   // is not re-ordered or if global dofs are present)
   const std::size_t bs = (global_dofs.empty() and reorder)
     ? compute_blocksize(*dofmap._ufc_dofmap) : 1;
-  dofmap.block_size = bs;
 
   // Compute a 'node' dofmap based on a UFC dofmap. Returns:
   // - node dofmap (node_dofmap)
@@ -191,17 +192,11 @@ void DofMapBuilder::build(DofMap& dofmap, const Mesh& mesh,
                                node_local_to_global0, mesh,
                                dofmap._global_dimension/bs);
 
-    // Set global offset for dofs owned by this process, and the local
-    // ownership size
-    const std::size_t node_offset_global = MPI::global_offset(mesh.mpi_comm(),
-                                                              num_owned_nodes,
-                                                              true);
-    dofmap._global_offset = bs*node_offset_global;
-    dofmap._local_ownership_size = bs*num_owned_nodes;
+    dofmap._index_map->init(num_owned_nodes, bs);
 
     // Sanity check
     dolfin_assert(MPI::sum(mesh.mpi_comm(),
-                           (std::size_t) dofmap._local_ownership_size)
+       (std::size_t) dofmap._index_map->size(IndexMap::MapSize::OWNED))
                   == dofmap._global_dimension);
 
     // Compute node re-ordering for process index locality and spatial
@@ -211,13 +206,13 @@ void DofMapBuilder::build(DofMap& dofmap, const Mesh& mesh,
     // (c) New local node index to new global node index
     // (d) Old local node index to new local node index
     std::vector<int> node_old_to_new_local;
-    compute_node_reordering(dofmap._local_to_global_unowned,
-                           dofmap._off_process_owner,
-                           node_old_to_new_local,
-                           shared_node_to_processes0,
-                           node_local_to_global0,
-                           node_graph0, node_ownership0, global_nodes0,
-                           mesh.mpi_comm());
+    dolfin_assert(dofmap._index_map);
+    compute_node_reordering(*dofmap._index_map,
+                            node_old_to_new_local,
+                            shared_node_to_processes0,
+                            node_local_to_global0,
+                            node_graph0, node_ownership0, global_nodes0,
+                            mesh.mpi_comm());
 
     // Update UFC-local-to-local map to account for re-ordering
     if (constrained_domain)
@@ -246,6 +241,11 @@ void DofMapBuilder::build(DofMap& dofmap, const Mesh& mesh,
       dofmap._shared_nodes[new_node] = it->second;
     }
 
+    // Update global_nodes for node reordering
+    dofmap._global_nodes.clear();
+    for (auto it = global_nodes0.begin(); it != global_nodes0.end(); ++it)
+      dofmap._global_nodes.insert(node_old_to_new_local[*it]);
+
     // Build dofmap from original node 'dof' map and applying the
     // 'old_to_new_local' map for the re-ordered node indices
     build_dofmap(dofmap_graph, node_graph0, node_old_to_new_local, bs);
@@ -264,12 +264,11 @@ void DofMapBuilder::build(DofMap& dofmap, const Mesh& mesh,
         dofmap._ufc_local_to_local[i] = i;
     }
 
-    dofmap._global_offset = 0;
-    dofmap._local_ownership_size = dofmap._global_dimension;
-
-    dofmap._local_to_global_unowned.clear();
-    dofmap._off_process_owner.clear();
+    dofmap._index_map->init(dofmap._global_dimension, bs);
     dofmap._shared_nodes.clear();
+
+    // Store global nodes
+    dofmap._global_nodes = global_nodes0;
   }
 
   // Clear ufc_local-to-local map if dofmap has no sub-maps
@@ -298,9 +297,17 @@ DofMapBuilder::build_sub_map_view(DofMap& sub_dofmap,
   dolfin_assert(parent_dofmap._ufc_dofmap);
   const ufc::dofmap& parent_ufc_dofmap = *parent_dofmap._ufc_dofmap;
 
+  // Topological dimension
+  const std::size_t D = mesh.topology().dim();
+
+  // Extract needs_entities as vector
+  std::vector<bool> needs_entities(D + 1);
+  for (std::size_t d = 0; d <= D; ++d)
+    needs_entities[d] = parent_ufc_dofmap.needs_mesh_entities(d);
+
   // Generate and number required mesh entities for local UFC map
   const std::vector<std::size_t> num_mesh_entities_local
-    = compute_num_mesh_entities_local(mesh, parent_ufc_dofmap);
+    = compute_num_mesh_entities_local(mesh, needs_entities);
 
   // Initialise UFC offset from parent
   std::size_t ufc_offset = parent_dofmap._ufc_offset;
@@ -322,21 +329,26 @@ DofMapBuilder::build_sub_map_view(DofMap& sub_dofmap,
 
   // Add offset to local UFC dofmap
   for (std::size_t i = 0; i < sub_dofmap_graph.size(); ++i)
+  {
     for (std::size_t j = 0; j < sub_dofmap_graph[i].size(); ++j)
+    {
       sub_dofmap_graph[i][j] += ufc_offset;
+    }
+  }
 
   // Store number of global mesh entities and set global dimension
-  sub_dofmap._num_mesh_entities_global = parent_dofmap._num_mesh_entities_global;
+  sub_dofmap._num_mesh_entities_global
+    = parent_dofmap._num_mesh_entities_global;
   dolfin_assert(!sub_dofmap._num_mesh_entities_global.empty());
   sub_dofmap._global_dimension
     = sub_dofmap._ufc_dofmap->global_dimension(sub_dofmap._num_mesh_entities_global);
 
   // Copy data from parent
-  sub_dofmap._local_to_global_unowned = parent_dofmap._local_to_global_unowned;
-  sub_dofmap._off_process_owner = parent_dofmap._off_process_owner;
+  // FIXME: Do we touch sub_dofmap.index_map() in this routine? If yes, then
+  //        this routine has incorrect constness!
+  sub_dofmap.index_map() = parent_dofmap.index_map();
   sub_dofmap._shared_nodes = parent_dofmap._shared_nodes;
   sub_dofmap._neighbours = parent_dofmap._neighbours;
-  sub_dofmap.block_size = parent_dofmap.block_size;
 
   // Store UFC local to re-ordered local if submap has any submaps
   if (sub_dofmap._ufc_dofmap->num_sub_dofmaps() > 0)
@@ -353,7 +365,7 @@ DofMapBuilder::build_sub_map_view(DofMap& sub_dofmap,
 
   // Map to re-ordered dofs
   const std::vector<int>& local_to_local = parent_dofmap._ufc_local_to_local;
-  const std::size_t bs = parent_dofmap.block_size;
+  const std::size_t bs = parent_dofmap.block_size();
   for (auto cell_map = sub_dofmap_graph.begin();
        cell_map != sub_dofmap_graph.end(); ++cell_map)
   {
@@ -363,9 +375,12 @@ DofMapBuilder::build_sub_map_view(DofMap& sub_dofmap,
       const std::size_t node = div.rem;
       const std::size_t component = div.quot;
 
-        // Get dof from UFC local-to-local map
+      // Get dof from UFC local-to-local map
       dolfin_assert(node < local_to_local.size());
-      const std::size_t current_dof = bs*local_to_local[node] + component;
+      std::size_t current_dof = bs*local_to_local[node] + component;
+
+      // Add multimesh offset
+      current_dof += parent_dofmap._multimesh_offset;
 
       // Set dof index in transformed dofmap
       *dof = current_dof;
@@ -379,7 +394,8 @@ DofMapBuilder::build_sub_map_view(DofMap& sub_dofmap,
   sub_dofmap._dofmap.clear();
   for (auto const &cell_dofs : sub_dofmap_graph)
   {
-    sub_dofmap._dofmap.insert(sub_dofmap._dofmap.end(), cell_dofs.begin(),
+    sub_dofmap._dofmap.insert(sub_dofmap._dofmap.end(),
+                              cell_dofs.begin(),
                               cell_dofs.end());
   }
 }
@@ -395,15 +411,13 @@ std::size_t DofMapBuilder::build_constrained_vertex_indices(
 
   // Get vertex sharing information (local index, [(sharing process p,
   // local index on p)])
-  const std::unordered_map<unsigned int, std::
-                             vector<std::pair<unsigned int, unsigned int>>>&
+  const std::unordered_map<unsigned int,
+                           std::vector<std::pair<unsigned int, unsigned int>>>&
     shared_vertices = DistributedMeshTools::compute_shared_entities(mesh, 0);
 
    // Mark shared vertices
   std::vector<bool> vertex_shared(mesh.num_vertices(), false);
-  std::unordered_map<unsigned int, std::vector<std::pair<unsigned int,
-      unsigned int>>>::const_iterator shared_vertex;
-  for (shared_vertex = shared_vertices.begin();
+  for (auto shared_vertex = shared_vertices.begin();
        shared_vertex != shared_vertices.end(); ++shared_vertex)
   {
     dolfin_assert(shared_vertex->first < vertex_shared.size());
@@ -518,9 +532,7 @@ std::size_t DofMapBuilder::build_constrained_vertex_indices(
   // Request master vertex index from master owner
   std::vector<std::vector<std::size_t>> master_send_buffer(MPI::size(mpi_comm));
   std::vector<std::vector<std::size_t>> local_slave_index(MPI::size(mpi_comm));
-  std::map<unsigned int,
-           std::pair<unsigned int, unsigned int>>::const_iterator master;
-  for (master = slave_to_master_vertices.begin();
+  for (auto master = slave_to_master_vertices.begin();
        master != slave_to_master_vertices.end(); ++master)
   {
     const unsigned int local_index = master->first;
@@ -588,20 +600,26 @@ void DofMapBuilder::build_local_ufc_dofmap(
   // Topological dimension
   const std::size_t D = mesh.topology().dim();
 
+  // Extract needs_entities as vector
+  std::vector<bool> needs_entities(D + 1);
+  for (std::size_t d = 0; d <= D; ++d)
+    needs_entities[d] = ufc_dofmap.needs_mesh_entities(d);
+
   // Generate and number required mesh entities (locally)
   std::vector<std::size_t> num_mesh_entities(D + 1, 0);
   for (std::size_t d = 0; d <= D; ++d)
   {
-    if (ufc_dofmap.needs_mesh_entities(d))
+    if (needs_entities[d])
     {
       mesh.init(d);
       num_mesh_entities[d] = mesh.num_entities(d);
     }
   }
 
-  // Create UFC cell and allocate memory
-  ufc::cell ufc_cell;
-  ufc_cell.entity_indices.resize(D + 1);
+  // Allocate entity indices array
+  std::vector<std::vector<std::size_t>> entity_indices(D+1);
+  for (std::size_t d = 0; d <= D; ++d)
+    entity_indices[d].resize(mesh.type().num_entities(d));
 
   // Build dofmap from ufc::dofmap
   dofmap.resize(mesh.num_cells(),
@@ -609,34 +627,15 @@ void DofMapBuilder::build_local_ufc_dofmap(
   std::vector<std::size_t> dof_holder(ufc_dofmap.num_element_dofs());
   for (CellIterator cell(mesh, "all"); !cell.end(); ++cell)
   {
-    const std::size_t cell_index = cell->index();
-    ufc_cell.orientation = -1;
-    if (!cell->mesh().cell_orientations().empty())
-    {
-      dolfin_assert(cell_index < cell->mesh().cell_orientations().size());
-      ufc_cell.orientation = cell->mesh().cell_orientations()[cell_index];
-    }
-    ufc_cell.topological_dimension = D;
-
-    for (std::size_t d = 0; d < D; d++)
-    {
-      ufc_cell.entity_indices[d].resize(cell->num_entities(d));
-      for (std::size_t i = 0; i < cell->num_entities(d); ++i)
-        ufc_cell.entity_indices[d][i] = cell->entities(d)[i];
-    }
-
-    // Cell index
-    ufc_cell.entity_indices[D].resize(1);
-    ufc_cell.entity_indices[D][0] = cell_index;
-
-    // Local cell index short-cut
-    ufc_cell.index = ufc_cell.entity_indices[D][0];
+    // Fill entity indices array
+    get_cell_entities_local(*cell, entity_indices, needs_entities);
 
     // Tabulate dofs for cell
-    ufc_dofmap.tabulate_dofs(dof_holder.data(), num_mesh_entities,
-                             ufc_cell);
+    ufc_dofmap.tabulate_dofs(dof_holder.data(),
+                             num_mesh_entities,
+                             entity_indices);
     std::copy(dof_holder.begin(), dof_holder.end(),
-              dofmap[cell_index].begin());
+              dofmap[cell->index()].begin());
   }
 }
 //-----------------------------------------------------------------------------
@@ -683,8 +682,8 @@ DofMapBuilder::compute_node_ownership(
   {
     if (shared_nodes[i] == 0)
     {
-      // Shared node - send out to matching process
-      // to determine ownership and other sharing processes
+      // Shared node - send out to matching process to determine
+      // ownership and other sharing processes
 
       // Send global index
       const std::size_t global_index = local_to_global[i];
@@ -696,13 +695,13 @@ DofMapBuilder::compute_node_ownership(
     }
   }
 
-  // Make note of current size of each send buffer
-  // i.e. the number of boundary nodes, labelled '0'
+  // Make note of current size of each send buffer i.e. the number of
+  // boundary nodes, labelled '0'
   for (unsigned int i = 0; i != num_processes; ++i)
     send_buffer[i][0] = send_buffer[i].size() - 1;
 
-  // Additionally send any ghost or ghost-shared nodes
-  // to determine sharing (but not ownership)
+  // Additionally send any ghost or ghost-shared nodes to determine
+  // sharing (but not ownership)
   for (std::size_t i = 0; i < num_nodes_local; ++i)
   {
     if (shared_nodes[i] == -3 or shared_nodes[i] == -2)
@@ -768,7 +767,8 @@ DofMapBuilder::compute_node_ownership(
     {
       std::vector<unsigned int>& gprocs = global_to_procs[*q];
       send_response[i].push_back(gprocs.size());
-      send_response[i].insert(send_response[i].end(), gprocs.begin(), gprocs.end());
+      send_response[i].insert(send_response[i].end(), gprocs.begin(),
+                              gprocs.end());
     }
 
   MPI::all_to_all(mpi_comm, send_response, recv_buffer);
@@ -900,11 +900,12 @@ void DofMapBuilder::compute_global_dofs(
                      "Global degree of freedom has dimension != 1");
       }
 
-      // Create dummy cell argument to tabulate single global dof
-      std::unique_ptr<ufc::cell> ufc_cell(new ufc::cell);
+      // Create dummy entity_indices argument to tabulate single
+      // global dof
+      std::vector<std::vector<std::size_t>> dummy_entity_indices;
       std::size_t dof_local = 0;
       ufc_dofmap->tabulate_dofs(&dof_local, num_mesh_entities_local,
-                                *ufc_cell);
+                                dummy_entity_indices);
 
       // Insert global dof index
       std::pair<std::set<std::size_t>::iterator, bool> ret
@@ -1022,8 +1023,8 @@ std::size_t DofMapBuilder::compute_blocksize(const ufc::dofmap& ufc_dofmap)
       // Assume dof map has block structure, then check
       has_block_structure = true;
 
-      // Create UFC sub-dofmaps and check that all sub dofmaps have the
-      // same number of dofs per entity
+      // Create UFC sub-dofmaps and check that all sub dofmaps have
+      // the same number of dofs per entity
       for (std::size_t i = 1; i < ufc_dofmap.num_sub_dofmaps(); ++i)
       {
         std::unique_ptr<ufc::dofmap>
@@ -1065,13 +1066,18 @@ std::shared_ptr<const ufc::dofmap> DofMapBuilder::build_ufc_node_graph(
   // Topological dimension
   const std::size_t D = mesh.topology().dim();
 
+  // Extract needs_entities as vector
+  std::vector<bool> needs_entities(D + 1);
+  for (std::size_t d = 0; d <= D; ++d)
+    needs_entities[d] = ufc_dofmap->needs_mesh_entities(d);
+
   // Generate and number required mesh entities (local & global, and
   // constrained global)
   std::vector<std::size_t> num_mesh_entities_local(D + 1, 0);
   std::vector<std::size_t> num_mesh_entities_global_unconstrained(D + 1, 0);
   for (std::size_t d = 0; d <= D; ++d)
   {
-    if (ufc_dofmap->needs_mesh_entities(d))
+    if (needs_entities[d])
     {
       mesh.init(d);
       DistributedMeshTools::number_entities(mesh, d);
@@ -1107,16 +1113,17 @@ std::shared_ptr<const ufc::dofmap> DofMapBuilder::build_ufc_node_graph(
   node_dofmap.clear();
   node_dofmap.resize(mesh.num_cells());
 
-  // Holder for UFC 64-bit dofmap integers
-  std::vector<std::size_t> ufc_nodes_global, ufc_nodes_local;
-
   // Get standard local elem2ent dimension
   const std::size_t local_dim = dofmaps[0]->num_element_dofs();
 
-  // Create UFC cell and allocate memory
-  ufc::cell ufc_cell_global, ufc_cell_local;
-  ufc_cell_global.entity_indices.resize(D + 1);
-  ufc_cell_local.entity_indices.resize(D + 1);
+  // Holder for UFC 64-bit dofmap integers
+  std::vector<std::size_t> ufc_nodes_global(local_dim);
+  std::vector<std::size_t> ufc_nodes_local(local_dim);
+
+  // Allocate entity indices array
+  std::vector<std::vector<std::size_t>> entity_indices(D+1);
+  for (std::size_t d = 0; d <= D; ++d)
+    entity_indices[d].resize(mesh.type().num_entities(d));
 
   // Resize local-to-global map
   node_local_to_global.resize(offset_local[1]);
@@ -1124,49 +1131,23 @@ std::shared_ptr<const ufc::dofmap> DofMapBuilder::build_ufc_node_graph(
   // Build dofmaps from ufc::dofmap
   for (CellIterator cell(mesh, "all"); !cell.end(); ++cell)
   {
-    // Set cell orientation
-    int cell_orientation = -1;
-    if(!cell->mesh().cell_orientations().empty())
-    {
-      dolfin_assert(cell->index() < cell->mesh().cell_orientations().size());
-     cell_orientation = cell->mesh().cell_orientations()[cell->index()];
-    }
-    ufc_cell_global.orientation = cell_orientation;
-    ufc_cell_local.orientation = cell_orientation;
-
-    // Get cell data (local)
-    for (std::size_t d = 0; d < D; ++d)
-    {
-      // Get cell local entities
-      ufc_cell_local.entity_indices[d].resize(cell->num_entities(d));
-      for (std::size_t i = 0; i < cell->num_entities(d); ++i)
-        ufc_cell_local.entity_indices[d][i] = cell->entities(d)[i];
-    }
-    ufc_cell_local.entity_indices[D].resize(1);
-    ufc_cell_local.entity_indices[D][0] = cell->index();
-    ufc_cell_local.index = cell->index();
-
-    // Get global cell entities/data (global)
-    cell->get_cell_data(ufc_cell_global);
-    cell->get_cell_topology(ufc_cell_global);
-
     // Get reference to container for cell dofs
     std::vector<la_index>& cell_nodes = node_dofmap[cell->index()];
     cell_nodes.resize(local_dim);
 
     // Tabulate standard UFC dof map for first space (local)
-    ufc_nodes_local.resize(local_dim);
+    get_cell_entities_local(*cell, entity_indices, needs_entities);
     dofmaps[0]->tabulate_dofs(ufc_nodes_local.data(),
                               num_mesh_entities_local,
-                              ufc_cell_local);
+                              entity_indices);
     std::copy(ufc_nodes_local.begin(), ufc_nodes_local.end(),
               cell_nodes.begin());
 
     // Tabulate standard UFC dof map for first space (global)
-    ufc_nodes_global.resize(local_dim);
+    get_cell_entities_global(*cell, entity_indices, needs_entities);
     dofmaps[0]->tabulate_dofs(ufc_nodes_global.data(),
                               num_mesh_entities_global_unconstrained,
-                              ufc_cell_global);
+                              entity_indices);
 
     // Build local-to-global map for nodes
     for (std::size_t i = 0; i < local_dim; ++i)
@@ -1200,6 +1181,11 @@ DofMapBuilder::build_ufc_node_graph_constrained(
   // Topological dimension
   const std::size_t D = mesh.topology().dim();
 
+  // Extract needs_entities as vector
+  std::vector<bool> needs_entities(D + 1);
+  for (std::size_t d = 0; d <= D; ++d)
+    needs_entities[d] = ufc_dofmap->needs_mesh_entities(d);
+
   // Generate and number required mesh entities (local & global, and
   // constrained global)
   std::vector<std::size_t> num_mesh_entities_local(D + 1, 0);
@@ -1207,7 +1193,7 @@ DofMapBuilder::build_ufc_node_graph_constrained(
   std::vector<bool> required_mesh_entities(D + 1, false);
   for (std::size_t d = 0; d <= D; ++d)
   {
-    if (ufc_dofmap->needs_mesh_entities(d))
+    if (needs_entities[d])
     {
       required_mesh_entities[d] = true;
       mesh.init(d);
@@ -1250,15 +1236,17 @@ DofMapBuilder::build_ufc_node_graph_constrained(
   node_dofmap.clear();
   node_dofmap.resize(mesh.num_cells());
 
-  // Holder for UFC 64-bit dofmap integers
-  std::vector<std::size_t> ufc_nodes_local, ufc_nodes_global,
-    ufc_nodes_global_constrained;
-
   // Get standard local element dimension
   const std::size_t local_dim = dofmaps[0]->num_element_dofs();
 
-  // Create UFC cell and allocate memory
-  ufc::cell ufc_cell;
+  // Holder for UFC 64-bit dofmap integers
+  std::vector<std::size_t> ufc_nodes_local(local_dim);
+  std::vector<std::size_t> ufc_nodes_global_constrained(local_dim);
+
+  // Allocate entity indices array
+  std::vector<std::vector<std::size_t>> entity_indices(D+1);
+  for (std::size_t d = 0; d <= D; ++d)
+    entity_indices[d].resize(mesh.type().num_entities(d));
 
   // Resize local-to-global map
   std::vector<std::size_t> node_local_to_global_constrained(offset_local[1]);
@@ -1272,29 +1260,19 @@ DofMapBuilder::build_ufc_node_graph_constrained(
     cell_nodes.resize(local_dim);
 
     // Tabulate standard UFC dof map for first space (local)
-    ufc_nodes_local.resize(local_dim);
-    get_cell_data_local(ufc_cell, *cell);
+    get_cell_entities_local(*cell, entity_indices, needs_entities);
     dofmaps[0]->tabulate_dofs(ufc_nodes_local.data(),
                               num_mesh_entities_local,
-                              ufc_cell);
+                              entity_indices);
     std::copy(ufc_nodes_local.begin(), ufc_nodes_local.end(),
               cell_nodes.begin());
 
-    // Tabulate standard UFC dof map for first space (global)
-    ufc_nodes_global.resize(local_dim);
-    cell->get_cell_data(ufc_cell);
-    cell->get_cell_topology(ufc_cell);
-    dofmaps[0]->tabulate_dofs(ufc_nodes_global.data(),
-                              num_mesh_entities_global_unconstrained,
-                              ufc_cell);
-
     // Tabulate standard UFC dof map for first space (global, constrained)
-    ufc_nodes_global_constrained.resize(local_dim);
-    get_cell_data_global_constrained(ufc_cell, *cell,
-                                     global_entity_indices);
+    get_cell_entities_global_constrained(*cell, entity_indices,
+                                         global_entity_indices, needs_entities);
     dofmaps[0]->tabulate_dofs(ufc_nodes_global_constrained.data(),
                               num_mesh_entities_global,
-                              ufc_cell);
+                              entity_indices);
 
     // Build local-to-global map for nodes
     for (std::size_t i = 0; i < local_dim; ++i)
@@ -1398,7 +1376,7 @@ void DofMapBuilder::compute_constrained_mesh_indices(
       else
       {
         // Get number of entities
-        std::map<unsigned int, std::set<unsigned int>> shared_entities;
+        std::map<std::int32_t, std::set<unsigned int>> shared_entities;
         const std::size_t num_entities
           = DistributedMeshTools::number_entities(mesh,
                                                   slave_to_master_mesh_entities,
@@ -1428,6 +1406,7 @@ void DofMapBuilder::compute_shared_nodes(
   std::fill(shared_nodes.begin(), shared_nodes.end(), -1);
 
   std::vector<std::size_t> facet_nodes(ufc_dofmap.num_facet_dofs());
+
   // Mark dofs associated ghost cells as ghost dofs (provisionally)
   bool has_ghost_cells = false;
   for (CellIterator c(mesh, "all"); !c.end(); ++c)
@@ -1497,8 +1476,7 @@ void DofMapBuilder::compute_shared_nodes(
 }
 //-----------------------------------------------------------------------------
 void DofMapBuilder::compute_node_reordering(
-  std::vector<std::size_t>& local_to_global_unowned,
-  std::vector<int>& off_process_owner,
+  IndexMap& index_map,
   std::vector<int>& old_to_new_local,
   const std::unordered_map<int, std::vector<int>>& node_to_sharing_processes,
   const std::vector<std::size_t>& old_local_to_global,
@@ -1517,7 +1495,9 @@ void DofMapBuilder::compute_node_reordering(
     else if (*node == -1)
       ++unowned_local_size;
     else
-      error("Invalid node ownership index.");
+      dolfin_error("DofMap.cpp",
+                   "compute node reordering",
+                   "Invalid node ownership index.");
   }
   dolfin_assert((unowned_local_size+owned_local_size) == node_ownership.size());
   dolfin_assert((unowned_local_size+owned_local_size)
@@ -1656,8 +1636,8 @@ void DofMapBuilder::compute_node_reordering(
 
   MPI::all_to_all(mpi_comm, send_buffer, recv_buffer);
 
-  local_to_global_unowned.resize(unowned_local_size);
-  off_process_owner.resize(unowned_local_size);
+  std::vector<std::size_t> local_to_global_unowned(unowned_local_size);
+  //  off_process_owner.resize(unowned_local_size);
   std::size_t off_process_node_counter = 0;
 
   for (std::size_t src = 0; src != mpi_size; ++src)
@@ -1674,7 +1654,7 @@ void DofMapBuilder::compute_node_reordering(
       const int received_old_node_index_local = it->second;
       local_to_global_unowned[off_process_node_counter]
         = received_new_node_index_global;
-      off_process_owner[off_process_node_counter] = src;
+      // off_process_owner[off_process_node_counter] = src;
 
       const int new_index_local = owned_local_size + off_process_node_counter;
       dolfin_assert(old_to_new_local[received_old_node_index_local] < 0);
@@ -1682,6 +1662,8 @@ void DofMapBuilder::compute_node_reordering(
       off_process_node_counter++;
     }
 
+  index_map.set_local_to_global(local_to_global_unowned);
+
   // Sanity check
   for (auto it : old_to_new_local)
   {
@@ -1715,68 +1697,105 @@ void DofMapBuilder::build_dofmap(
   }
 }
 //-----------------------------------------------------------------------------
-void DofMapBuilder::get_cell_data_local(ufc::cell& ufc_cell,
-                                        const Cell& cell)
+void DofMapBuilder::get_cell_entities_local(const Cell& cell,
+  std::vector<std::vector<std::size_t>>& entity_indices,
+  const std::vector<bool>& needs_mesh_entities)
 {
   const std::size_t D = cell.mesh().topology().dim();
-  ufc_cell.entity_indices.resize(D + 1);
-
-  // Set cell orientation
-  ufc_cell.orientation = cell.orientation();
-
-  // Set data
   for (std::size_t d = 0; d < D; ++d)
-  {
-    // Get cell local entities
-    ufc_cell.entity_indices[d].resize(cell.num_entities(d));
-    for (std::size_t i = 0; i < cell.num_entities(d); ++i)
-      ufc_cell.entity_indices[d][i] = cell.entities(d)[i];
-  }
-  ufc_cell.entity_indices[D].resize(1);
-  ufc_cell.entity_indices[D][0] = cell.index();
-  ufc_cell.index = cell.index();
+    if (needs_mesh_entities[d])
+      for (std::size_t i = 0; i < cell.num_entities(d); ++i)
+        entity_indices[d][i] = cell.entities(d)[i];
+  // Handle cell index separately because cell.entities(D) doesn't work.
+  if (needs_mesh_entities[D])
+    entity_indices[D][0] = cell.index();
 }
 //-----------------------------------------------------------------------------
-void DofMapBuilder::get_cell_data_global_constrained(
-  ufc::cell& ufc_cell, const Cell& cell,
-  const std::vector<std::vector<std::size_t>>& global_entity_indices)
+void DofMapBuilder::get_cell_entities_global(const Cell& cell,
+  std::vector<std::vector<std::size_t>>& entity_indices,
+  const std::vector<bool>& needs_mesh_entities)
 {
-  const std::size_t D = cell.mesh().topology().dim();
-  ufc_cell.entity_indices.resize(D + 1);
-
+  const MeshTopology& topology = cell.mesh().topology();
+  const std::size_t D = topology.dim();
   for (std::size_t d = 0; d < D; ++d)
   {
-    ufc_cell.entity_indices[d].resize(cell.num_entities(d));
-    if (!global_entity_indices[d].empty())
+    if (needs_mesh_entities[d])
     {
-      ufc_cell.entity_indices[d].resize(cell.num_entities(d));
-      for (std::size_t i = 0; i < cell.num_entities(d); ++i)
+      if (topology.have_global_indices(d)) // TODO: Check if this ever will be false in here
       {
-        ufc_cell.entity_indices[d][i]
-          = global_entity_indices[d][cell.entities(d)[i]];
+        const std::vector<std::size_t>& global_indices = topology.global_indices(d);
+        for (std::size_t i = 0; i < cell.num_entities(d); ++i)
+          entity_indices[d][i] = global_indices[cell.entities(d)[i]];
+      }
+      else
+      {
+        for (std::size_t i = 0; i < cell.num_entities(d); ++i)
+          entity_indices[d][i] = cell.entities(d)[i];
       }
     }
   }
-  ufc_cell.entity_indices[D].resize(1);
-  ufc_cell.entity_indices[D][0] = cell.index();
-  ufc_cell.index = cell.index();
+  // Handle cell index separately because cell.entities(D) doesn't work.
+  if (needs_mesh_entities[D])
+  {
+    if (topology.have_global_indices(D))
+      entity_indices[D][0] = cell.global_index();
+    else
+      entity_indices[D][0] = cell.index();
+  }
+}
+// TODO: The above and below functions are _very_ similar, can they be combined?
+//-----------------------------------------------------------------------------
+void DofMapBuilder::get_cell_entities_global_constrained(const Cell& cell,
+  std::vector<std::vector<std::size_t>>& entity_indices,
+  const std::vector<std::vector<std::size_t>>& global_entity_indices,
+  const std::vector<bool>& needs_mesh_entities)
+{
+  const std::size_t D = cell.mesh().topology().dim();
+  for (std::size_t d = 0; d < D; ++d)
+  {
+    if (needs_mesh_entities[d])
+    {
+      if (global_entity_indices[d].empty())
+      {
+        dolfin_error("DofMapBuilder.cpp",
+                     "get_cell_entities_global_constrained",
+                     "Missing global entity indices needed for cell entity tabulation.");
+      }
+      if (!global_entity_indices[d].empty()) // TODO: Can this be false? If so the entity_indices array will contain garbage
+      {
+        const std::vector<std::size_t>& global_indices = global_entity_indices[d];
+        for (std::size_t i = 0; i < cell.num_entities(d); ++i)
+          entity_indices[d][i] = global_indices[cell.entities(d)[i]];
+      }
+    }
+  }
+  // Handle cell index separately because cell.entities(D) doesn't work.
+  if (needs_mesh_entities[D])
+  {
+    if (global_entity_indices[D].empty())
+    {
+      dolfin_error("DofMapBuilder.cpp",
+                   "get_cell_entities_global_constrained",
+                   "Missing global cell index needed for cell index tabulation.");
+    }
+    //entity_indices[D][0] = cell.index(); // This was the line here before, don't understand how that didn't fail miserably.
+    entity_indices[D][0] = global_entity_indices[D][cell.index()];
+  }
 }
 //-----------------------------------------------------------------------------
-std::vector<std::size_t>
-DofMapBuilder::compute_num_mesh_entities_local(const Mesh& mesh,
-                                               const ufc::dofmap& ufc_dofmap)
+std::vector<std::size_t> DofMapBuilder::compute_num_mesh_entities_local(
+  const Mesh& mesh, const std::vector<bool>& needs_mesh_entities)
 {
   const std::size_t D = mesh.topology().dim();
   std::vector<std::size_t> num_mesh_entities_local(D + 1);
   for (std::size_t d = 0; d <= D; ++d)
   {
-    if (ufc_dofmap.needs_mesh_entities(d))
+    if (needs_mesh_entities[d])
     {
       mesh.init(d);
       num_mesh_entities_local[d] = mesh.num_entities(d);
     }
   }
-
   return num_mesh_entities_local;
 }
 //-----------------------------------------------------------------------------
