@@ -1,4 +1,4 @@
-// Copyright (C) 2008-2013 Anders Logg, Ola Skavhaug and Garth N. Wells
+// Copyright (C) 2008-2015 Anders Logg, Ola Skavhaug and Garth N. Wells
 //
 // This file is part of DOLFIN.
 //
@@ -18,15 +18,13 @@
 // Modified by Niclas Jansson 2009.
 // Modified by Garth Wells 2009-2012
 // Modified by Mikael Mortensen 2012.
-//
-// First added:  2008-08-12
-// Last changed: 2012-11-05
+// Modified by Martin Alnaes, 2015
 
 #ifndef __DOF_MAP_BUILDER_H
 #define __DOF_MAP_BUILDER_H
 
-#include <memory>
 #include <map>
+#include <memory>
 #include <set>
 #include <unordered_map>
 #include <utility>
@@ -42,6 +40,7 @@ namespace dolfin
 
   class DofMap;
   class Mesh;
+  class IndexMap;
   class SubDomain;
   class UFC;
 
@@ -52,8 +51,8 @@ namespace dolfin
 
   public:
 
-    /// Build dofmap. The constrained domain may be
-    /// a null pointer, in which case it is ignored.
+    /// Build dofmap. The constrained domain may be a null pointer, in
+    /// which case it is ignored.
     static void build(DofMap& dofmap, const Mesh& dolfin_mesh,
                       std::shared_ptr<const SubDomain> constrained_domain);
 
@@ -176,8 +175,7 @@ namespace dolfin
       const Mesh& mesh);
 
     static void compute_node_reordering(
-      std::vector<std::size_t>& local_to_global_unowned,
-      std::vector<int>& off_process_owner,
+      IndexMap& index_map,
       std::vector<int>& old_to_new_local,
       const std::unordered_map<int, std::vector<int>>& node_to_sharing_processes,
       const std::vector<std::size_t>& old_local_to_global,
@@ -186,18 +184,23 @@ namespace dolfin
       const std::set<std::size_t>& global_nodes,
       const MPI_Comm mpi_comm);
 
-    static void get_cell_data_local(ufc::cell& ufc_cell,
-                                    const Cell& cell);
+    static void get_cell_entities_local(const Cell& cell,
+      std::vector<std::vector<std::size_t>>& entity_indices,
+      const std::vector<bool>& needs_mesh_entities);
 
-    static void get_cell_data_global_constrained(
-      ufc::cell& ufc_cell, const Cell& cell,
-      const std::vector<std::vector<std::size_t>>& global_entity_indices);
+    static void get_cell_entities_global(const Cell& cell,
+      std::vector<std::vector<std::size_t>>& entity_indices,
+      const std::vector<bool>& needs_mesh_entities);
+
+    static void get_cell_entities_global_constrained(const Cell& cell,
+      std::vector<std::vector<std::size_t>>& entity_indices,
+      const std::vector<std::vector<std::size_t>>& global_entity_indices,
+      const std::vector<bool>& needs_mesh_entities);
 
     // Compute number of mesh entities for dimensions required by
     // dofmap
-    static std::vector<std::size_t>
-      compute_num_mesh_entities_local(const Mesh& mesh,
-                                      const ufc::dofmap& ufc_dofmap);
+    static std::vector<std::size_t> compute_num_mesh_entities_local(
+      const Mesh& mesh, const std::vector<bool>& needs_mesh_entities);
 
   };
 }
