@@ -20,7 +20,7 @@
 // Modified by Ola Skavhaug 2009
 //
 // First added:  2008-09-11
-// Last changed: 2013-09-19
+// Last changed: 2014-06-11
 
 #ifndef __FUNCTION_SPACE_H
 #define __FUNCTION_SPACE_H
@@ -30,7 +30,7 @@
 #include <vector>
 
 #include <memory>
-#include <boost/unordered_map.hpp>
+#include <unordered_map>
 #include <dolfin/common/Array.h>
 #include <dolfin/common/Variable.h>
 #include <dolfin/common/Hierarchical.h>
@@ -121,7 +121,7 @@ namespace dolfin
     ///         Another function space.
     bool operator== (const FunctionSpace& V) const;
 
-    /// Unequality operator
+    /// Inequality operator
     ///
     /// *Arguments*
     ///     V (_FunctionSpace_)
@@ -177,6 +177,29 @@ namespace dolfin
     ///         The subspace.
     std::shared_ptr<FunctionSpace> operator[] (std::size_t i) const;
 
+    /// Extract subspace for component
+    ///
+    /// *Arguments*
+    ///     component (std::size_t)
+    ///         Index of the subspace.
+    /// *Returns*
+    ///     _FunctionSpace_
+    ///         The subspace.
+    std::shared_ptr<FunctionSpace> sub(std::size_t component) const
+    { return extract_sub_space({component}); }
+
+    /// Extract subspace for component
+    ///
+    /// *Arguments*
+    ///     component (std::vector<std::size_t>)
+    ///         The component.
+    /// *Returns*
+    ///     _FunctionSpace_
+    ///         The subspace.
+    std::shared_ptr<FunctionSpace>
+    sub(const std::vector<std::size_t>& component) const
+    { return extract_sub_space(component); }
+
     /// Extract subspace for component
     ///
     /// *Arguments*
@@ -200,14 +223,14 @@ namespace dolfin
     /// from new to old dofs
     ///
     /// *Arguments*
-    ///     collapsed_dofs (boost::unordered_map<std::size_t, std::size_t>)
+    ///     collapsed_dofs (std::unordered_map<std::size_t, std::size_t>)
     ///         The map from new to old dofs.
     ///
     /// *Returns*
     ///     _FunctionSpace_
     ///       The new function space.
     std::shared_ptr<FunctionSpace>
-    collapse(boost::unordered_map<std::size_t, std::size_t>& collapsed_dofs) const;
+    collapse(std::unordered_map<std::size_t, std::size_t>& collapsed_dofs) const;
 
     /// Check if function space has given cell
     ///
@@ -240,6 +263,37 @@ namespace dolfin
     ///         The component (relative to superspace).
     std::vector<std::size_t> component() const;
 
+    /// Tabulate the coordinates of all dofs on this process. This
+    /// function is typically used by preconditioners that require the
+    /// spatial coordinates of dofs, for example for re-partitioning or
+    /// nullspace computations.
+    ///
+    /// *Arguments*
+    ///     mesh (_Mesh_)
+    ///         The mesh.
+    ///
+    /// *Returns*
+    ///     std::vector<double>
+    ///         The dof coordinates (x0, y0, x1, y1, . . .)
+    std::vector<double> tabulate_dof_coordinates() const;
+
+    /// Set dof entries in vector to value*x[i], where [x][i] is the
+    /// coordinate of the dof spatial coordinate. Parallel layout of
+    /// vector must be consistent with dof map range This function is
+    /// typically used to construct the null space of a matrix
+    /// operator, e.g. rigid body rotations.
+    ///
+    /// *Arguments*
+    ///     vector (_GenericVector_)
+    ///         The vector to set.
+    ///     value (double)
+    ///         The value to multiply to coordinate by.
+    ///     component (std::size_t)
+    ///         The coordinate index.
+    ///     mesh (_Mesh_)
+    ///         The mesh.
+    void set_x(GenericVector& x, double value, std::size_t component) const;
+
     /// Return informal string representation (pretty-print)
     ///
     /// *Arguments*
@@ -256,6 +310,15 @@ namespace dolfin
 
   private:
 
+    // General interpolation from any GenericFunction on any mesh
+    void interpolate_from_any(GenericVector& expansion_coefficients,
+                              const GenericFunction& v) const;
+
+    // Specialised interpolate routine when functions are related by a
+    // parent mesh
+    void interpolate_from_parent(GenericVector& expansion_coefficients,
+                                 const GenericFunction& v) const;
+
     // The mesh
     std::shared_ptr<const Mesh> _mesh;
 
@@ -265,12 +328,12 @@ namespace dolfin
     // The dofmap
     std::shared_ptr<const GenericDofMap> _dofmap;
 
-    // The component (for sub spaces)
+    // The component (for subspaces)
     std::vector<std::size_t> _component;
 
-    // Cache of sub spaces
+    // Cache of subspaces
     mutable std::map<std::vector<std::size_t>,
-      std::shared_ptr<FunctionSpace> > subspaces;
+                     std::shared_ptr<FunctionSpace> > _subspaces;
 
   };
 
