@@ -26,7 +26,6 @@
 #define BOOST_NO_HASH
 
 #include <boost/graph/adjacency_list.hpp>
-#include <boost/unordered_set.hpp>
 #include <dolfin/common/Set.h>
 
 namespace dolfin
@@ -35,7 +34,7 @@ namespace dolfin
   /// Typedefs for simple graph data structures
 
   /// Vector of unordered sets
-  typedef dolfin::Set<std::size_t> graph_set_type;
+  typedef dolfin::Set<int> graph_set_type;
   typedef std::vector<graph_set_type> Graph;
 
 }
