@@ -16,9 +16,6 @@
 // along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
 //
 // Modified by Ola Skavhaug 2009.
-//
-// First added:  2003-07-15
-// Last changed: 2013-03-11
 
 #ifndef __GENERIC_FILE_H
 #define __GENERIC_FILE_H
@@ -40,6 +37,7 @@ namespace dolfin
   template <typename T> class MeshFunction;
   template <typename T> class MeshValueCollection;
   class Parameters;
+  class Table;
 
   class GenericFile
   {
@@ -67,15 +65,16 @@ namespace dolfin
     virtual void operator>> (MeshValueCollection<double>& mesh_markers);
     virtual void operator>> (MeshValueCollection<bool>& mesh_markers);
     virtual void operator>> (Parameters& parameters);
+    virtual void operator>> (Table& table);
     virtual void operator>> (std::vector<int>& x);
     virtual void operator>> (std::vector<std::size_t>& x);
     virtual void operator>> (std::vector<double>& x);
     virtual void operator>> (std::map<std::size_t, int>& map);
     virtual void operator>> (std::map<std::size_t, std::size_t>& map);
     virtual void operator>> (std::map<std::size_t, double>& map);
-    virtual void operator>> (std::map<std::size_t, std::vector<int> >& array_map);
-    virtual void operator>> (std::map<std::size_t, std::vector<std::size_t> >& array_map);
-    virtual void operator>> (std::map<std::size_t, std::vector<double> >& array_map);
+    virtual void operator>> (std::map<std::size_t, std::vector<int>>& array_map);
+    virtual void operator>> (std::map<std::size_t, std::vector<std::size_t>>& array_map);
+    virtual void operator>> (std::map<std::size_t, std::vector<double>>& array_map);
     virtual void operator>> (Function& u);
 
     // Output
@@ -103,18 +102,21 @@ namespace dolfin
     virtual void operator<< (const std::pair<const Function*, double> u);
 
     virtual void operator<< (const Parameters& parameters);
+    virtual void operator<< (const Table& table);
     virtual void operator<< (const std::vector<int>& x);
     virtual void operator<< (const std::vector<std::size_t>& x);
     virtual void operator<< (const std::vector<double>& x);
     virtual void operator<< (const std::map<std::size_t, int>& map);
     virtual void operator<< (const std::map<std::size_t, std::size_t>& map);
     virtual void operator<< (const std::map<std::size_t, double>& map);
-    virtual void operator<< (const std::map<std::size_t, std::vector<int> >& array_map);
-    virtual void operator<< (const std::map<std::size_t, std::vector<std::size_t> >& array_map);
-    virtual void operator<< (const std::map<std::size_t, std::vector<double> >& array_map);
+    virtual void operator<< (const std::map<std::size_t, std::vector<int>>& array_map);
+    virtual void operator<< (const std::map<std::size_t,
+                             std::vector<std::size_t> >& array_map);
+    virtual void operator<< (const std::map<std::size_t,
+                             std::vector<double> >& array_map);
 
-    void read();
-    virtual void write();
+    void _read();
+    void _write(std::size_t process_number);
 
     // Return filename
     std::string name() const
