@@ -21,8 +21,6 @@
 // Last Changed: 2013-10-24
 
 #include <boost/filesystem.hpp>
-#include <boost/lexical_cast.hpp>
-
 #include <dolfin/common/MPI.h>
 #include <dolfin/log/log.h>
 #include "HDF5File.h"
@@ -71,7 +69,7 @@ hid_t HDF5Interface::open_file(MPI_Comm mpi_comm, const std::string filename,
     {
       dolfin_error("HDF5Interface.cpp",
                    "open HDF5 file",
-                   "File does not exist");
+                   "File \"%s\" does not exist", filename.c_str());
     }
 
     if (mode == "a")
@@ -110,15 +108,31 @@ void HDF5Interface::flush_file(const hid_t hdf5_file_handle)
   dolfin_assert(status != HDF5_FAIL);
 }
 //-----------------------------------------------------------------------------
+std::string HDF5Interface::get_filename(hid_t hdf5_file_handle)
+{
+  // Get length of filename
+  ssize_t length = H5Fget_name(hdf5_file_handle, NULL, 0);
+  dolfin_assert(length > 0);
+
+  // Allocate memory
+  std::vector<char> name(length +1);
+
+  // Retrive filename
+  length = H5Fget_name(hdf5_file_handle, name.data(), length + 1);
+  dolfin_assert(length > 0);
+
+  return std::string(name.begin(), name.end());
+}
+//-----------------------------------------------------------------------------
 const std::string HDF5Interface::get_attribute_type(
                   const hid_t hdf5_file_handle,
-                  const std::string dataset_name,
+                  const std::string dataset_path,
                   const std::string attribute_name)
 {
   herr_t status;
 
   // Open dataset or group by name
-  const hid_t dset_id = H5Oopen(hdf5_file_handle, dataset_name.c_str(),
+  const hid_t dset_id = H5Oopen(hdf5_file_handle, dataset_path.c_str(),
                                 H5P_DEFAULT);
   dolfin_assert(dset_id != HDF5_FAIL);
 
@@ -173,13 +187,13 @@ const std::string HDF5Interface::get_attribute_type(
 }
 //-----------------------------------------------------------------------------
 void HDF5Interface::delete_attribute(const hid_t hdf5_file_handle,
-                                     const std::string dataset_name,
+                                     const std::string dataset_path,
                                      const std::string attribute_name)
 {
   herr_t status;
 
   // Open dataset or group by name
-  const hid_t dset_id = H5Oopen(hdf5_file_handle, dataset_name.c_str(),
+  const hid_t dset_id = H5Oopen(hdf5_file_handle, dataset_path.c_str(),
                                 H5P_DEFAULT);
   dolfin_assert(dset_id != HDF5_FAIL);
 
@@ -205,15 +219,14 @@ herr_t HDF5Interface::attribute_iteration_function(hid_t loc_id,
 //-----------------------------------------------------------------------------
 const std::vector<std::string>
 HDF5Interface::list_attributes(const hid_t hdf5_file_handle,
-                               const std::string dataset_name)
+                               const std::string dataset_path)
 {
   // Open dataset or group by name
-  const hid_t dset_id = H5Oopen(hdf5_file_handle, dataset_name.c_str(),
+  const hid_t dset_id = H5Oopen(hdf5_file_handle, dataset_path.c_str(),
                                 H5P_DEFAULT);
   dolfin_assert(dset_id != HDF5_FAIL);
 
   hsize_t n = 0;
-
   std::vector<std::string> out_string;
   herr_t status = H5Aiterate2(dset_id, H5_INDEX_NAME, H5_ITER_INC, &n,
                               attribute_iteration_function,
@@ -228,14 +241,14 @@ HDF5Interface::list_attributes(const hid_t hdf5_file_handle,
 }
 //-----------------------------------------------------------------------------
 bool HDF5Interface::has_attribute(const hid_t hdf5_file_handle,
-                                  const std::string dataset_name,
+                                  const std::string dataset_path,
                                   const std::string attribute_name)
 {
   herr_t status;
   htri_t has_attr;
 
   // Open dataset or group by name
-  const hid_t dset_id = H5Oopen(hdf5_file_handle, dataset_name.c_str(),
+  const hid_t dset_id = H5Oopen(hdf5_file_handle, dataset_path.c_str(),
                                 H5P_DEFAULT);
   dolfin_assert(dset_id != HDF5_FAIL);
 
@@ -277,10 +290,10 @@ bool HDF5Interface::has_group(const hid_t hdf5_file_handle,
 }
 //-----------------------------------------------------------------------------
 bool HDF5Interface::has_dataset(const hid_t hdf5_file_handle,
-                                const std::string dataset_name)
+                                const std::string dataset_path)
 {
   hid_t lapl_id = H5Pcreate(H5P_LINK_ACCESS);
-  htri_t link_status = H5Lexists(hdf5_file_handle, dataset_name.c_str(), lapl_id);
+  htri_t link_status = H5Lexists(hdf5_file_handle, dataset_path.c_str(), lapl_id);
   dolfin_assert(link_status >= 0);
 
   // Close link access properties
@@ -323,11 +336,11 @@ void HDF5Interface::add_group(const hid_t hdf5_file_handle,
   }
 }
 //-----------------------------------------------------------------------------
-std::size_t HDF5Interface::dataset_rank(const hid_t hdf5_file_handle,
-                                        const std::string dataset_name)
+int HDF5Interface::dataset_rank(const hid_t hdf5_file_handle,
+                                const std::string dataset_path)
 {
   // Open dataset
-  const hid_t dset_id = H5Dopen2(hdf5_file_handle, dataset_name.c_str(),
+  const hid_t dset_id = H5Dopen2(hdf5_file_handle, dataset_path.c_str(),
                                  H5P_DEFAULT);
   dolfin_assert(dset_id != HDF5_FAIL);
 
@@ -348,12 +361,12 @@ std::size_t HDF5Interface::dataset_rank(const hid_t hdf5_file_handle,
   return rank;
 }
 //-----------------------------------------------------------------------------
-std::vector<std::size_t>
-HDF5Interface::get_dataset_size(const hid_t hdf5_file_handle,
-                                const std::string dataset_name)
+std::vector<std::int64_t>
+HDF5Interface::get_dataset_shape(const hid_t hdf5_file_handle,
+                                 const std::string dataset_path)
 {
   // Open named dataset
-  const hid_t dset_id = H5Dopen2(hdf5_file_handle, dataset_name.c_str(),
+  const hid_t dset_id = H5Dopen2(hdf5_file_handle, dataset_path.c_str(),
                                  H5P_DEFAULT);
   dolfin_assert(dset_id != HDF5_FAIL);
 
@@ -377,11 +390,11 @@ HDF5Interface::get_dataset_size(const hid_t hdf5_file_handle,
   status = H5Dclose(dset_id);
   dolfin_assert(status != HDF5_FAIL);
 
-  return std::vector<std::size_t>(size.begin(), size.end());
+  return std::vector<std::int64_t>(size.begin(), size.end());
 }
 //-----------------------------------------------------------------------------
-std::size_t HDF5Interface::num_datasets_in_group(const hid_t hdf5_file_handle,
-                                                 const std::string group_name)
+int HDF5Interface::num_datasets_in_group(const hid_t hdf5_file_handle,
+                                         const std::string group_name)
 {
   // Get group info by name
   H5G_info_t group_info;
