@@ -306,30 +306,30 @@ void HDF5Utility::build_local_mesh(Mesh& mesh, const LocalMeshData& mesh_data)
 
   // Create mesh for editing
   MeshEditor editor;
-  dolfin_assert(mesh_data.tdim != 0);
-  editor.open(mesh, mesh_data.cell_type, mesh_data.tdim, mesh_data.gdim);
+  dolfin_assert(mesh_data.topology.dim != 0);
+  editor.open(mesh, mesh_data.topology.cell_type, mesh_data.topology.dim, mesh_data.geometry.dim);
 
   // Iterate over vertices and add to mesh
-  editor.init_vertices_global(mesh_data.num_global_vertices,
-                              mesh_data.num_global_vertices);
-  for (std::size_t i = 0; i < mesh_data.num_global_vertices; ++i)
+  editor.init_vertices_global(mesh_data.geometry.num_global_vertices,
+                              mesh_data.geometry.num_global_vertices);
+  for (std::int64_t i = 0; i < mesh_data.geometry.num_global_vertices; ++i)
   {
-    const std::size_t index = mesh_data.vertex_indices[i];
-    const std::vector<double> coords(mesh_data.vertex_coordinates[i].begin(),
-                                     mesh_data.vertex_coordinates[i].end());
-    Point p(mesh_data.gdim, coords.data());
+    const std::size_t index = mesh_data.geometry.vertex_indices[i];
+    const std::vector<double> coords(mesh_data.geometry.vertex_coordinates[i].begin(),
+                                     mesh_data.geometry.vertex_coordinates[i].end());
+    Point p(mesh_data.geometry.dim, coords.data());
     editor.add_vertex(index, p);
   }
 
   // Iterate over cells and add to mesh
-  editor.init_cells_global(mesh_data.num_global_cells,
-                           mesh_data.num_global_cells);
+  editor.init_cells_global(mesh_data.topology.num_global_cells,
+                           mesh_data.topology.num_global_cells);
 
-  for (std::size_t i = 0; i < mesh_data.num_global_cells; ++i)
+  for (std::int64_t i = 0; i < mesh_data.topology.num_global_cells; ++i)
   {
-    const std::size_t index = mesh_data.global_cell_indices[i];
-    const std::vector<std::size_t> v(mesh_data.cell_vertices[i].begin(),
-                                     mesh_data.cell_vertices[i].end());
+    const std::size_t index = mesh_data.topology.global_cell_indices[i];
+    const std::vector<std::size_t> v(mesh_data.topology.cell_vertices[i].begin(),
+                                     mesh_data.topology.cell_vertices[i].end());
     editor.add_cell(index, v);
   }
 
