@@ -31,10 +31,10 @@
 
 #include "pugixml.hpp"
 
-#include <dolfin/common/Array.h>
 #include <dolfin/common/constants.h>
 #include <dolfin/common/MPI.h>
 #include <dolfin/common/NoDeleter.h>
+#include <dolfin/common/Timer.h>
 #include <dolfin/function/Function.h>
 #include <dolfin/la/GenericVector.h>
 #include <dolfin/log/log.h>
@@ -42,13 +42,13 @@
 #include <dolfin/mesh/LocalMeshValueCollection.h>
 #include <dolfin/mesh/Mesh.h>
 #include <dolfin/mesh/MeshPartitioning.h>
-#include <dolfin/common/Timer.h>
+#include <dolfin/parameter/GlobalParameters.h>
 #include "XMLFunctionData.h"
-#include "XMLLocalMeshSAX.h"
 #include "XMLMesh.h"
 #include "XMLMeshFunction.h"
 #include "XMLMeshValueCollection.h"
 #include "XMLParameters.h"
+#include "XMLTable.h"
 #include "XMLVector.h"
 #include "XMLFile.h"
 
@@ -75,27 +75,38 @@ XMLFile::~XMLFile()
 //-----------------------------------------------------------------------------
 void XMLFile::operator>> (Mesh& input_mesh)
 {
-  if (MPI::size(input_mesh.mpi_comm()) == 1)
+  if (MPI::rank(input_mesh.mpi_comm()) == 0)
   {
     // Create XML doc and get DOLFIN node
     pugi::xml_document xml_doc;
     load_xml_doc(xml_doc);
-    const pugi::xml_node dolfin_node = get_dolfin_xml_node(xml_doc);
+    pugi::xml_node dolfin_node = get_dolfin_xml_node(xml_doc);
 
     // Read mesh
     XMLMesh::read(input_mesh, dolfin_node);
   }
-  else
+
+  if (MPI::size(input_mesh.mpi_comm()) > 1)
   {
-    // Read local mesh data
-    Timer t("XML: readSAX");
-    LocalMeshData local_mesh_data(_mpi_comm);
-    XMLLocalMeshSAX xml_object(_mpi_comm, local_mesh_data, _filename);
-    xml_object.read();
-    t.stop();
+    // Distribute local data
+    input_mesh.domains().clear();
+    LocalMeshData local_mesh_data(input_mesh);
+
+    // FIXME: To be removed when advanced parallel XML IO is removed
+    // Add mesh domain data
+    if (MPI::rank(input_mesh.mpi_comm()) == 0)
+    {
+      // Create XML doc and get DOLFIN node
+      pugi::xml_document xml_doc;
+      load_xml_doc(xml_doc);
+      pugi::xml_node dolfin_node = get_dolfin_xml_node(xml_doc);
+      XMLMesh::read_domain_data(local_mesh_data, dolfin_node);
+    }
 
     // Partition and build mesh
-    MeshPartitioning::build_distributed_mesh(input_mesh, local_mesh_data);
+    const std::string ghost_mode = dolfin::parameters["ghost_mode"];
+    MeshPartitioning::build_distributed_mesh(input_mesh, local_mesh_data,
+                                             ghost_mode);
   }
 }
 //-----------------------------------------------------------------------------
@@ -114,19 +125,6 @@ void XMLFile::operator<< (const Mesh& output_mesh)
   save_xml_doc(doc);
 }
 //-----------------------------------------------------------------------------
-void XMLFile::operator>> (LocalMeshData& input_data)
-{
-  XMLLocalMeshSAX xml_object(_mpi_comm, input_data, _filename);
-  xml_object.read();
-}
-//-----------------------------------------------------------------------------
-void XMLFile::operator<< (const LocalMeshData& output_data)
-{
-  dolfin_error("XMLFile.cpp",
-               "write local mesh data to XML file",
-               "Not implemented");
-}
-//-----------------------------------------------------------------------------
 void XMLFile::operator>> (GenericVector& input)
 {
   pugi::xml_document xml_doc;
@@ -178,8 +176,8 @@ void XMLFile::read_vector(std::vector<double>& input,
 //-----------------------------------------------------------------------------
 void XMLFile::operator<< (const GenericVector& output)
 {
-  // Open file on process 0 for distributed objects and on all processes
-  // for local objects
+  // Open file on process 0 for distributed objects and on all
+  // processes for local objects
   if (MPI::rank(_mpi_comm) == 0)
   {
     pugi::xml_document doc;
@@ -216,6 +214,27 @@ void XMLFile::operator<< (const Parameters& output)
   }
 }
 //-----------------------------------------------------------------------------
+void XMLFile::operator>> (Table& input)
+{
+  dolfin_error("XMLFile.cpp",
+               "read table from XML file",
+               "Not implemented");
+}
+//-----------------------------------------------------------------------------
+void XMLFile::operator<< (const Table& output)
+{
+  if (MPI::size(_mpi_comm) > 1)
+    dolfin_error("XMLFile.cpp",
+                 "write table to XML file",
+                 "XMLTable is not colletive. Use separate XMLFile with "
+                 "MPI_COMM_SELF on each process or single process only");
+  pugi::xml_document doc;
+  load_xml_doc(doc);
+  pugi::xml_node node = write_dolfin(doc);
+  XMLTable::write(output, node);
+  save_xml_doc(doc);
+}
+//-----------------------------------------------------------------------------
 void XMLFile::operator>>(Function& input)
 {
   // Create XML doc and get DOLFIN node
@@ -282,7 +301,7 @@ void XMLFile::read_mesh_function(MeshFunction<T>& t,
     // Broadcast and set dimension
 
     // Build local data
-    LocalMeshValueCollection<T> local_data(mvc, dim);
+    LocalMeshValueCollection<T> local_data(_mpi_comm, mvc, dim);
 
     // Distribute MeshValueCollection
     MeshPartitioning::build_distributed_value_collection<T>(mvc, local_data,
@@ -337,7 +356,7 @@ void XMLFile::read_mesh_value_collection(MeshValueCollection<T>& t,
     }
 
     // Create local data and build value collection
-    LocalMeshValueCollection<T> local_data(tmp_collection,
+    LocalMeshValueCollection<T> local_data(_mpi_comm, tmp_collection,
                                            tmp_collection.dim());
 
     // Build mesh value collection
@@ -369,7 +388,7 @@ void XMLFile::load_xml_doc(pugi::xml_document& xml_doc) const
   const boost::filesystem::path path(_filename);
   const std::string extension = boost::filesystem::extension(path);
 
-  // FIXME: Check that file exists
+  // Check that file exists
   if (!boost::filesystem::is_regular_file(_filename))
   {
     dolfin_error("XMLFile.cpp",
@@ -377,6 +396,22 @@ void XMLFile::load_xml_doc(pugi::xml_document& xml_doc) const
                  "Unable to open file \"%s\"", _filename.c_str());
   }
 
+  // Get file size if running in parallel
+  if (dolfin::MPI::size(_mpi_comm) > 1)
+  {
+    const double size = boost::filesystem::file_size(path)/(1024.0*1024.0);
+
+    // Print warning if file size is greater than threshold
+    const std::size_t warning_size
+      = dolfin::parameters["warn_on_xml_file_size"];
+    if(size >= warning_size)
+    {
+      warning("XML file '%s' is very large. XML files are parsed in serial, \
+which is not scalable. Use XMDF/HDF5 for scalable IO in parallel",
+              path.filename().c_str());
+    }
+  }
+
   // Load xml file (unzip if necessary) into parser
   if (extension == ".gz")
   {
@@ -397,12 +432,13 @@ void XMLFile::load_xml_doc(pugi::xml_document& xml_doc) const
   else
     result = xml_doc.load_file(_filename.c_str());
 
-  // Check the XML file was opened successfully
-  if (!result)
+  // Check the XML file was opened successfully, allow empty file
+  if (!result && result.status != pugi::status_no_document_element)
   {
     dolfin_error("XMLFile.cpp",
                  "read data from XML file",
-                 "Error while parsing XML");
+                 "Error while parsing XML with status \"%s\"",
+                 result.description());
   }
 }
 //-----------------------------------------------------------------------------
@@ -448,8 +484,12 @@ XMLFile::get_dolfin_xml_node(pugi::xml_document& xml_doc) const
 //-----------------------------------------------------------------------------
 pugi::xml_node XMLFile::write_dolfin(pugi::xml_document& xml_doc)
 {
-  pugi::xml_node node = xml_doc.append_child("dolfin");
-  node.append_attribute("xmlns:dolfin") = "http://fenicsproject.org";
+  pugi::xml_node node = xml_doc.child("dolfin");
+  if (!node)
+  {
+    node = xml_doc.append_child("dolfin");
+    node.append_attribute("xmlns:dolfin") = "http://fenicsproject.org";
+  }
   return node;
 }
 //-----------------------------------------------------------------------------
