@@ -28,10 +28,13 @@
 #define __GENERIC_VECTOR_H
 
 #include <algorithm>
+#include <cstdint>
 #include <utility>
 #include <vector>
-#include <dolfin/log/log.h>
+#include <dolfin/common/ArrayView.h>
 #include <dolfin/common/types.h>
+#include <dolfin/log/log.h>
+#include "IndexMap.h"
 #include "TensorLayout.h"
 #include "GenericTensor.h"
 
@@ -51,13 +54,37 @@ namespace dolfin
     //--- Implementation of the GenericTensor interface ---
 
     /// Initialize zero tensor using sparsity pattern
+    /// FIXME: This needs to be implemented on backend side! Remove it!
     virtual void init(const TensorLayout& tensor_layout)
     {
       if (!empty())
-        error("GenericVector cannot be initialised more than once");
+      {
+        dolfin_error("GenericVector.h",
+                     "initialize vector",
+                     "Vector cannot be initialised more than once");
+      }
+
       std::vector<dolfin::la_index> ghosts;
+      std::vector<std::size_t> local_to_global(tensor_layout.index_map(0)->size(IndexMap::MapSize::ALL));
+
+      // FIXME: should just pass index_map to init()
+      for (std::size_t i = 0; i != local_to_global.size(); ++i)
+        local_to_global[i] = tensor_layout.index_map(0)->local_to_global(i);
+
+      // FIXME: temporary hack - needs passing tensor layout directly to backend
+      if (tensor_layout.is_ghosted() == TensorLayout::Ghosts::GHOSTED)
+      {
+        const std::size_t nowned
+          = tensor_layout.index_map(0)->size(IndexMap::MapSize::OWNED);
+        const std::size_t nghosts
+          = tensor_layout.index_map(0)->size(IndexMap::MapSize::UNOWNED);
+        ghosts.resize(nghosts);
+        for (std::size_t i = 0; i != nghosts; ++i)
+          ghosts[i] = local_to_global[i + nowned];
+      }
+
       init(tensor_layout.mpi_comm(), tensor_layout.local_range(0),
-           tensor_layout.local_to_global_map[0], ghosts);
+           local_to_global, ghosts);
       zero();
     }
 
@@ -70,8 +97,8 @@ namespace dolfin
     { dolfin_assert(dim == 0); return size(); }
 
     /// Return local ownership range
-    virtual std::pair<std::size_t, std::size_t>
-      local_range(std::size_t dim) const
+    virtual std::pair<std::int64_t, std::int64_t>
+    local_range(std::size_t dim) const
     { dolfin_assert(dim == 0); return local_range(); }
 
     /// Get block of values using global indices
@@ -109,24 +136,13 @@ namespace dolfin
     /// Add block of values using global indices
     virtual void
       add(const double* block,
-          const std::vector<const std::vector<dolfin::la_index>* >& rows)
-    { add(block, rows[0]->size(), rows[0]->data()); }
-
-    /// Add block of values using local indices
-    virtual void
-      add_local(const double* block,
-          const std::vector<const std::vector<dolfin::la_index>* >& rows)
-    { add_local(block, rows[0]->size(), rows[0]->data()); }
-
-    /// Add block of values using global indices
-    virtual void add(const double* block,
-                     const std::vector<std::vector<dolfin::la_index> >& rows)
+          const std::vector<ArrayView<const dolfin::la_index>>& rows)
     { add(block, rows[0].size(), rows[0].data()); }
 
     /// Add block of values using local indices
     virtual void
-      add_local(const double* block,
-                const std::vector<std::vector<dolfin::la_index> >& rows)
+    add_local(const double* block,
+              const std::vector<ArrayView<const dolfin::la_index>>& rows)
     { add_local(block, rows[0].size(), rows[0].data()); }
 
     /// Set all entries to zero and keep any sparse structure
@@ -152,6 +168,7 @@ namespace dolfin
 
     /// Initialise vector with given ownership range and with ghost
     /// values
+    /// FIXME: Reimplement using init(const TensorLayout&) and deprecate
     virtual void init(MPI_Comm comm,
                       std::pair<std::size_t, std::size_t> range,
                       const std::vector<std::size_t>& local_to_global_map,
@@ -164,7 +181,7 @@ namespace dolfin
     virtual std::size_t local_size() const = 0;
 
     /// Return local ownership range of a vector
-    virtual std::pair<std::size_t, std::size_t> local_range() const = 0;
+    virtual std::pair<std::int64_t, std::int64_t> local_range() const = 0;
 
     /// Determine whether global vector index is owned by this process
     virtual bool owns_index(std::size_t i) const = 0;
@@ -267,24 +284,6 @@ namespace dolfin
     /// Assignment operator
     virtual const GenericVector& operator= (double a) = 0;
 
-    /// Return pointer to underlying data (const version)
-    virtual const double* data() const
-    {
-      dolfin_error("GenericVector.h",
-                   "return pointer to underlying vector data",
-                   "Not implemented by current linear algebra backend");
-      return 0;
-    }
-
-    /// Return pointer to underlying data
-    virtual double* data()
-    {
-      dolfin_error("GenericVector.h",
-                   "return pointer to underlying vector data",
-                   "Not implemented by current linear algebra backend");
-      return 0;
-    }
-
     //--- Convenience functions ---
 
     /// Get value of given entry
