@@ -39,69 +39,47 @@ Parameters KrylovSolver::default_parameters()
 {
   Parameters p("krylov_solver");
 
-  p.add("relative_tolerance",      1.0e-6);
-  p.add("absolute_tolerance",      1.0e-15);
-  p.add("divergence_limit",        1.0e4);
-  p.add("maximum_iterations",      10000);
-  p.add("report",                  true);
-  p.add("monitor_convergence",     false);
-  p.add("error_on_nonconvergence", true);
-  p.add("nonzero_initial_guess",   false);
-
-  // FIXME: This should be removed, see
-  // https://bugs.launchpad.net/dolfin/+bug/988494
-  p.add("use_petsc_cusp_hack", false);
-
-  // GMRES options
-  Parameters p_gmres("gmres");
-  p_gmres.add("restart", 30);
-  p.add(p_gmres);
-
-  // General preconditioner options
-  Parameters p_pc("preconditioner");
-  p_pc.add("shift_nonzero", 0.0);
-
-  // Re-use options
-  std::set<std::string> structure_options;
-  structure_options.insert("same");
-  structure_options.insert("same_nonzero_pattern");
-  structure_options.insert("different_nonzero_pattern");
-  p_pc.add("structure", "different_nonzero_pattern", structure_options);
-
-  p_pc.add("report", false);
-
-  // ILU preconditioner options
-  Parameters p_pc_ilu("ilu");
-  p_pc_ilu.add("fill_level", 0);
-
-  // Schwartz preconditioner options
-  Parameters p_pc_schwarz("schwarz");
-  p_pc_schwarz.add("overlap", 1);
-
-  // Add sub-preconditioner options
-  p_pc.add(p_pc_ilu);
-  p_pc.add(p_pc_schwarz);
-
-  // Add preconditioner options
-  p.add(p_pc);
+  p.add<double>("relative_tolerance");
+  p.add<double>("absolute_tolerance");
+  p.add<double>("divergence_limit");
+  p.add<int>("maximum_iterations");
+  p.add<bool>("report");
+  p.add<bool>("monitor_convergence");
+  p.add<bool>("error_on_nonconvergence");
+  p.add<bool>("nonzero_initial_guess");
 
   return p;
 }
 //-----------------------------------------------------------------------------
-KrylovSolver::KrylovSolver(std::string method, std::string preconditioner)
+KrylovSolver::KrylovSolver(MPI_Comm comm, std::string method,
+                           std::string preconditioner)
 {
   // Initialize solver
-  init(method, preconditioner);
+  init(method, preconditioner, comm);
+}
+//-----------------------------------------------------------------------------
+KrylovSolver::KrylovSolver(std::string method, std::string preconditioner)
+  : KrylovSolver(MPI_COMM_WORLD, method, preconditioner)
+{
+  // Do nothing
+}
+//-----------------------------------------------------------------------------
+KrylovSolver::KrylovSolver(MPI_Comm comm,
+                           std::shared_ptr<const GenericLinearOperator> A,
+                           std::string method, std::string preconditioner)
+{
+  // Initialize solver
+  init(method, preconditioner, comm);
+
+  // Set operator
+  set_operator(A);
 }
 //-----------------------------------------------------------------------------
 KrylovSolver::KrylovSolver(std::shared_ptr<const GenericLinearOperator> A,
                            std::string method, std::string preconditioner)
+  : KrylovSolver(MPI_COMM_WORLD, A, method, preconditioner)
 {
-  // Initialize solver
-  init(method, preconditioner);
-
-  // Set operator
-  set_operator(A);
+  // Do nothing
 }
 //-----------------------------------------------------------------------------
 KrylovSolver::~KrylovSolver()
@@ -126,11 +104,6 @@ KrylovSolver::set_operators(std::shared_ptr<const GenericLinearOperator> A,
   solver->set_operators(A, P);
 }
 //-----------------------------------------------------------------------------
-void KrylovSolver::set_nullspace(const VectorSpaceBasis& nullspace)
-{
-  solver->set_nullspace(nullspace);
-}
-//-----------------------------------------------------------------------------
 std::size_t KrylovSolver::solve(GenericVector& x, const GenericVector& b)
 {
   dolfin_assert(solver);
@@ -153,15 +126,15 @@ std::size_t KrylovSolver::solve(const GenericLinearOperator& A,
   return solver->solve(A, x, b);
 }
 //-----------------------------------------------------------------------------
-void KrylovSolver::init(std::string method, std::string preconditioner)
+void KrylovSolver::init(std::string method, std::string preconditioner, MPI_Comm comm)
 {
   // Get default linear algebra factory
   DefaultFactory factory;
 
   // Get list of available methods and preconditioners
-  std::vector<std::pair<std::string, std::string> >
+  const std::map<std::string, std::string>
     methods = factory.krylov_solver_methods();
-  std::vector<std::pair<std::string, std::string> >
+  const std::map<std::string, std::string>
     preconditioners = factory.krylov_solver_preconditioners();
 
   // Check that method is available
@@ -188,7 +161,7 @@ void KrylovSolver::init(std::string method, std::string preconditioner)
   parameters = dolfin::parameters("krylov_solver");
 
   // Initialize solver
-  solver = factory.create_krylov_solver(method, preconditioner);
+  solver = factory.create_krylov_solver(comm, method, preconditioner);
   solver->parameters.update(parameters);
 }
 //-----------------------------------------------------------------------------
