@@ -23,6 +23,7 @@
 #define __LINEAR_ALGEBRA_OBJECT_H
 
 #include <memory>
+#include <dolfin/common/MPI.h>
 #include <dolfin/common/Variable.h>
 
 namespace dolfin
@@ -110,6 +111,9 @@ namespace dolfin
     virtual std::shared_ptr<LinearAlgebraObject> shared_instance()
     { return std::shared_ptr<LinearAlgebraObject>(); }
 
+    /// Return MPI communicator
+    virtual MPI_Comm mpi_comm() const = 0;
+
   };
 
   /// Cast object to its derived class, if possible (non-const version).
