@@ -1,4 +1,5 @@
-// Copyright (C) 2004-2012 Johan Hoffman, Johan Jansson and Anders Logg
+// Copyright (C) 2004-2012 Johan Hoffman, Johan Jansson, Anders Logg
+// and Garth N. Wells
 //
 // This file is part of DOLFIN.
 //
@@ -30,8 +31,9 @@
 #ifdef HAS_PETSC
 
 #include <map>
-#include <string>
 #include <memory>
+#include <string>
+
 #include <petscmat.h>
 #include <petscsys.h>
 
@@ -43,6 +45,7 @@ namespace dolfin
 
   class PETScVector;
   class TensorLayout;
+  class VectorSpaceBasis;
 
   /// This class provides a simple matrix class based on PETSc.
   /// It is a wrapper for a PETSc matrix pointer (Mat)
@@ -56,10 +59,14 @@ namespace dolfin
   {
   public:
 
-    /// Create empty matrix
-    PETScMatrix(bool use_gpu=false);
+    /// Create empty matrix (on MPI_COMM_WORLD)
+    PETScMatrix();
 
-    /// Create a wrapper around a PETSc Mat pointer
+    /// Create empty matrix
+    explicit PETScMatrix(MPI_Comm comm);
+
+    /// Create a wrapper around a PETSc Mat pointer. The Mat object
+    /// should have been created, e.g. via PETSc MatrCreate.
     explicit PETScMatrix(Mat A);
 
     /// Copy constructor
@@ -81,9 +88,12 @@ namespace dolfin
     { return PETScBaseMatrix::size(dim); }
 
     /// Return local ownership range
-    std::pair<std::size_t, std::size_t> local_range(std::size_t dim) const
+    std::pair<std::int64_t, std::int64_t> local_range(std::size_t dim) const
     { return PETScBaseMatrix::local_range(dim); }
 
+    /// Return number of non-zero entries in matrix (collective)
+    std::size_t nnz() const;
+
     /// Set all entries to zero and keep any sparse structure
     virtual void zero();
 
@@ -106,7 +116,7 @@ namespace dolfin
     /// Return copy of matrix
     virtual std::shared_ptr<GenericMatrix> copy() const;
 
-    /// Intialize vector z to be compatible with the matrix-vector product
+    /// Initialize vector z to be compatible with the matrix-vector product
     /// y = Ax. In the parallel case, both size and layout are
     /// important.
     ///
@@ -121,20 +131,33 @@ namespace dolfin
                      std::size_t m, const dolfin::la_index* rows,
                      std::size_t n, const dolfin::la_index* cols) const;
 
-    /// Set block of values
+    /// Set block of values using global indices
     virtual void set(const double* block,
                      std::size_t m, const dolfin::la_index* rows,
                      std::size_t n, const dolfin::la_index* cols);
 
-    /// Add block of values
+    /// Set block of values using local indices
+    virtual void set_local(const double* block,
+                           std::size_t m, const dolfin::la_index* rows,
+                           std::size_t n, const dolfin::la_index* cols);
+
+    /// Add block of values using global indices
     virtual void add(const double* block,
                      std::size_t m, const dolfin::la_index* rows,
                      std::size_t n, const dolfin::la_index* cols);
 
+    /// Add block of values using local indices
+    virtual void add_local(const double* block,
+                           std::size_t m, const dolfin::la_index* rows,
+                           std::size_t n, const dolfin::la_index* cols);
+
     /// Add multiple of given matrix (AXPY operation)
     virtual void axpy(double a, const GenericMatrix& A,
                       bool same_nonzero_pattern);
 
+    /// Return norm of matrix
+    double norm(std::string norm_type) const;
+
     /// Get non-zero values of given row
     virtual void getrow(std::size_t row,
                         std::vector<std::size_t>& columns,
@@ -145,18 +168,27 @@ namespace dolfin
                         const std::vector<std::size_t>& columns,
                         const std::vector<double>& values);
 
-    /// Set given rows to zero
+    /// Set given rows (global row indices) to zero
     virtual void zero(std::size_t m, const dolfin::la_index* rows);
 
-    /// Set given rows to identity matrix
+    /// Set given rows (local row indices) to zero
+    virtual void zero_local(std::size_t m, const dolfin::la_index* rows);
+
+    /// Set given rows (global row indices) to identity matrix
     virtual void ident(std::size_t m, const dolfin::la_index* rows);
 
+    /// Set given rows (local row indices) to identity matrix
+    virtual void ident_local(std::size_t m, const dolfin::la_index* rows);
+
     // Matrix-vector product, y = Ax
     virtual void mult(const GenericVector& x, GenericVector& y) const;
 
     // Matrix-vector product, y = A^T x
     virtual void transpmult(const GenericVector& x, GenericVector& y) const;
 
+    /// Get diagonal of a matrix
+    virtual void get_diagonal(GenericVector& x) const;
+
     /// Set diagonal of a matrix
     virtual void set_diagonal(const GenericVector& x);
 
@@ -177,25 +209,41 @@ namespace dolfin
     /// Return linear algebra backend factory
     virtual GenericLinearAlgebraFactory& factory() const;
 
-    //--- Special PETScFunctions ---
+    //--- Special PETSc Functions ---
 
-    /// Return norm of matrix
-    double norm(std::string norm_type) const;
+    /// Sets the prefix used by PETSc when searching the options
+    /// database
+    void set_options_prefix(std::string options_prefix);
+
+    /// Returns the prefix used by PETSc when searching the options
+    /// database
+    std::string get_options_prefix() const;
+
+    /// Call PETSc function MatSetFromOptions on the PETSc Mat object
+    void set_from_options();
 
     /// Assignment operator
     const PETScMatrix& operator= (const PETScMatrix& A);
 
+    /// Attach nullspace to matrix (typically used by Krylov solvers
+    /// when solving singular systems)
+    void set_nullspace(const VectorSpaceBasis& nullspace);
+
+    /// Attach near nullspace to matrix (used by preconditioners, such
+    /// as smoothed aggregation algerbraic multigrid)
+    void set_near_nullspace(const VectorSpaceBasis& nullspace);
+
     /// Dump matrix to PETSc binary format
     void binary_dump(std::string file_name) const;
 
   private:
 
+    // Create PETSc nullspace object
+    MatNullSpace create_petsc_nullspace(const VectorSpaceBasis& nullspace) const;
+
     // PETSc norm types
     static const std::map<std::string, NormType> norm_types;
 
-    // PETSc matrix architecture
-    const bool _use_gpu;
-
   };
 
 }
