@@ -18,7 +18,7 @@
 // Modified by Anders Logg 2011
 //
 // First added:  2010-02-25
-// Last changed: 2011-10-19
+// Last changed: 2014-07-09
 
 #ifndef __DOLFIN_PETSC_PRECONDITIONER_H
 #define __DOLFIN_PETSC_PRECONDITIONER_H
@@ -34,7 +34,6 @@
 #include <dolfin/common/Variable.h>
 #include <dolfin/la/PETScObject.h>
 #include <dolfin/parameter/Parameters.h>
-#include "GenericPreconditioner.h"
 
 namespace dolfin
 {
@@ -49,29 +48,21 @@ namespace dolfin
   /// PETScKrylovSolver and set the preconditioner type and
   /// parameters.
 
-  class PETScPreconditioner : public PETScObject,
-    public GenericPreconditioner, public Variable
+  class PETScPreconditioner : public PETScObject, public Variable
   {
   public:
 
+    static void set_type(PETScKrylovSolver& solver, std::string type);
+
     /// Create a particular preconditioner object
     explicit PETScPreconditioner(std::string type = "default");
 
     /// Destructor
     virtual ~PETScPreconditioner();
 
-    /// Set the precondtioner type and parameters
+    /// Set the preconditioner type and parameters
     virtual void set(PETScKrylovSolver& solver);
 
-    /// Set the (near) null space of the preconditioner operator
-    /// (matrix). This is required for certain preconditioner types,
-    /// e.g. smoothed aggregation multigrid
-    void set_nullspace(const VectorSpaceBasis& near_nullspace);
-
-    /// Return the PETSc null space
-    MatNullSpace near_nullspace() const
-    { return petsc_near_nullspace; }
-
     /// Set the coordinates of the operator (matrix) rows and
     /// geometric dimension d. This is can be used by required for
     /// certain preconditioners, e.g. ML. The input for this function
@@ -81,20 +72,22 @@ namespace dolfin
     // FIXME: Document
     void
       set_fieldsplit(PETScKrylovSolver& solver,
-                     const std::vector<std::vector<dolfin::la_index> >& fields,
+                     const std::vector<std::vector<dolfin::la_index>>& fields,
                      const std::vector<std::string>& split_names);
 
     /// Return informal string representation (pretty-print)
     std::string str(bool verbose) const;
 
-    /// Rerturn a list of available preconditioners
-    static std::vector<std::pair<std::string, std::string> > preconditioners();
+    /// Return a list of available preconditioners
+    static std::map<std::string, std::string> preconditioners();
 
     /// Default parameter values
-    static Parameters default_parameters();
+    //static Parameters default_parameters();
 
     friend class PETScSNESSolver;
 
+    friend class PETScTAOSolver;
+
   private:
 
     /// Named preconditioner
@@ -104,15 +97,9 @@ namespace dolfin
     static const std::map<std::string, const PCType> _methods;
 
     // Available preconditioner descriptions
-    static const std::vector<std::pair<std::string, std::string> >
+    static const std::map<std::string, std::string>
       _methods_descr;
 
-    // Near null space vectors
-    std::vector<PETScVector> _near_nullspace;
-
-    // PETSc near null space.
-    MatNullSpace petsc_near_nullspace;
-
     // Operator row coordinates
     std::vector<double> _coordinates;
 
