@@ -1,4 +1,5 @@
-// Copyright (C) 2004-2010 Johan Hoffman, Johan Jansson and Anders Logg
+// Copyright (C) 2004-2016 Johan Hoffman, Johan Jansson, Anders Logg
+// and Garth N. Wells
 //
 // This file is part of DOLFIN.
 //
@@ -20,15 +21,13 @@
 // Modified by Ola Skavhaug, 2008.
 // Modified by Martin Alnæs, 2008.
 // Modified by Fredrik Valdmanis, 2011.
-//
-// First added:  2004-01-01
-// Last changed: 2011-09-29
 
-#ifndef __PETSC_VECTOR_H
-#define __PETSC_VECTOR_H
+#ifndef __DOLFIN_PETSC_VECTOR_H
+#define __DOLFIN_PETSC_VECTOR_H
 
 #ifdef HAS_PETSC
 
+#include <cstdint>
 #include <map>
 #include <memory>
 #include <string>
@@ -38,7 +37,7 @@
 #include <petscsys.h>
 #include <petscvec.h>
 
-#include <dolfin/log/dolfin_log.h>
+#include <dolfin/log/log.h>
 #include <dolfin/common/types.h>
 #include "GenericVector.h"
 #include "PETScObject.h"
@@ -46,7 +45,7 @@
 namespace dolfin
 {
 
-  class GenericSparsityPattern;
+  class SparsityPattern;
   template<typename T> class Array;
 
   /// This class provides a simple vector class based on PETSc.
@@ -61,19 +60,24 @@ namespace dolfin
   {
   public:
 
-    /// Create empty vector
+    /// Create empty vector (on MPI_COMM_WORLD)
     PETScVector();
 
+    /// Create empty vector on an MPI communicator
+    explicit PETScVector(MPI_Comm comm);
+
     /// Create vector of size N
-    PETScVector(MPI_Comm comm, std::size_t N, bool use_gpu=false);
+    PETScVector(MPI_Comm comm, std::size_t N);
 
     /// Create vector
-    explicit PETScVector(const GenericSparsityPattern& sparsity_pattern);
+    explicit PETScVector(const SparsityPattern& sparsity_pattern);
 
     /// Copy constructor
     PETScVector(const PETScVector& x);
 
-    /// Create vector wrapper of PETSc Vec pointer
+    /// Create vector wrapper of PETSc Vec pointer. The reference
+    /// counter of the Vec will be increased, and decreased upon
+    /// destruction of this object.
     explicit PETScVector(Vec x);
 
     /// Destructor
@@ -105,7 +109,8 @@ namespace dolfin
     virtual void init(MPI_Comm comm,
                       std::pair<std::size_t, std::size_t> range);
 
-    /// Initialize vector with given ownership range and with ghost values
+    /// Initialize vector with given ownership range and with ghost
+    /// values
     virtual void init(MPI_Comm comm,
                       std::pair<std::size_t, std::size_t> range,
                       const std::vector<std::size_t>& local_to_global_map,
@@ -124,7 +129,7 @@ namespace dolfin
     virtual std::size_t local_size() const;
 
     /// Return ownership range of a vector
-    virtual std::pair<std::size_t, std::size_t> local_range() const;
+    virtual std::pair<std::int64_t, std::int64_t> local_range() const;
 
     /// Determine whether global vector index is owned by this process
     virtual bool owns_index(std::size_t i) const;
@@ -234,37 +239,41 @@ namespace dolfin
 
     //--- Special PETSc functions ---
 
+    /// Sets the prefix used by PETSc when searching the options
+    /// database
+    void set_options_prefix(std::string options_prefix);
+
+    /// Returns the prefix used by PETSc when searching the options
+    /// database
+    std::string get_options_prefix() const;
+
+    /// Call PETSc function VecSetFromOptions on the underlying Vec
+    /// object
+    void set_from_options();
+
     /// Return pointer to PETSc Vec object
     Vec vec() const;
 
     /// Assignment operator
     const PETScVector& operator= (const PETScVector& x);
 
-    friend class PETScBaseMatrix;
-    friend class PETScMatrix;
+    /// Switch underlying PETSc object. Intended for internal library
+    /// usage.
+    void reset(Vec vec);
 
   private:
 
     // Initialise PETSc vector
-    void _init(MPI_Comm comm, std::pair<std::size_t, std::size_t> range,
+    void _init(std::pair<std::size_t, std::size_t> range,
                const std::vector<std::size_t>& local_to_global_map,
                const std::vector<la_index>& ghost_indices);
 
-    // Return true if vector is distributed
-    bool distributed() const;
-
     // PETSc Vec pointer
     Vec _x;
 
-    // Global-to-local map for ghost values
-    std::unordered_map<std::size_t, std::size_t> ghost_global_to_local;
-
     // PETSc norm types
     static const std::map<std::string, NormType> norm_types;
 
-    // PETSc vector architecture
-    const bool _use_gpu;
-
   };
 
 }
