@@ -33,58 +33,107 @@
 using namespace dolfin;
 
 //-----------------------------------------------------------------------------
-SLEPcEigenSolver::SLEPcEigenSolver(const PETScMatrix& A)
-  : _matA(reference_to_no_delete_pointer(const_cast<PETScMatrix&>(A)))
+SLEPcEigenSolver::SLEPcEigenSolver(MPI_Comm comm)
 {
-  dolfin_assert(A.size(0) == A.size(1));
+  // Set up solver environment
+  EPSCreate(comm, &_eps);
 
   // Set default parameter values
   parameters = default_parameters();
-
-  // Set up solver environment
-  EPSCreate(PETSC_COMM_WORLD, &_eps);
 }
 //-----------------------------------------------------------------------------
-SLEPcEigenSolver::SLEPcEigenSolver(const PETScMatrix& A, const PETScMatrix& B)
-  : _matA(reference_to_no_delete_pointer(A)), _matB(reference_to_no_delete_pointer(B))
+SLEPcEigenSolver::SLEPcEigenSolver(EPS eps) : _eps(eps)
 {
-  dolfin_assert(A.size(0) == A.size(1));
-  dolfin_assert(B.size(0) == A.size(0));
-  dolfin_assert(B.size(1) == A.size(1));
+  PetscErrorCode ierr;
+  if (_eps)
+  {
+    // Increment reference count since we holding a pointer to it
+    ierr = PetscObjectReference((PetscObject)_eps);
+    if (ierr != 0) petsc_error(ierr, __FILE__, "PetscObjectReference");
+  }
+  else
+  {
+    dolfin_error("SLEPcEigenSolver.cpp",
+                 "intialise SLEPcEigenSolver with SLEPc EPS object",
+                 "SLEPc EPS must be initialised (EPSCreate) before wrapping");
+  }
 
   // Set default parameter values
   parameters = default_parameters();
-
-  // Set up solver environment
-  EPSCreate(PETSC_COMM_WORLD, &_eps);
 }
 //-----------------------------------------------------------------------------
 SLEPcEigenSolver::SLEPcEigenSolver(std::shared_ptr<const PETScMatrix> A)
-  : _matA(A)
+  : SLEPcEigenSolver(A,  nullptr)
 {
-  dolfin_assert(A->size(0) == A->size(1));
+  // TODO: deprecate
 
-  // Set default parameter values
-  parameters = default_parameters();
+  // Do nothing (handled by other constructor)
+}
+//-----------------------------------------------------------------------------
+SLEPcEigenSolver::SLEPcEigenSolver(MPI_Comm comm,
+                                   std::shared_ptr<const PETScMatrix> A)
+  : SLEPcEigenSolver(comm, A, nullptr)
+{
+  // TODO: deprecate
 
-  // Set up solver environment
-  EPSCreate(PETSC_COMM_WORLD, &_eps);
+  // Do nothing (handled by other constructor)
 }
 //-----------------------------------------------------------------------------
 SLEPcEigenSolver::SLEPcEigenSolver(std::shared_ptr<const PETScMatrix> A,
                                    std::shared_ptr<const PETScMatrix> B)
-  : _matA(A), _matB(B)
+  : _matA(A), _matB(B), _eps(nullptr)
 
 {
+  // TODO: deprecate
+
+  dolfin_assert(A);
   dolfin_assert(A->size(0) == A->size(1));
-  dolfin_assert(B->size(0) == A->size(0));
-  dolfin_assert(B->size(1) == A->size(1));
+  if (B)
+  {
+    dolfin_assert(B->size(0) == A->size(0));
+    dolfin_assert(B->size(1) == A->size(1));
+  }
+
+  // Set up solver environment
+  EPSCreate(A->mpi_comm(), &_eps);
+
+  // Set operators
+  dolfin_assert(_eps);
+  if (B)
+    EPSSetOperators(_eps, _matA->mat(), _matB->mat());
+  else
+    EPSSetOperators(_eps, _matA->mat(), NULL);
+
+  // Set default parameter values
+  parameters = default_parameters();
+}
+//-----------------------------------------------------------------------------
+SLEPcEigenSolver::SLEPcEigenSolver(MPI_Comm comm,
+                                   std::shared_ptr<const PETScMatrix> A,
+                                   std::shared_ptr<const PETScMatrix> B)
+  : _matA(A), _matB(B), _eps(nullptr)
+{
+  // TODO: deprecate
+
+  dolfin_assert(A);
+  dolfin_assert(A->size(0) == A->size(1));
+  if (B)
+  {
+    dolfin_assert(B->size(0) == A->size(0));
+    dolfin_assert(B->size(1) == A->size(1));
+  }
 
   // Set default parameter values
   parameters = default_parameters();
 
   // Set up solver environment
-  EPSCreate(PETSC_COMM_WORLD, &_eps);
+  EPSCreate(comm, &_eps);
+
+  // Set operators
+  if (B)
+    EPSSetOperators(_eps, _matA->mat(), _matB->mat());
+  else
+    EPSSetOperators(_eps, _matA->mat(), NULL);
 }
 //-----------------------------------------------------------------------------
 SLEPcEigenSolver::~SLEPcEigenSolver()
@@ -94,59 +143,71 @@ SLEPcEigenSolver::~SLEPcEigenSolver()
     EPSDestroy(&_eps);
 }
 //-----------------------------------------------------------------------------
+void SLEPcEigenSolver::set_operators(std::shared_ptr<const PETScMatrix> A,
+                                     std::shared_ptr<const PETScMatrix> B)
+{
+  // Set operators
+  dolfin_assert(_eps);
+  if (B)
+     EPSSetOperators(_eps, _matA->mat(), _matB->mat());
+  else
+    EPSSetOperators(_eps, _matA->mat(), NULL);
+}
+//-----------------------------------------------------------------------------
 void SLEPcEigenSolver::solve()
 {
-  solve(_matA->size(0));
+  // Get operators
+  Mat A, B;
+  dolfin_assert(_eps);
+  EPSGetOperators(_eps, &A, &B);
+
+  // Wrap operator as short-cut to get size
+  PETScMatrix A_wrapped(A);
+  solve(A_wrapped.size(0));
 }
 //-----------------------------------------------------------------------------
 void SLEPcEigenSolver::solve(std::size_t n)
 {
-  dolfin_assert(_matA);
-
-  // Associate matrix (matrices) with eigenvalue solver
-  dolfin_assert(_matA->size(0) == _matA->size(1));
-  if (_matB)
-  {
-    dolfin_assert(_matB->size(0) == _matB->size(1)
-                  && _matB->size(0) == _matA->size(0));
-    EPSSetOperators(_eps, _matA->mat(), _matB->mat());
-  }
-  else
-    EPSSetOperators(_eps, _matA->mat(), NULL);
-
   // Set number of eigenpairs to compute
+  dolfin_assert(_matA);
   dolfin_assert(n <= _matA->size(0));
+  dolfin_assert(_eps);
   EPSSetDimensions(_eps, n, PETSC_DECIDE, PETSC_DECIDE);
 
-  // Set parameters from local parameters
+  // Set parameters set on SLEPcEigenSolver object
   read_parameters();
 
-  // Set parameters from PETSc parameter database
-  std::string prefix = std::string(parameters["options_prefix"]);
-  if (prefix != "default")
-  {
-    // Make sure that the prefix has a '_' at the end if the user
-    // didn't provide it
-    char lastchar = *prefix.rbegin();
-    if (lastchar != '_')
-      prefix += "_";
-
-    EPSSetOptionsPrefix(_eps, prefix.c_str());
-  }
+  // Set any options from the PETSc database
   EPSSetFromOptions(_eps);
 
-  if (parameters["verbose"])
+  // FIXME: need to be able to turn the monitor off
+  if (parameters["verbose"].is_set())
   {
-    KSP ksp;
-    ST st;
-    EPSMonitorSet(_eps, EPSMonitorAll, NULL, NULL);
-    EPSGetST(_eps, &st);
-    STGetKSP(st, &ksp);
-    KSPMonitorSet(ksp, KSPMonitorDefault, NULL, NULL);
-    EPSView(_eps, PETSC_VIEWER_STDOUT_SELF);
+    if (parameters["verbose"])
+    {
+      #if PETSC_VERSION_MAJOR == 3 && PETSC_VERSION_MINOR <= 6 && PETSC_VERSION_RELEASE == 1
+      KSP ksp;
+      ST st;
+      EPSMonitorSet(_eps, EPSMonitorAll,
+                    PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)_eps)),
+                    NULL);
+      EPSGetST(_eps, &st);
+      STGetKSP(st, &ksp);
+      KSPMonitorSet(ksp, KSPMonitorDefault,
+                    PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)ksp)),
+                    NULL);
+      EPSView(_eps, PETSC_VIEWER_STDOUT_SELF);
+      #else
+      PetscViewerAndFormat *vf;
+      PetscViewerAndFormatCreate(PETSC_VIEWER_STDOUT_WORLD, PETSC_VIEWER_DEFAULT, &vf);
+      EPSMonitorSet(_eps,(PetscErrorCode (*)(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,
+                                             PetscReal*,PetscInt,void*))EPSMonitorAll,vf,
+                    (PetscErrorCode (*)(void**))PetscViewerAndFormatDestroy);
+      #endif
+    }
   }
 
-  // Solve
+  // Solve eigenvalue problem
   EPSSolve(_eps);
 
   // Check for convergence
@@ -156,7 +217,7 @@ void SLEPcEigenSolver::solve(std::size_t n)
     warning("Eigenvalue solver did not converge");
 
   // Report solver status
-  dolfin::la_index num_iterations = 0;
+  PetscInt num_iterations = 0;
   EPSGetIterationNumber(_eps, &num_iterations);
 
   EPSType eps_type = NULL;
@@ -187,10 +248,11 @@ void SLEPcEigenSolver::get_eigenpair(double& lr, double& lc,
 void SLEPcEigenSolver::get_eigenvalue(double& lr, double& lc,
                                       std::size_t i) const
 {
-  const dolfin::la_index ii = static_cast<dolfin::la_index>(i);
+  dolfin_assert(_eps);
+  const PetscInt ii = static_cast<PetscInt>(i);
 
   // Get number of computed values
-  dolfin::la_index num_computed_eigenvalues;
+  PetscInt num_computed_eigenvalues;
   EPSGetConverged(_eps, &num_computed_eigenvalues);
 
   if (ii < num_computed_eigenvalues)
@@ -216,10 +278,11 @@ void SLEPcEigenSolver::get_eigenpair(double& lr, double& lc,
                                      PETScVector& r, PETScVector& c,
                                      std::size_t i) const
 {
-  const dolfin::la_index ii = static_cast<dolfin::la_index>(i);
+  dolfin_assert(_eps);
+  const PetscInt ii = static_cast<PetscInt>(i);
 
   // Get number of computed eigenvectors/values
-  dolfin::la_index num_computed_eigenvalues;
+  PetscInt num_computed_eigenvalues;
   EPSGetConverged(_eps, &num_computed_eigenvalues);
 
   if (ii < num_computed_eigenvalues)
@@ -228,8 +291,7 @@ void SLEPcEigenSolver::get_eigenpair(double& lr, double& lc,
     _matA->init_vector(r, 0);
     _matA->init_vector(c, 0);
 
-    dolfin_assert(r.vec());
-    dolfin_assert(c.vec());
+    // Get eigen pairs
     EPSGetEigenpair(_eps, ii, &lr, &lc, r.vec(), c.vec());
   }
   else
@@ -242,26 +304,71 @@ void SLEPcEigenSolver::get_eigenpair(double& lr, double& lc,
 //-----------------------------------------------------------------------------
 std::size_t SLEPcEigenSolver::get_number_converged() const
 {
-  dolfin::la_index num_conv;
+  PetscInt num_conv;
+  dolfin_assert(_eps);
   EPSGetConverged(_eps, &num_conv);
   return num_conv;
 }
 //-----------------------------------------------------------------------------
 void SLEPcEigenSolver::set_deflation_space(const PETScVector& deflation_space)
 {
+  dolfin_assert(_eps);
+  dolfin_assert(deflation_space.vec());
   Vec x = deflation_space.vec();
+  dolfin_assert(_eps);
   EPSSetDeflationSpace(_eps, 1, &x);
 }
 //-----------------------------------------------------------------------------
+void SLEPcEigenSolver::set_options_prefix(std::string options_prefix)
+{
+  // Set options prefix
+  dolfin_assert(_eps);
+  PetscErrorCode ierr = EPSSetOptionsPrefix(_eps, options_prefix.c_str());
+  if (ierr != 0) petsc_error(ierr, __FILE__, "EPSSetOptionsPrefix");
+}
+//-----------------------------------------------------------------------------
+std::string SLEPcEigenSolver::get_options_prefix() const
+{
+  dolfin_assert(_eps);
+  const char* prefix = NULL;
+  PetscErrorCode ierr = EPSGetOptionsPrefix(_eps, &prefix);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "EPSGetOptionsPrefix");
+  return std::string(prefix);
+}
+//-----------------------------------------------------------------------------
 void SLEPcEigenSolver::read_parameters()
 {
-  set_problem_type(parameters["problem_type"]);
-  set_spectrum(parameters["spectrum"]);
-  set_solver(parameters["solver"]);
-  set_tolerance(parameters["tolerance"], parameters["maximum_iterations"]);
-  set_spectral_transform(parameters["spectral_transform"],
-                         parameters["spectral_shift"]);
+  if (parameters["problem_type"].is_set())
+    set_problem_type(parameters["problem_type"]);
 
+  if (parameters["spectrum"].is_set())
+    set_spectrum(parameters["spectrum"]);
+
+  if (parameters["solver"].is_set())
+    set_solver(parameters["solver"]);
+
+  if (parameters["tolerance"].is_set() or parameters["maximum_iterations"].is_set())
+  {
+    const double tol = parameters["tolerance"].is_set() ? (double)parameters["tolerance"] : PETSC_DEFAULT;
+    const int max_it  = parameters["maximum_iterations"].is_set() ? (int)parameters["maximum_iterations"] : PETSC_DEFAULT;
+
+    set_tolerance(tol, max_it);
+  }
+
+  if (parameters["spectral_transform"].is_set())
+  {
+    if (parameters["spectral_shift"].is_set())
+    {
+      set_spectral_transform(parameters["spectral_transform"],
+                             parameters["spectral_shift"]);
+    }
+    else
+    {
+      dolfin_error("SLEPcEigenSolver.cpp",
+                   "set spectral transform",
+                   "For an spectral transform, the spectral shift parameter must be set");
+    }
+  }
 }
 //-----------------------------------------------------------------------------
 void SLEPcEigenSolver::set_problem_type(std::string type)
@@ -270,6 +377,7 @@ void SLEPcEigenSolver::set_problem_type(std::string type)
   if (type == "default")
     return;
 
+  dolfin_assert(_eps);
   if (type == "hermitian")
     EPSSetProblemType(_eps, EPS_HEP);
   else if (type == "non_hermitian")
@@ -294,6 +402,7 @@ void SLEPcEigenSolver::set_spectral_transform(std::string transform,
   if (transform == "default")
     return;
 
+  dolfin_assert(_eps);
   ST st;
   EPSGetST(_eps, &st);
   if (transform == "shift-and-invert")
@@ -316,6 +425,7 @@ void SLEPcEigenSolver::set_spectrum(std::string spectrum)
     return;
 
   // Choose spectrum
+  dolfin_assert(_eps);
   if (spectrum == "largest magnitude")
     EPSSetWhichEigenpairs(_eps, EPS_LARGEST_MAGNITUDE);
   else if (spectrum == "smallest magnitude")
@@ -331,17 +441,20 @@ void SLEPcEigenSolver::set_spectrum(std::string spectrum)
   else if (spectrum == "target magnitude")
   {
     EPSSetWhichEigenpairs(_eps, EPS_TARGET_MAGNITUDE);
-    EPSSetTarget(_eps, parameters["spectral_shift"]);
+    if (parameters["spectral_shift"].is_set())
+      EPSSetTarget(_eps, parameters["spectral_shift"]);
   }
   else if (spectrum == "target real")
   {
     EPSSetWhichEigenpairs(_eps, EPS_TARGET_REAL);
-    EPSSetTarget(_eps, parameters["spectral_shift"]);
+    if (parameters["spectral_shift"].is_set())
+      EPSSetTarget(_eps, parameters["spectral_shift"]);
   }
   else if (spectrum == "target imaginary")
   {
     EPSSetWhichEigenpairs(_eps, EPS_TARGET_IMAGINARY);
-    EPSSetTarget(_eps, parameters["spectral_shift"]);
+    if (parameters["spectral_shift"].is_set())
+      EPSSetTarget(_eps, parameters["spectral_shift"]);
   }
   else
   {
@@ -360,9 +473,9 @@ void SLEPcEigenSolver::set_solver(std::string solver)
   if (solver == "default")
     return;
 
-  // Choose solver
-  // (Note that lanczos will give PETSc error unless problem_type is
-  // set to 'hermitian' or 'gen_hermitian')
+  // Choose solver (Note that lanczos will give PETSc error unless
+  // problem_type is set to 'hermitian' or 'gen_hermitian')
+  dolfin_assert(_eps);
   if (solver == "power")
     EPSSetType(_eps, EPSPOWER);
   else if (solver == "subspace")
@@ -385,15 +498,17 @@ void SLEPcEigenSolver::set_solver(std::string solver)
   }
 }
 //-----------------------------------------------------------------------------
-void SLEPcEigenSolver::set_tolerance(double tolerance, std::size_t maxiter)
+void SLEPcEigenSolver::set_tolerance(double tolerance, int maxiter)
 {
   dolfin_assert(tolerance > 0.0);
+  dolfin_assert(_eps);
   EPSSetTolerances(_eps, tolerance, maxiter);
 }
 //-----------------------------------------------------------------------------
 std::size_t SLEPcEigenSolver::get_iteration_number() const
 {
-  dolfin::la_index num_iter;
+  dolfin_assert(_eps);
+  PetscInt num_iter;
   EPSGetIterationNumber(_eps, &num_iter);
   return num_iter;
 }
