@@ -20,17 +20,20 @@
 // Modified by Jan Blechta 2013
 //
 // First added:  2006-06-05
-// Last changed: 2013-02-21
+// Last changed: 2016-06-10
 
 #include <algorithm>
+#include <array>
 
 #include <dolfin/geometry/Point.h>
-#include <dolfin/log/dolfin_log.h>
+#include <dolfin/log/log.h>
 #include "Cell.h"
 #include "CellType.h"
+#include "HexahedronCell.h"
 #include "IntervalCell.h"
 #include "MeshFunction.h"
 #include "PointCell.h"
+#include "QuadrilateralCell.h"
 #include "TetrahedronCell.h"
 #include "TriangleCell.h"
 #include "Vertex.h"
@@ -39,7 +42,6 @@ using namespace dolfin;
 
 namespace dolfin
 {
-
   // Comparison operator for sorting based on global indices
   class GlobalSort
   {
@@ -54,7 +56,6 @@ namespace dolfin
     const std::vector<std::size_t>& g;
 
   };
-
 }
 
 //-----------------------------------------------------------------------------
@@ -81,6 +82,10 @@ CellType* CellType::create(Type type)
     return new TriangleCell();
   case tetrahedron:
     return new TetrahedronCell();
+  case quadrilateral:
+    return new QuadrilateralCell();
+  case hexahedron:
+    return new HexahedronCell();
   default:
     dolfin_error("CellType.cpp",
                  "create cell type",
@@ -103,6 +108,10 @@ CellType::Type CellType::string2type(std::string type)
     return triangle;
   else if (type == "tetrahedron")
     return tetrahedron;
+  else if (type == "quadrilateral")
+    return quadrilateral;
+  else if (type == "hexahedron")
+    return hexahedron;
   else
   {
     dolfin_error("CellType.cpp",
@@ -122,9 +131,13 @@ std::string CellType::type2string(Type type)
   case interval:
     return "interval";
   case triangle:
-    return "triangle";
+   return "triangle";
   case tetrahedron:
     return "tetrahedron";
+  case quadrilateral:
+    return "quadrilateral";
+  case hexahedron:
+    return "hexahedron";
   default:
     dolfin_error("CellType.cpp",
                  "convert cell type to string",
@@ -134,6 +147,101 @@ std::string CellType::type2string(Type type)
   return "";
 }
 //-----------------------------------------------------------------------------
+CellType::Type CellType::entity_type(std::size_t i) const
+{
+  if (i == dim())
+    return _cell_type;
+  else if (i == dim() - 1)
+    return _facet_type;
+  else if (i == 1)
+    return Type::interval;
+ return Type::point;
+}
+//-----------------------------------------------------------------------------
+std::size_t CellType::orientation(const Cell& cell, const Point& up) const
+{
+  Point n = cell.cell_normal();
+  return (n.dot(up) < 0.0 ? 1 : 0);
+}
+//-----------------------------------------------------------------------------
+double CellType::h(const MeshEntity& entity) const
+{
+  // Get mesh geometry
+  const MeshGeometry& geometry = entity.mesh().geometry();
+
+  // Get number of cell vertices
+  const int num_vertices = entity.num_entities(0);
+
+  // Get the coordinates (Points) of the vertices
+  const unsigned int* vertices = entity.entities(0);
+  dolfin_assert(vertices);
+  std::array<Point, 4> points;
+  dolfin_assert(num_vertices <= 4);
+  for (int i = 0; i < num_vertices; ++i)
+    points[i] = geometry.point(vertices[i]);
+
+  // Get maximum edge length
+  double h = 0.0;
+  for (int i = 0; i < num_vertices; ++i)
+  {
+    for (int j = i + 1; j < num_vertices; ++j)
+      h = std::max(h, points[i].distance(points[j]));
+  }
+
+  return h;
+}
+//-----------------------------------------------------------------------------
+double CellType::diameter(const MeshEntity& entity) const
+{
+  deprecation("CellType::diameter()", "2016.1",
+              "Use CellType::circumradius() or CellType::h() instead.");
+
+  return 2.0*circumradius(entity);
+}
+//-----------------------------------------------------------------------------
+double CellType::inradius(const Cell& cell) const
+{
+  // Check cell type
+  if (_cell_type != interval && _cell_type != triangle
+      && _cell_type != tetrahedron)
+  {
+    dolfin_error("Cell.h",
+                 "compute cell inradius",
+                 "formula not implemented for non-simplicial cells");
+  }
+
+  // Pick dim
+  const size_t d = dim();
+
+  // Compute volume
+  const double V = volume(cell);
+
+  // Handle degenerate case
+  if (V == 0.0)
+    return 0.0;
+
+  // Compute total area of facets
+  double A = 0;
+  for (std::size_t i = 0; i <= d; i++)
+    A += facet_area(cell, i);
+
+  // See Jonathan Richard Shewchuk: What Is a Good Linear Finite
+  // Element?, online:
+  // http://www.cs.berkeley.edu/~jrs/papers/elemj.pdf
+  return d*V/A;
+}
+//-----------------------------------------------------------------------------
+double CellType::radius_ratio(const Cell& cell) const
+{
+  const double r = inradius(cell);
+
+  // Handle degenerate case
+  if (r == 0.0)
+    return 0.0;
+  else
+    return dim()*r/circumradius(cell);
+}
+//-----------------------------------------------------------------------------
 bool CellType::ordered(const Cell& cell, const std::vector<std::size_t>&
                        local_to_global_vertex_indices) const
 {
@@ -286,50 +394,3 @@ bool CellType::increasing(std::size_t n0, const unsigned int* v0,
   return true;
 }
 //-----------------------------------------------------------------------------
-std::size_t CellType::orientation(const Cell& cell, const Point& up) const
-{
-  Point n = cell.cell_normal();
-  return (n.dot(up) < 0.0 ? 1 : 0);
-}
-//-----------------------------------------------------------------------------
-double CellType::inradius(const Cell& cell) const
-{
-  // Check cell type
-  if (_cell_type != interval && _cell_type != triangle
-      && _cell_type != tetrahedron)
-  {
-    dolfin_error("Cell.h",
-                 "compute cell inradius",
-                 "formula not implemented for non-simplicial cells");
-  }
-
-  // Pick dim
-  const size_t d = dim();
-
-  // Compute volume
-  const double V = volume(cell);
-
-  // Handle degenerate case
-  if (V == 0.0) {return 0.0;}
-
-  // Compute total area of facets
-  double A = 0;
-  for (std::size_t i = 0; i <= d; i++)
-    A += facet_area(cell, i);
-
-  // See Jonathan Richard Shewchuk: What Is a Good Linear Finite Element?,
-  // online: http://www.cs.berkeley.edu/~jrs/papers/elemj.pdf
-  return d*V/A;
-}
-//-----------------------------------------------------------------------------
-double CellType::radius_ratio(const Cell& cell) const
-{
-  const double r = inradius(cell);
-
-  // Handle degenerate case
-  if (r == 0.0)
-    return 0.0;
-  else
-    return 2.0*dim()*r/diameter(cell);
-}
-//-----------------------------------------------------------------------------
