@@ -110,14 +110,9 @@ namespace dolfin
   {
   public:
 
-    EdgeFunction(const Mesh& mesh) : MeshFunction<T>(mesh, 1) {}
-
     EdgeFunction(std::shared_ptr<const Mesh> mesh)
       : MeshFunction<T>(mesh, 1) {}
 
-    EdgeFunction(const Mesh& mesh, const T& value)
-      : MeshFunction<T>(mesh, 1, value) {}
-
     EdgeFunction(std::shared_ptr<const Mesh> mesh, const T& value)
       : MeshFunction<T>(mesh, 1, value) {}
 
