@@ -17,16 +17,13 @@
 //
 // Modified by Johan Hoffman 2007
 // Modified by Magnus Vikstrøm 2007
-// Modified by Garth N. Wells 2007-2013
+// Modified by Garth N. Wells 2007-2015
 // Modified by Niclas Jansson 2008
 // Modified by Kristoffer Selim 2008
 // Modified by Andre Massing 2009-2010
 // Modified by Marie E. Rognes 2012
 // Modified by Mikael Mortensen 2012
 // Modified by Jan Blechta 2013
-//
-// First added:  2006-05-08
-// Last changed: 2014-08-11
 
 #ifndef __MESH_H
 #define __MESH_H
@@ -380,9 +377,6 @@ namespace dolfin
     /// Compute all entities and connectivity.
     void init() const;
 
-    /// Clear all mesh data.
-    void clear();
-
     /// Clean out all auxiliary topology data. This clears all
     /// topological data, except the connectivity between cells and
     /// vertices.
@@ -438,38 +432,6 @@ namespace dolfin
     ///         The point around which to rotate the mesh.
     void rotate(double angle, std::size_t axis, const Point& point);
 
-    /// Move coordinates of mesh according to new boundary coordinates.
-    ///
-    /// *Arguments*
-    ///     boundary (_BoundaryMesh_)
-    ///         A mesh containing just the boundary cells.
-    ///
-    /// *Returns*
-    ///     MeshDisplacement
-    ///         Displacement encapsulated in Expression subclass
-    ///         MeshDisplacement.
-    std::shared_ptr<MeshDisplacement> move(BoundaryMesh& boundary);
-
-    /// Move coordinates of mesh according to adjacent mesh with
-    /// common global vertices.
-    ///
-    /// *Arguments*
-    ///     mesh (_Mesh_)
-    ///         A _Mesh_ object.
-    ///
-    /// *Returns*
-    ///     MeshDisplacement
-    ///         Displacement encapsulated in Expression subclass
-    ///         MeshDisplacement.
-    std::shared_ptr<MeshDisplacement> move(Mesh& mesh);
-
-    /// Move coordinates of mesh according to displacement function.
-    ///
-    /// *Arguments*
-    ///     displacement (_GenericFunction_)
-    ///         A _GenericFunction_ object.
-    void move(const GenericFunction& displacement);
-
     /// Smooth internal vertices of mesh by local averaging.
     ///
     /// *Arguments*
@@ -533,15 +495,15 @@ namespace dolfin
     ///     std::vector<std::size_t>
     ///         The colors as a mesh function over entities of the mesh.
     const std::vector<std::size_t>&
-      color(std::vector<std::size_t> coloring_type) const;
+    color(std::vector<std::size_t> coloring_type) const;
 
-    /// Compute minimum cell diameter.
+    /// Compute minimum cell size in mesh, measured greatest distance
+    /// between any two vertices of a cell.
     ///
     /// *Returns*
     ///     double
-    ///         The minimum cell diameter, the diameter is computed as
-    ///         two times the circumradius
-    ///         (http://mathworld.wolfram.com).
+    ///         The minimum cell size. The size is computed using
+    ///         Cell::h()
     ///
     /// *Example*
     ///     .. note::
@@ -549,13 +511,13 @@ namespace dolfin
     ///         No example code available for this function.
     double hmin() const;
 
-    /// Compute maximum cell diameter.
+    /// Compute maximum cell size in mesh, measured greatest distance
+    /// between any two vertices of a cell.
     ///
     /// *Returns*
     ///     double
-    ///         The maximum cell diameter, the diameter is computed as
-    ///         two times the circumradius
-    ///         (http://mathworld.wolfram.com).
+    ///         The maximum cell size. The size is computed using
+    ///         Cell::h()
     ///
     /// *Example*
     ///     .. note::
@@ -634,13 +596,14 @@ namespace dolfin
     MPI_Comm mpi_comm() const
     { return _mpi_comm; }
 
+    // Friend in fem_utils.h
+    friend Mesh create_mesh(Function&);
+
   private:
 
     // Friends
     friend class MeshEditor;
     friend class TopologyComputation;
-    friend class MeshOrdering;
-    friend class BinaryFile;
 
     // Mesh topology
     MeshTopology _topology;
