@@ -96,11 +96,16 @@ namespace dolfin
       _entity.init(mesh, dim, 0);
       mesh.init(dim);
 
-      pos_end = mesh.topology().size(dim); 
+      pos_end = mesh.topology().size(dim);
       if (opt == "regular")
         pos_end = mesh.topology().ghost_offset(dim);
       else if (opt == "ghost")
         _pos = mesh.topology().ghost_offset(dim);
+      else if (opt != "all")
+        dolfin_error("MeshEntityIterator.h",
+                     "initialize MeshEntityIterator",
+                     "unknown opt=\"%s\", choose from "
+                     "opt=[\"regular\", \"ghost\", \"all\"]", opt.c_str());
     }
 
     /// Create iterator for entities of given dimension connected to
