@@ -27,9 +27,8 @@
 #include <map>
 #include <vector>
 
-#include <boost/scoped_array.hpp>
 #include <memory>
-#include <boost/unordered_set.hpp>
+#include <unordered_set>
 #include <dolfin/common/Hierarchical.h>
 #include <dolfin/common/MPI.h>
 #include <dolfin/common/NoDeleter.h>
@@ -48,14 +47,14 @@ namespace dolfin
   class MeshEntity;
 
   /// A MeshFunction is a function that can be evaluated at a set of
-  /// mesh entities. A MeshFunction is discrete and is only defined
-  /// at the set of mesh entities of a fixed topological dimension.
-  /// A MeshFunction may for example be used to store a global
-  /// numbering scheme for the entities of a (parallel) mesh, marking
-  /// sub domains or boolean markers for mesh refinement.
+  /// mesh entities. A MeshFunction is discrete and is only defined at
+  /// the set of mesh entities of a fixed topological dimension.  A
+  /// MeshFunction may for example be used to store a global numbering
+  /// scheme for the entities of a (parallel) mesh, marking sub
+  /// domains or boolean markers for mesh refinement.
 
   template <typename T> class MeshFunction : public Variable,
-    public Hierarchical<MeshFunction<T> >
+    public Hierarchical<MeshFunction<T>>
   {
   public:
 
@@ -67,13 +66,6 @@ namespace dolfin
     /// *Arguments*
     ///     mesh (_Mesh_)
     ///         The mesh to create mesh function on.
-    explicit MeshFunction(const Mesh& mesh);
-
-    /// Create empty mesh function on given mesh (shared_ptr version)
-    ///
-    /// *Arguments*
-    ///     mesh (_Mesh_)
-    ///         The mesh to create mesh function on.
     explicit MeshFunction(std::shared_ptr<const Mesh> mesh);
 
     /// Create mesh function of given dimension on given mesh
@@ -83,16 +75,6 @@ namespace dolfin
     ///         The mesh to create mesh function on.
     ///     dim (std::size_t)
     ///         The mesh entity dimension for the mesh function.
-    MeshFunction(const Mesh& mesh, std::size_t dim);
-
-    /// Create mesh function of given dimension on given mesh
-    /// (shared_ptr version)
-    ///
-    /// *Arguments*
-    ///     mesh (_Mesh_)
-    ///         The mesh to create mesh function on.
-    ///     dim (std::size_t)
-    ///         The mesh entity dimension for the mesh function.
     MeshFunction(std::shared_ptr<const Mesh> mesh, std::size_t dim);
 
     /// Create mesh of given dimension on given mesh and initialize
@@ -105,30 +87,9 @@ namespace dolfin
     ///         The mesh entity dimension.
     ///     value (T)
     ///         The value.
-    MeshFunction(const Mesh& mesh, std::size_t dim, const T& value);
-
-    /// Create mesh of given dimension on given mesh and initialize
-    /// to a value (shared_ptr version)
-    ///
-    /// *Arguments*
-    ///     mesh (_Mesh_)
-    ///         The mesh to create mesh function on.
-    ///     dim (std::size_t)
-    ///         The mesh entity dimension.
-    ///     value (T)
-    ///         The value.
     MeshFunction(std::shared_ptr<const Mesh> mesh, std::size_t dim,
                  const T& value);
 
-    /// Create function from data file
-    ///
-    /// *Arguments*
-    ///     mesh (_Mesh_)
-    ///         The mesh to create mesh function on.
-    ///     filename (std::string)
-    ///         The filename to create mesh function from.
-    MeshFunction(const Mesh& mesh, const std::string filename);
-
     /// Create function from data file (shared_ptr version)
     ///
     /// *Arguments*
@@ -139,16 +100,6 @@ namespace dolfin
     MeshFunction(std::shared_ptr<const Mesh> mesh,
                  const std::string filename);
 
-    /// Create function from a MeshValueCollecion
-    ///
-    /// *Arguments*
-    ///     mesh (_Mesh_)
-    ///         The mesh to create mesh function on.
-    ///     value_collection (_MeshValueCollection_ <T>)
-    ///         The mesh value collection for the mesh function data.
-    MeshFunction(const Mesh& mesh,
-                 const MeshValueCollection<T>& value_collection);
-
     /// Create function from a MeshValueCollecion (shared_ptr version)
     ///
     /// *Arguments*
@@ -309,30 +260,8 @@ namespace dolfin
     ///         The mesh.
     ///     dim (std::size_t)
     ///         The dimension.
-    void init(const Mesh& mesh, std::size_t dim);
-
-    /// Initialize mesh function for given topological dimension
-    /// (shared_ptr version)
-    ///
-    /// *Arguments*
-    ///     mesh (_Mesh_)
-    ///         The mesh.
-    ///     dim (std::size_t)
-    ///         The dimension.
     void init(std::shared_ptr<const Mesh> mesh, std::size_t dim);
 
-    /// Initialize mesh function for given topological dimension of
-    /// given size
-    ///
-    /// *Arguments*
-    ///     mesh (_Mesh_)
-    ///         The mesh.
-    ///     dim (std::size_t)
-    ///         The dimension.
-    ///     size (std::size_t)
-    ///         The size.
-    void init(const Mesh& mesh, std::size_t dim, std::size_t size);
-
     /// Initialize mesh function for given topological dimension of
     /// given size (shared_ptr version)
     ///
@@ -386,9 +315,10 @@ namespace dolfin
 
   private:
 
-    // Values at the set of mesh entities. We don't use a std::vector<T>
-    // here because it has trouble with bool, which C++ specialises.
-    boost::scoped_array<T> _values;
+    // Values at the set of mesh entities. We don't use a
+    // std::vector<T> here because it has trouble with bool, which C++
+    // specialises.
+    std::unique_ptr<T[]> _values;
 
     // The mesh
     std::shared_ptr<const Mesh> _mesh;
@@ -407,115 +337,62 @@ namespace dolfin
   // Implementation of MeshFunction
   //---------------------------------------------------------------------------
   template <typename T>
-    MeshFunction<T>::MeshFunction() : Variable("f", "unnamed MeshFunction"),
-    Hierarchical<MeshFunction<T> >(*this), _values(0), _dim(0),
-    _size(0)
+  MeshFunction<T>::MeshFunction() : MeshFunction(nullptr)
   {
     // Do nothing
   }
   //---------------------------------------------------------------------------
   template <typename T>
-    MeshFunction<T>::MeshFunction(const Mesh& mesh)
+  MeshFunction<T>::MeshFunction(std::shared_ptr<const Mesh> mesh)
     : Variable("f", "unnamed MeshFunction"),
-    Hierarchical<MeshFunction<T> >(*this),
-    _mesh(reference_to_no_delete_pointer(mesh)), _dim(0), _size(0)
+      Hierarchical<MeshFunction<T>>(*this), _mesh(mesh), _dim(0), _size(0)
   {
     // Do nothing
   }
   //---------------------------------------------------------------------------
   template <typename T>
-    MeshFunction<T>::MeshFunction(std::shared_ptr<const Mesh> mesh)
+  MeshFunction<T>::MeshFunction(std::shared_ptr<const Mesh> mesh,
+                                std::size_t dim)
     : Variable("f", "unnamed MeshFunction"),
-    Hierarchical<MeshFunction<T> >(*this), _mesh(mesh), _dim(0), _size(0)
-  {
-    // Do nothing
-  }
-  //---------------------------------------------------------------------------
-  template <typename T>
-  MeshFunction<T>::MeshFunction(const Mesh& mesh, std::size_t dim)
-    : Variable("f", "unnamed MeshFunction"),
-    Hierarchical<MeshFunction<T> >(*this),
-    _mesh(reference_to_no_delete_pointer(mesh)) , _dim(0), _size(0)
+      Hierarchical<MeshFunction<T>>(*this), _mesh(mesh), _dim(0), _size(0)
   {
     init(dim);
   }
   //---------------------------------------------------------------------------
   template <typename T>
-    MeshFunction<T>::MeshFunction(std::shared_ptr<const Mesh> mesh,
-                                  std::size_t dim)
-    : Variable("f", "unnamed MeshFunction"),
-    Hierarchical<MeshFunction<T> >(*this), _mesh(mesh), _dim(0), _size(0)
+  MeshFunction<T>::MeshFunction(std::shared_ptr<const Mesh> mesh,
+                                std::size_t dim, const T& value)
+    : MeshFunction(mesh, dim)
+
   {
-    init(dim);
-  }
-  //---------------------------------------------------------------------------
-  template <typename T>
-  MeshFunction<T>::MeshFunction(const Mesh& mesh, std::size_t dim,
-                                const T& value)
-    : Variable("f", "unnamed MeshFunction"),
-    Hierarchical<MeshFunction<T> >(*this),
-    _mesh(reference_to_no_delete_pointer(mesh)), _dim(0), _size(0)
-  {
-    init(dim);
     set_all(value);
   }
   //---------------------------------------------------------------------------
-  template <typename T>
-    MeshFunction<T>::MeshFunction(std::shared_ptr<const Mesh> mesh,
-                                   std::size_t dim, const T& value)
-  : Variable("f", "unnamed MeshFunction"),
-    Hierarchical<MeshFunction<T> >(*this), _mesh(mesh), _dim(0), _size(0)
-  {
-    init(dim);
-    set_all(value);
-  }
-  //---------------------------------------------------------------------------
-  template <typename T>
-  MeshFunction<T>::MeshFunction(const Mesh& mesh, const std::string filename)
-    : Variable("f", "unnamed MeshFunction"),
-    Hierarchical<MeshFunction<T> >(*this),
-    _mesh(reference_to_no_delete_pointer(mesh)), _dim(0), _size(0)
-  {
-    File file(filename);
-    file >> *this;
-  }
-  //---------------------------------------------------------------------------
   template <typename T>
     MeshFunction<T>::MeshFunction(std::shared_ptr<const Mesh> mesh,
                                   const std::string filename)
     : Variable("f", "unnamed MeshFunction"),
-    Hierarchical<MeshFunction<T> >(*this), _mesh(mesh), _dim(0), _size(0)
+    Hierarchical<MeshFunction<T>>(*this), _mesh(mesh), _dim(0), _size(0)
   {
     File file(filename);
     file >> *this;
   }
   //---------------------------------------------------------------------------
-  template <typename T>
-  MeshFunction<T>::MeshFunction(const Mesh& mesh,
-    const MeshValueCollection<T>& value_collection) :
-    Variable("f", "unnamed MeshFunction"),
-      Hierarchical<MeshFunction<T> >(*this),
-      _mesh(reference_to_no_delete_pointer(mesh)),
-      _dim(value_collection.dim()), _size(0)
-  {
-    *this = value_collection;
-  }
-  //---------------------------------------------------------------------------
   template <typename T>
     MeshFunction<T>::MeshFunction(std::shared_ptr<const Mesh> mesh,
                                   const MeshValueCollection<T>& value_collection)
     : Variable("f", "unnamed MeshFunction"),
-      Hierarchical<MeshFunction<T> >(*this), _mesh(mesh),
+      Hierarchical<MeshFunction<T>>(*this), _mesh(mesh),
       _dim(value_collection.dim()), _size(0)
   {
     *this = value_collection;
   }
   //---------------------------------------------------------------------------
   template <typename T>
-    MeshFunction<T>::MeshFunction(std::shared_ptr<const Mesh> mesh,
-                                  std::size_t dim, const MeshDomains& domains)
+  MeshFunction<T>::MeshFunction(std::shared_ptr<const Mesh> mesh,
+                                std::size_t dim, const MeshDomains& domains)
     : Variable("f", "unnamed MeshFunction"),
-      Hierarchical<MeshFunction<T> >(*this), _mesh(mesh), _dim(0), _size(0)
+      Hierarchical<MeshFunction<T>>(*this), _mesh(mesh), _dim(0), _size(0)
   {
     dolfin_assert(_mesh);
 
@@ -551,7 +428,7 @@ namespace dolfin
   template <typename T>
   MeshFunction<T>::MeshFunction(const MeshFunction<T>& f) :
     Variable("f", "unnamed MeshFunction"),
-    Hierarchical<MeshFunction<T> >(*this), _dim(0), _size(0)
+    Hierarchical<MeshFunction<T>>(*this), _dim(0), _size(0)
   {
     *this = f;
   }
@@ -566,7 +443,7 @@ namespace dolfin
     _size = f._size;
     std::copy(f._values.get(), f._values.get() + _size, _values.get());
 
-    Hierarchical<MeshFunction<T> >::operator=(f);
+    Hierarchical<MeshFunction<T>>::operator=(f);
 
     return *this;
   }
@@ -592,7 +469,7 @@ namespace dolfin
     set_all(std::numeric_limits<T>::max());
 
     // Iterate over all values
-    boost::unordered_set<std::size_t> entities_values_set;
+    std::unordered_set<std::size_t> entities_values_set;
     typename std::map<std::pair<std::size_t, std::size_t>, T>::const_iterator it;
     const std::map<std::pair<std::size_t, std::size_t>, T>& values
       = mesh_value_collection.values();
@@ -708,7 +585,7 @@ namespace dolfin
     const MeshFunction<T>& MeshFunction<T>::operator= (const T& value)
   {
     set_all(value);
-    //Hierarchical<MeshFunction<T> >::operator=(value);
+    //Hierarchical<MeshFunction<T>>::operator=(value);
     return *this;
   }
   //---------------------------------------------------------------------------
@@ -739,13 +616,6 @@ namespace dolfin
     init(_mesh, dim, size);
   }
   //---------------------------------------------------------------------------
-  template <typename T>
-    void MeshFunction<T>::init(const Mesh& mesh, std::size_t dim)
-  {
-    mesh.init(dim);
-    init(mesh, dim, mesh.size(dim));
-  }
-  //---------------------------------------------------------------------------
   template <typename T>
     void MeshFunction<T>::init(std::shared_ptr<const Mesh> mesh,
                                std::size_t dim)
@@ -755,22 +625,6 @@ namespace dolfin
     init(mesh, dim, mesh->size(dim));
   }
   //---------------------------------------------------------------------------
-  template <typename T>
-    void MeshFunction<T>::init(const Mesh& mesh, std::size_t dim,
-                               std::size_t size)
-  {
-    // Initialize mesh for entities of given dimension
-    mesh.init(dim);
-    dolfin_assert(mesh.size(dim) == size);
-
-    // Initialize data
-    if (_size != size)
-      _values.reset(new T[size]);
-    _mesh = reference_to_no_delete_pointer(mesh);
-    _dim = dim;
-    _size = size;
-  }
-  //---------------------------------------------------------------------------
   template <typename T>
     void MeshFunction<T>::init(std::shared_ptr<const Mesh> mesh,
                                std::size_t dim, std::size_t size)
