@@ -19,9 +19,10 @@
 // Last changed: 2012-06-25
 
 #include <vector>
-#include <boost/shared_ptr.hpp>
+#include <memory>
 #include <dolfin/common/NoDeleter.h>
 #include <dolfin/log/log.h>
+#include <dolfin/log/Progress.h>
 #include "Cell.h"
 #include "Mesh.h"
 #include "MeshOrdering.h"
