@@ -16,11 +16,12 @@
 // along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
 //
 // First added:  2006-05-08
-// Last changed: 2012-10-25
+// Last changed: 2014-07-02
 
 #ifndef __MESH_TOPOLOGY_H
 #define __MESH_TOPOLOGY_H
 
+#include <cstdint>
 #include <map>
 #include <utility>
 #include <vector>
@@ -65,6 +66,10 @@ namespace dolfin
     /// Return global number of entities for given dimension
     std::size_t size_global(std::size_t dim) const;
 
+    /// Return number of regular (non-ghost) entities
+    /// or equivalently, the offset of where ghost entities begin
+    std::size_t ghost_offset(std::size_t dim) const;
+
     /// Clear all data
     void clear();
 
@@ -82,6 +87,9 @@ namespace dolfin
     /// dimension dim
     void init_global_indices(std::size_t dim, std::size_t size);
 
+    /// Initialise the offset index of ghost entities for this dimension
+    void init_ghost(std::size_t dim, std::size_t index);
+
     /// Set global index for entity of dimension dim and with local
     /// index
     void set_global_index(std::size_t dim, std::size_t local_index,
@@ -100,7 +108,7 @@ namespace dolfin
       return _global_indices[d];
     }
 
-    /// Check if global indices are available for entiries of
+    /// Check if global indices are available for entities of
     /// dimension dim
     bool have_global_indices(std::size_t dim) const
     {
@@ -108,16 +116,33 @@ namespace dolfin
       return !_global_indices[dim].empty();
     }
 
+    /// Check whether there are any shared entities calculated
+    /// of dimension dim
+    bool have_shared_entities(unsigned int dim) const
+    { return (_shared_entities.find(dim) != _shared_entities.end()); }
+
     /// Return map from shared entities (local index) to processes
     /// that share the entity
-    std::map<unsigned int, std::set<unsigned int> >&
+    std::map<std::int32_t, std::set<unsigned int> >&
       shared_entities(unsigned int dim);
 
-    /// Return map from shared entiies (local index) to process that
+    /// Return map from shared entities (local index) to process that
     /// share the entity (const version)
-    const std::map<unsigned int, std::set<unsigned int> >&
+    const std::map<std::int32_t, std::set<unsigned int> >&
       shared_entities(unsigned int dim) const;
 
+    /// Return mapping from local ghost cell index to owning process
+    /// Since ghost cells are at the end of the range, this is just
+    /// a vector over those cells
+    std::vector<unsigned int>& cell_owner()
+    { return _cell_owner;  }
+
+    /// Return mapping from local ghost cell index to owning process (const version)
+    /// Since ghost cells are at the end of the range, this is just
+    /// a vector over those cells
+    const std::vector<unsigned int>& cell_owner() const
+    { return _cell_owner;  }
+
     /// Return connectivity for given pair of topological dimensions
     dolfin::MeshConnectivity& operator() (std::size_t d0, std::size_t d1);
 
@@ -143,16 +168,16 @@ namespace dolfin
     //                 to avoid circular dependencies in the header files
     std::map<std::vector<std::size_t>,
       std::pair<std::vector<std::size_t>,
-      std::vector<std::vector<std::size_t> > > > coloring;
+      std::vector<std::vector<std::size_t>>>> coloring;
 
   private:
 
-    // Friends
-    friend class BinaryFile;
-
     // Number of mesh entities for each topological dimension
     std::vector<unsigned int> num_entities;
 
+    // Number of ghost indices for each topological dimension
+    std::vector<std::size_t> ghost_offset_index;
+
     // Global number of mesh entities for each topological dimension
     std::vector<std::size_t> global_num_entities;
 
@@ -161,9 +186,14 @@ namespace dolfin
 
     // For entities of a given dimension d , maps each shared entity
     // (local index) to a list of the processes sharing the vertex
-    std::map<unsigned int, std::map<unsigned int, std::set<unsigned int> > >
+    std::map<unsigned int, std::map<std::int32_t, std::set<unsigned int>>>
       _shared_entities;
 
+    // For cells which are "ghosted", locate the owning process,
+    // using a vector rather than a map,
+    // since ghost cells are always at the end of the range.
+    std::vector<unsigned int> _cell_owner;
+
     // Connectivity for pairs of topological dimensions
     std::vector<std::vector<MeshConnectivity> > connectivity;
 
