@@ -85,14 +85,6 @@ void PointCell::create_entities(boost::multi_array<unsigned int, 2>& e,
                "Entities on a point cell are not defined");
 }
 //-----------------------------------------------------------------------------
-void PointCell::refine_cell(Cell& cell, MeshEditor& editor,
-                            std::size_t& current_cell) const
-{
-  dolfin_error("PointCell.cpp",
-               "refine cell",
-               "Refinement of a point cell is not defined");
-}
-//-----------------------------------------------------------------------------
 double PointCell::volume(const MeshEntity& triangle) const
 {
   dolfin_error("PointCell.cpp",
@@ -101,11 +93,12 @@ double PointCell::volume(const MeshEntity& triangle) const
   return 0.0;
 }
 //-----------------------------------------------------------------------------
-double PointCell::diameter(const MeshEntity& triangle) const
+
+double PointCell::circumradius(const MeshEntity& point) const
 {
   dolfin_error("PointCell.cpp",
-               "find diameter of cell",
-               "Diameter of a point cell is not defined");
+               "find circumradious of cell",
+               "Circumradius of a point cell is not defined");
   return 0.0;
 }
 //-----------------------------------------------------------------------------
