@@ -24,6 +24,7 @@
 #ifndef __POINT_CELL_H
 #define __POINT_CELL_H
 
+#include <boost/multi_array.hpp>
 #include "CellType.h"
 
 namespace dolfin
@@ -41,38 +42,35 @@ namespace dolfin
     /// Return topological dimension of cell
     std::size_t dim() const;
 
-    /// Return number of entitites of given topological dimension
+    /// Return number of entities of given topological dimension
     std::size_t num_entities(std::size_t dim) const;
 
-    /// Return number of vertices for entity of given topological dimension
+    /// Return number of vertices for entity of given topological
+    /// dimension
     std::size_t num_vertices(std::size_t dim) const;
 
     /// Return orientation of the cell
     std::size_t orientation(const Cell& cell) const;
 
     /// Create entities e of given topological dimension from vertices v
-    void create_entities(std::vector<std::vector<unsigned int> >& e,
+    void create_entities(boost::multi_array<unsigned int, 2>& e,
                          std::size_t dim, const unsigned int* v) const;
 
     /// Order entities locally (connectivity 1-0, 2-0, 2-1)
-    void
-      order(Cell& cell,
-            const std::vector<std::size_t>& local_to_global_vertex_indices) const;
-
-    /// Refine cell uniformly
-    void refine_cell(Cell& cell, MeshEditor& editor,
-                     std::size_t& current_cell) const;
+    void order(Cell& cell, const std::vector<std::size_t>&
+               local_to_global_vertex_indices) const;
 
     /// Compute (generalized) volume (area) of triangle
     double volume(const MeshEntity& triangle) const;
 
-    /// Compute diameter of triangle
-    double diameter(const MeshEntity& triangle) const;
+    /// Compute circumradius of PointCell
+    double circumradius(const MeshEntity& point) const;
 
     /// Compute squared distance to given point
     double squared_distance(const Cell& cell, const Point& point) const;
 
-    /// Compute component i of normal of given facet with respect to the cell
+    /// Compute component i of normal of given facet with respect to
+    /// the cell
     double normal(const Cell& cell, std::size_t facet, std::size_t i) const;
 
     /// Compute of given facet with respect to the cell
@@ -81,7 +79,8 @@ namespace dolfin
     /// Compute normal to given cell (viewed as embedded in 1D)
     Point cell_normal(const Cell& cell) const;
 
-    /// Compute the area/length of given facet with respect to the cell
+    /// Compute the area/length of given facet with respect to the
+    /// cell
     double facet_area(const Cell& cell, std::size_t facet) const;
 
     /// Order entities locally
@@ -92,7 +91,7 @@ namespace dolfin
 
     /// Check whether given entity collides with cell
     bool collides(const Cell& cell, const MeshEntity& entity) const;
-    
+
     /// Compute triangulation of intersection of two cells
     virtual std::vector<double>
     triangulate_intersection(const Cell& c0, const Cell& c1) const;
@@ -100,6 +99,10 @@ namespace dolfin
     /// Return description of cell type
     std::string description(bool plural) const;
 
+    /// Mapping of DOLFIN/UFC vertex ordering to VTK/XDMF ordering
+    std::vector<std::int8_t> vtk_mapping() const
+    { return {0}; }
+
   private:
 
     // Find local index of edge i according to ordering convention
