@@ -25,6 +25,7 @@
 #define __TRIANGLE_CELL_H
 
 #include <vector>
+#include <boost/multi_array.hpp>
 #include "CellType.h"
 
 namespace dolfin
@@ -52,19 +53,15 @@ namespace dolfin
     std::size_t orientation(const Cell& cell) const;
 
     /// Create entities e of given topological dimension from vertices v
-    void create_entities(std::vector<std::vector<unsigned int> >& e,
+    void create_entities(boost::multi_array<unsigned int, 2>& e,
                          std::size_t dim,
                          const unsigned int* v) const;
 
-    /// Refine cell uniformly
-    void refine_cell(Cell& cell, MeshEditor& editor,
-                     std::size_t& current_cell) const;
-
     /// Compute (generalized) volume (area) of triangle
     double volume(const MeshEntity& triangle) const;
 
     /// Compute diameter of triangle
-    double diameter(const MeshEntity& triangle) const;
+    double circumradius(const MeshEntity& triangle) const;
 
     /// Compute squared distance to given point (3D enabled)
     double squared_distance(const Cell& cell, const Point& point) const;
@@ -107,6 +104,10 @@ namespace dolfin
     /// Return description of cell type
     std::string description(bool plural) const;
 
+    /// Mapping of DOLFIN/UFC vertex ordering to VTK/XDMF ordering
+    std::vector<std::int8_t> vtk_mapping() const
+    { return {0, 1, 2}; }
+
   private:
 
     // Find local index of edge i according to ordering convention
