@@ -47,16 +47,18 @@ namespace dolfin
   public:
 
     /// Create nonlinear solver
-    NewtonSolver();
+    explicit NewtonSolver(MPI_Comm comm=MPI_COMM_WORLD);
 
     /// Create nonlinear solver using provided linear solver
     ///
     /// *Arguments*
+    ///     comm (_MPI_Ccmm_)
+    ///         The MPI communicator.
     ///     solver (_GenericLinearSolver_)
     ///         The linear solver.
     ///     factory (_GenericLinearAlgebraFactory_)
     ///         The factory.
-    NewtonSolver(std::shared_ptr<GenericLinearSolver> solver,
+    NewtonSolver(MPI_Comm comm, std::shared_ptr<GenericLinearSolver> solver,
                  GenericLinearAlgebraFactory& factory);
 
     /// Destructor
@@ -113,13 +115,28 @@ namespace dolfin
     ///         Parameter values.
     static Parameters default_parameters();
 
-  private:
+  protected:
 
-    // Convergence test
+    /// Convergence test. It may be overloaded using virtual inheritance and
+    /// this base criterion may be called from derived, both in C++ and Python.
+    ///
+    /// *Arguments*
+    ///     r (_GenericVector_)
+    ///         Residual for criterion evaluation.
+    ///     nonlinear_problem (_NonlinearProblem_)
+    ///         The nonlinear problem.
+    ///     iteration (std::size_t)
+    ///         Newton iteration number.
+    ///
+    /// *Returns*
+    ///     bool
+    ///         Whether convergence occured.
     virtual bool converged(const GenericVector& r,
                            const NonlinearProblem& nonlinear_problem,
                            std::size_t iteration);
 
+  private:
+
     // Current number of Newton iterations
     std::size_t _newton_iteration;
 
