@@ -21,7 +21,7 @@
 // First added:  2012-10-13
 // Last changed: 2013-11-21
 
-#ifdef ENABLE_PETSC_SNES
+#ifdef HAS_PETSC
 
 #include <map>
 #include <string>
@@ -31,7 +31,6 @@
 
 #include <dolfin/common/MPI.h>
 #include <dolfin/common/NoDeleter.h>
-#include <dolfin/common/timing.h>
 #include <dolfin/common/Timer.h>
 #include <dolfin/la/PETScKrylovSolver.h>
 #include <dolfin/la/PETScMatrix.h>
@@ -43,30 +42,8 @@
 
 using namespace dolfin;
 
-#if PETSC_VERSION_MAJOR == 3 && PETSC_VERSION_MINOR == 3
 // Mapping from method string to PETSc
-const std::map<std::string, std::pair<std::string, const SNESType> >
-PETScSNESSolver::_methods
-= { {"default",     {"default SNES method", ""}},
-    {"ls",          {"Line search method", SNESLS}},
-    ("tr",          {"Trust region method",  SNESTR}},
-    {"test",        {"Tool to verify Jacobian approximation", SNESTEST}},
-    {"ngmres",      {"Nonlinear generalised minimum residual method",
-                     SNESNGMRES}},
-    {"nrichardson", {"Richardson nonlinear method (Picard iteration)",
-                     SNESNRICHARDSON}},
-    {"virs",        {"Reduced space active set solver method (for bounds)",
-                     SNESVIRS}},
-    {"viss",        {"Reduced space active set solver method (for bounds)",
-                     SNESVISS}},
-    {"qn",          {"Limited memory quasi-Newton", SNESQN}},
-    {"ncg",         {"Nonlinear conjugate gradient method", SNESNCG}},
-    {"fas",         {"Full Approximation Scheme nonlinear multigrid method",
-                     SNESFAS}},
-    {"ms",          {"Multistage smoothers", SNESMS}} };
-#elif PETSC_VERSION_MAJOR == 3 && PETSC_VERSION_MINOR >= 4
-// Mapping from method string to PETSc
-const std::map<std::string, std::pair<std::string, const SNESType> >
+const std::map<std::string, std::pair<std::string, const SNESType>>
 PETScSNESSolver::_methods
 = { {"default",      {"default SNES method", ""}},
     {"newtonls",     {"Line search method", SNESNEWTONLS}},
@@ -89,12 +66,11 @@ PETScSNESSolver::_methods
     {"aspin",        {"Additive-Schwarz Preconditioned Inexact Newton",
                       SNESASPIN}},
     {"ms",           {"Multistage smoothers", SNESMS}} };
-#endif
 
 //-----------------------------------------------------------------------------
-std::vector<std::pair<std::string, std::string> > PETScSNESSolver::methods()
+std::vector<std::pair<std::string, std::string>> PETScSNESSolver::methods()
 {
-  std::vector<std::pair<std::string, std::string> > available_methods;
+  std::vector<std::pair<std::string, std::string>> available_methods;
   for (auto it = _methods.begin(); it != _methods.end(); ++it)
     available_methods.push_back(std::make_pair(it->first, it->second.first));
   return available_methods;
@@ -106,23 +82,43 @@ Parameters PETScSNESSolver::default_parameters()
   p.rename("snes_solver");
   p.add("solution_tolerance", 1.0e-16);
   p.add("maximum_residual_evaluations", 2000);
-  p.add("options_prefix", "default");
   p.remove("convergence_criterion");
   p.remove("relaxation_parameter");
   p.remove("method");
   p.add("method", "default");
-  p.add("line_search", "basic",  {"basic", "bt", "l2", "cp" });
+  p.add("line_search", "basic",  {"basic", "bt", "l2", "cp", "nleqerr"});
   p.add("sign", "default", {"default", "nonnegative", "nonpositive"});
 
   return p;
 }
 //-----------------------------------------------------------------------------
-PETScSNESSolver::PETScSNESSolver(std::string nls_type) :
-  _snes(NULL)
+PETScSNESSolver::PETScSNESSolver(MPI_Comm comm) : _snes(nullptr),
+                                                  _has_explicit_bounds(false)
 {
+  // Create SNES object
+  SNESCreate(comm, &_snes);
+
+  // Set parameter values
+  parameters = default_parameters();
+}
+//-----------------------------------------------------------------------------
+PETScSNESSolver::PETScSNESSolver(std::string nls_type) : _snes(nullptr),
+                                                         _has_explicit_bounds(false)
+{
+  // Create SNES object
+  SNESCreate(MPI_COMM_WORLD, &_snes);
+
+  const std::map<std::string, std::pair<std::string, const SNESType>>::const_iterator
+    method = _methods.find(nls_type);
 
   // Check that the requested method is known
-  if (_methods.count(nls_type) == 0)
+  if (method != _methods.end())
+  {
+    // Set solver type
+    if (method->first != "default")
+      SNESSetType(_snes, method->second.second);
+  }
+  else
   {
     dolfin_error("PETScSNESSolver.cpp",
                  "create PETSc SNES solver",
@@ -131,8 +127,6 @@ PETScSNESSolver::PETScSNESSolver(std::string nls_type) :
 
   // Set parameter values
   parameters = default_parameters();
-
-  init(nls_type);
 }
 //-----------------------------------------------------------------------------
 PETScSNESSolver::~PETScSNESSolver()
@@ -141,26 +135,6 @@ PETScSNESSolver::~PETScSNESSolver()
     SNESDestroy(&_snes);
 }
 //-----------------------------------------------------------------------------
-void PETScSNESSolver::init(const std::string& method)
-{
-  if (_snes)
-    SNESDestroy(&_snes);
-
-  // Create SNES object
-  SNESCreate(PETSC_COMM_WORLD, &_snes);
-
-  // Set solver type
-  if (method != "default")
-  {
-    auto it = _methods.find(method);
-    dolfin_assert(it != _methods.end());
-    SNESSetType(_snes, it->second.second);
-  }
-
-  // Set to default to not having explicit bounds
-  has_explicit_bounds = false;
-}
-//-----------------------------------------------------------------------------
 std::pair<std::size_t, bool>
 PETScSNESSolver::solve(NonlinearProblem& nonlinear_problem,
                        GenericVector& x,
@@ -191,14 +165,13 @@ PETScSNESSolver::solve(NonlinearProblem& nonlinear_problem,
     _lb(&lb.down_cast<PETScVector>(), NoDeleter());
   this->lb = _lb;
   this->ub = _ub;
-  has_explicit_bounds = true;
+  _has_explicit_bounds = true;
 
   return this->solve(nonlinear_problem, x);
 }
 //-----------------------------------------------------------------------------
-void
-PETScSNESSolver::init(NonlinearProblem& nonlinear_problem,
-                       GenericVector& x)
+void PETScSNESSolver::init(NonlinearProblem& nonlinear_problem,
+                           GenericVector& x)
 {
   Timer timer("SNES solver init");
   PETScMatrix A;
@@ -206,9 +179,6 @@ PETScSNESSolver::init(NonlinearProblem& nonlinear_problem,
   // Set linear solver parameters
   set_linear_solver_parameters();
 
-  const bool report = parameters["report"];
-
-
   _snes_ctx.nonlinear_problem = &nonlinear_problem;
   _snes_ctx.x = &x.down_cast<PETScVector>();
   VecDuplicate(_snes_ctx.x->vec(), &_snes_ctx.f_tmp);
@@ -225,58 +195,78 @@ PETScSNESSolver::init(NonlinearProblem& nonlinear_problem,
                   &_snes_ctx);
   SNESSetObjective(_snes, PETScSNESSolver::FormObjective, &_snes_ctx);
 
-  std::string prefix = std::string(parameters["options_prefix"]);
-  if (prefix != "default")
-  {
-    // Make sure that the prefix has a '_' at the end if the user
-    // didn't provide it
-    char lastchar = *prefix.rbegin();
-    if (lastchar != '_')
-      prefix += "_";
-
-    SNESSetOptionsPrefix(_snes, prefix.c_str());
-  }
-
   // Set some options from the parameters
-  if (report)
-    SNESMonitorSet(_snes, SNESMonitorDefault, PETSC_NULL, PETSC_NULL);
+  if (parameters["report"].is_set())
+  {
+    if (parameters["report"])
+    {
+      PetscErrorCode ierr;
+      #if PETSC_VERSION_MAJOR == 3 && PETSC_VERSION_MINOR <= 6 && PETSC_VERSION_RELEASE == 1
+      ierr = SNESMonitorSet(_snes, SNESMonitorDefault,
+                            PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)_snes)),
+                            NULL);
+      if (ierr != 0) petsc_error(ierr, __FILE__, "SNESMonitorSet");
+      #else
+      PetscViewer viewer = PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)_snes));
+      PetscViewerFormat format = PETSC_VIEWER_DEFAULT;
+      PetscViewerAndFormat *vf;
+      PetscViewerAndFormatCreate(viewer,format,&vf);
+      ierr = SNESMonitorSet(_snes,
+                            (PetscErrorCode (*)(SNES,PetscInt,PetscReal,void*)) SNESMonitorDefault,
+                            vf,
+                            (PetscErrorCode (*)(void**))PetscViewerAndFormatDestroy);
+      if (ierr != 0) petsc_error(ierr, __FILE__, "SNESMonitorSet");
+      #endif
+    }
+  }
 
   // Set the bounds, if any
   set_bounds(x);
 
   // Set the method
-  if (std::string(parameters["method"]) != "default")
+  const std::string method = parameters["method"];
+  if (method != "default")
   {
-    auto it = _methods.find(std::string(parameters["method"]));
+    auto it = _methods.find(method);
     dolfin_assert(it != _methods.end());
     SNESSetType(_snes, it->second.second);
-  // If
-  //      a) the user has set bounds (is_vi())
-  // AND  b) the user has not set a solver (method == default)
-  // THEN set a good method that supports bounds
-  // (most methods do not support bounds)
+
+    // Check if bounds/sign are set when VI method requested
+    if ((method == "vinewtonrsls" || method == "vinewtonssls") && !is_vi())
+    {
+      dolfin_error("PETScSNESSolver.cpp",
+                   "set up SNES VI solver",
+                   "Need to set bounds or sign for vinewtonrsls or vinewtonssls"
+                   " methods");
+    }
   }
-  else if (std::string(parameters["method"]) == "default" && is_vi())
+  else if (method == "default" && is_vi())
   {
-    #if PETSC_VERSION_MAJOR == 3 && PETSC_VERSION_MINOR == 3 && PETSC_VERSION_RELEASE
-    auto it = _methods.find("viss");
-    #else
+    // If
+    //      a) the user has set bounds (is_vi())
+    // AND  b) the user has not set a solver (method == default)
+    // THEN set a good method that supports bounds
+    // (most methods do not support bounds)
     auto it = _methods.find("vinewtonssls");
-    #endif
     dolfin_assert(it != _methods.end());
     SNESSetType(_snes, it->second.second);
   }
 
   SNESLineSearch linesearch;
-
-  #if PETSC_VERSION_MAJOR == 3 && PETSC_VERSION_MINOR < 4
-  SNESGetSNESLineSearch(_snes, &linesearch);
-  #else
   SNESGetLineSearch(_snes, &linesearch);
-  #endif
 
-  if (report)
-    SNESLineSearchSetMonitor(linesearch, PETSC_TRUE);
+  if (parameters["report"].is_set())
+  {
+    if (parameters["report"])
+    {
+      #if PETSC_VERSION_MAJOR == 3 && PETSC_VERSION_MINOR <= 6 && PETSC_VERSION_RELEASE == 1
+      SNESLineSearchSetMonitor(linesearch, PETSC_TRUE);
+      #else
+      SNESLineSearchMonitor(linesearch);
+      #endif
+    }
+  }
+
   const std::string line_search_type = std::string(parameters["line_search"]);
   SNESLineSearchSetType(linesearch, line_search_type.c_str());
 
@@ -292,6 +282,38 @@ PETScSNESSolver::init(NonlinearProblem& nonlinear_problem,
   SNESSetFromOptions(_snes);
 }
 //-----------------------------------------------------------------------------
+void PETScSNESSolver::set_options_prefix(std::string options_prefix)
+{
+  // Set options prefix
+  dolfin_assert(_snes);
+  PetscErrorCode ierr = SNESSetOptionsPrefix(_snes, options_prefix.c_str());
+  if (ierr != 0) petsc_error(ierr, __FILE__, "SNESSetOptionsPrefix");
+}
+//-----------------------------------------------------------------------------
+std::string PETScSNESSolver::get_options_prefix() const
+{
+  dolfin_assert(_snes);
+  const char* prefix = NULL;
+  PetscErrorCode ierr = SNESGetOptionsPrefix(_snes, &prefix);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "SNESGetOptionsPrefix");
+  return std::string(prefix);
+}
+//-----------------------------------------------------------------------------
+void PETScSNESSolver::set_from_options() const
+{
+  dolfin_assert(_snes);
+  PetscErrorCode ierr = SNESSetFromOptions(_snes);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "SNESSetFromOptions");
+}
+//-----------------------------------------------------------------------------
+MPI_Comm PETScSNESSolver::mpi_comm() const
+{
+  dolfin_assert(_snes);
+  MPI_Comm mpi_comm = MPI_COMM_NULL;
+  PetscObjectGetComm((PetscObject)_snes, &mpi_comm);
+  return mpi_comm;
+}
+//-----------------------------------------------------------------------------
 std::pair<std::size_t, bool>
 PETScSNESSolver::solve(NonlinearProblem& nonlinear_problem,
                        GenericVector& x)
@@ -308,7 +330,7 @@ PETScSNESSolver::solve(NonlinearProblem& nonlinear_problem,
   // same as the vector that holds the current solution
   // guess in the dolfin form.
   PETScVector x_copy(x.down_cast<PETScVector>());
-  SNESSolve(_snes, PETSC_NULL, x_copy.vec());
+  SNESSolve(_snes, NULL, x_copy.vec());
   x.zero();
   x.axpy(1.0, x_copy);
 
@@ -321,8 +343,7 @@ PETScSNESSolver::solve(NonlinearProblem& nonlinear_problem,
 
   const bool report = parameters["report"];
 
-  MPI_Comm comm = MPI_COMM_NULL;
-  PetscObjectGetComm((PetscObject)_snes, &comm);
+  MPI_Comm comm = mpi_comm();
   if (reason > 0 && report && dolfin::MPI::rank(comm) == 0)
   {
     info("PETSc SNES solver converged in %d iterations with convergence reason %s.",
@@ -388,37 +409,6 @@ PetscErrorCode PETScSNESSolver::FormObjective(SNES snes, Vec x,
   return 0;
 }
 //-----------------------------------------------------------------------------
-#if PETSC_VERSION_MAJOR == 3 && PETSC_VERSION_MINOR <= 4
-PetscErrorCode PETScSNESSolver::FormJacobian(SNES snes, Vec x, Mat* A, Mat* P,
-                                             MatStructure* flag, void* ctx)
-{
-  // Interface does not presently support a preconditioner that
-  // differs from operator A
-  if (*A != *P)
-  {
-    dolfin_error("PETScSNESSolver.cpp",
-                 "for Jacobian",
-                 "Matrix object incompatibility. The Jacobian matrix must not be reset when using PETSc SNES.");
-  }
-
-  // Get nonlinear problem object
-  auto snes_ctx = static_cast<struct snes_ctx_t*>(ctx);
-  NonlinearProblem* nonlinear_problem = snes_ctx->nonlinear_problem;
-
-  // Wrap the PETSc objects
-  PETScMatrix A_wrap(*P);
-  PETScVector x_wrap(x);
-
-  // Form Jacobian
-  PETScVector f;
-  nonlinear_problem->form(A_wrap, f, x_wrap);
-  nonlinear_problem->J(A_wrap, x_wrap);
-
-  *flag = SAME_NONZERO_PATTERN;
-
-  return 0;
-}
-#else
 PetscErrorCode PETScSNESSolver::FormJacobian(SNES snes, Vec x, Mat A, Mat P,
                                              void* ctx)
 {
@@ -446,7 +436,6 @@ PetscErrorCode PETScSNESSolver::FormJacobian(SNES snes, Vec x, Mat A, Mat P,
 
   return 0;
 }
-#endif
 //-----------------------------------------------------------------------------
 void PETScSNESSolver::set_linear_solver_parameters()
 {
@@ -460,12 +449,27 @@ void PETScSNESSolver::set_linear_solver_parameters()
   ierr = KSPGetPC(ksp, &pc);
   if (ierr != 0) petsc_error(ierr, __FILE__, "KSPGetPC");
 
-  MPI_Comm comm = MPI_COMM_NULL;
-  PetscObjectGetComm((PetscObject)_snes, &comm);
-
-  if (parameters["report"])
-    KSPMonitorSet(ksp, KSPMonitorDefault, PETSC_NULL, PETSC_NULL);
+  // Get MPI communicator
+  MPI_Comm comm = mpi_comm();
 
+  if (parameters["report"].is_set())
+  {
+    if (parameters["report"])
+    {
+      #if PETSC_VERSION_MAJOR == 3 && PETSC_VERSION_MINOR <= 6 && PETSC_VERSION_RELEASE == 1
+      KSPMonitorSet(ksp, KSPMonitorDefault, PETSC_VIEWER_STDOUT_(comm), NULL);
+      #else
+      PetscViewer viewer = PETSC_VIEWER_STDOUT_(comm);
+      PetscViewerFormat format = PETSC_VIEWER_DEFAULT;
+      PetscViewerAndFormat *vf;
+      ierr = PetscViewerAndFormatCreate(viewer,format,&vf);
+      ierr = KSPMonitorSet(ksp,
+                           (PetscErrorCode (*)(KSP,PetscInt,PetscReal,void*)) KSPMonitorDefault,
+                           vf,
+                           (PetscErrorCode (*)(void**))PetscViewerAndFormatDestroy);
+      #endif
+    }
+  }
   const std::string linear_solver  = parameters["linear_solver"];
   const std::string preconditioner = parameters["preconditioner"];
 
@@ -488,10 +492,6 @@ void PETScSNESSolver::set_linear_solver_parameters()
 
     Parameters krylov_parameters = parameters("krylov_solver");
 
-    // GMRES restart parameter
-    const int gmres_restart = krylov_parameters("gmres")["restart"];
-    KSPGMRESSetRestart(ksp, gmres_restart);
-
     // Non-zero initial guess
     const bool nonzero_guess = krylov_parameters["nonzero_initial_guess"];
     if (nonzero_guess)
@@ -500,7 +500,23 @@ void PETScSNESSolver::set_linear_solver_parameters()
       KSPSetInitialGuessNonzero(ksp, PETSC_FALSE);
 
     if (krylov_parameters["monitor_convergence"])
-      KSPMonitorSet(ksp, KSPMonitorTrueResidualNorm, 0, 0);
+    {
+      #if PETSC_VERSION_MAJOR == 3 && PETSC_VERSION_MINOR <= 6 && PETSC_VERSION_RELEASE == 1
+      KSPMonitorSet(ksp, KSPMonitorTrueResidualNorm,
+                    PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)ksp)),
+                    NULL);
+
+      #else
+      PetscViewer viewer = PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)ksp));
+      PetscViewerFormat format = PETSC_VIEWER_DEFAULT;
+      PetscViewerAndFormat *vf;
+      ierr = PetscViewerAndFormatCreate(viewer,format,&vf);
+      ierr = KSPMonitorSet(ksp,
+                         (PetscErrorCode (*)(KSP,PetscInt,PetscReal,void*)) KSPMonitorTrueResidualNorm,
+                         vf,
+                         (PetscErrorCode (*)(void**))PetscViewerAndFormatDestroy);
+      #endif
+    }
 
     // Set tolerances
     const int max_iters = krylov_parameters["maximum_iterations"];
@@ -573,21 +589,12 @@ void PETScSNESSolver::set_bounds(GenericVector& x)
     dolfin_assert(_snes);
     const std::string sign   = parameters["sign"];
     const std::string method = parameters["method"];
-    #if PETSC_VERSION_MAJOR == 3 && PETSC_VERSION_MINOR == 3
-    if (method != "virs" && method != "viss" && method != "default")
-    {
-      dolfin_error("PETScSNESSolver.cpp",
-                   "set variational inequality bounds",
-                   "Need to use virs or viss methods if bounds are set");
-    }
-    #else
     if (method != "vinewtonrsls" && method != "vinewtonssls" && method != "default")
     {
       dolfin_error("PETScSNESSolver.cpp",
                    "set variational inequality bounds",
                    "Need to use vinewtonrsls or vinewtonssls methods if bounds are set");
     }
-    #endif
 
     if (sign != "default")
     {
@@ -600,24 +607,13 @@ void PETScSNESSolver::set_bounds(GenericVector& x)
       VecDuplicate(_x.vec(), &lb);
       if (sign == "nonnegative")
       {
-        #if PETSC_VERSION_MAJOR == 3 && PETSC_VERSION_MINOR <= 4
-        VecSet(ub, SNES_VI_INF);
-        #else
         VecSet(ub, PETSC_INFINITY);
-        #endif
-
         VecSet(lb, 0.0);
       }
       else if (sign == "nonpositive")
       {
         VecSet(ub, 0.0);
-
-        #if PETSC_VERSION_MAJOR == 3 && PETSC_VERSION_MINOR <= 4
-        VecSet(lb, SNES_VI_INF);
-        #else
         VecSet(lb, PETSC_INFINITY);
-        #endif
-
       }
       else
       {
@@ -630,7 +626,7 @@ void PETScSNESSolver::set_bounds(GenericVector& x)
       VecDestroy(&ub);
       VecDestroy(&lb);
     }
-    else if (has_explicit_bounds == true)
+    else if (_has_explicit_bounds)
     {
       const PETScVector* lb = this->lb.get();
       const PETScVector* ub = this->ub.get();
@@ -642,14 +638,14 @@ void PETScSNESSolver::set_bounds(GenericVector& x)
 bool PETScSNESSolver::is_vi() const
 {
   const std::string sign = parameters["sign"];
-  if (sign != "default" && this->has_explicit_bounds == true)
+  if (sign != "default" && this->_has_explicit_bounds)
   {
     dolfin_error("PETScSNESSolver.cpp",
                  "set variational inequality bounds",
                  "Both the sign parameter and the explicit bounds are set");
     return false;
   }
-  else if (sign != "default" || this->has_explicit_bounds == true)
+  else if (sign != "default" || this->_has_explicit_bounds)
     return true;
   else
     return false;
