@@ -37,7 +37,7 @@
 using namespace dolfin;
 
 //----------------------------------------------------------------------------
-VTKPlottableGenericFunction1D::VTKPlottableGenericFunction1D(boost::shared_ptr<const Function> function)
+VTKPlottableGenericFunction1D::VTKPlottableGenericFunction1D(std::shared_ptr<const Function> function)
   : VTKPlottableGenericFunction(function),
     _actor(vtkSmartPointer<vtkXYPlotActor>::New())
 {
@@ -45,8 +45,8 @@ VTKPlottableGenericFunction1D::VTKPlottableGenericFunction1D(boost::shared_ptr<c
   // Do nothing
 }
 //----------------------------------------------------------------------------
-VTKPlottableGenericFunction1D::VTKPlottableGenericFunction1D(boost::shared_ptr<const Expression> expression,
-                                                             boost::shared_ptr<const Mesh> mesh)
+VTKPlottableGenericFunction1D::VTKPlottableGenericFunction1D(std::shared_ptr<const Expression> expression,
+                                                             std::shared_ptr<const Mesh> mesh)
   : VTKPlottableGenericFunction(expression, mesh),
     _actor(vtkSmartPointer<vtkXYPlotActor>::New())
 {
@@ -120,7 +120,7 @@ bool VTKPlottableGenericFunction1D::is_compatible(const Variable &var) const
   return VTKPlottableGenericFunction::is_compatible(var);
 }
 //----------------------------------------------------------------------------
-void VTKPlottableGenericFunction1D::update(boost::shared_ptr<const Variable> var,
+void VTKPlottableGenericFunction1D::update(std::shared_ptr<const Variable> var,
                                            const Parameters& p, int framecounter)
 {
   VTKPlottableGenericFunction::update(var, p, framecounter);
