@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 """This module provides functionality to compile PyDOLFIN compatible
 extension modules."""
 
@@ -122,8 +123,21 @@ import_array();
 %%{
 #include <ufc.h>
 %%}
-%%include "swig/ufc_shared_ptr_classes.i"
-%%import(module="ufc") "ufc.h"
+%%shared_ptr(ufc::function)
+%%shared_ptr(ufc::dofmap)
+%%shared_ptr(ufc::finite_element)
+%%shared_ptr(ufc::function)
+%%shared_ptr(ufc::form)
+%%ignore ufc::integral;
+%%ignore ufc::cell_integral;
+%%ignore ufc::exterior_facet_integral;
+%%ignore ufc::interior_facet_integral;
+%%ignore ufc::vertex_integral;
+%%ignore ufc::custom_integral;
+%%ignore ufc::cutcell_integral;
+%%ignore ufc::interface_integral;
+%%ignore ufc::overlap_integral;
+%%include <ufc.h>
 
 // Local shared_ptr declarations
 %(shared_ptr_declarations)s
@@ -147,6 +161,9 @@ del tmp, get_swigversion
 
 """
 
+re_symbol = re.compile(r"(?:[a-zA-Z_]*::)*[\w\.]+")
+re_number = re.compile("[-+]?(\d+(\.\d*)?|\.\d+)([eE][-+]?\d+)?")
+
 def expression_to_code_fragments(expr, arguments, generic_function_members=None):
     "A help function which extract a dict with code snippets from an expression"
 
@@ -158,7 +175,7 @@ def expression_to_code_fragments(expr, arguments, generic_function_members=None)
 
     for i, c in enumerate(expr):
         # Find groups of connected alphanumeric letters
-        symbols = re.findall(r"(?:[a-zA-Z_]*::)*[\w\.]+", c)
+        symbols = re_symbol.findall(c)
         assert isinstance(symbols, list)
         variables.update(symbols)
         for sym in symbols:
@@ -181,16 +198,12 @@ def expression_to_code_fragments(expr, arguments, generic_function_members=None)
     variables.difference_update(_all_math)
 
     # Remove the numerals from the variables
-    numerals = [v for v in variables if v[0] in "0123456789"]
+    numerals = [v for v in variables if re_number.match(v)]
     variables.difference_update(numerals)
 
     # Remove bool literals
     variables.difference_update(["true", "false"])
 
-    # Remove any exponential representation
-    exponentials = [v for v in variables if bool(re.search("e[0-9,\-,\.,\+]+", v))]
-    variables.difference_update(exponentials+["e"])
-
     # Remove namespace-specified variables
     namespacevars = [v for v in variables if "::" in v]
     variables.difference_update(namespacevars)
