@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 """This module provides a simple way to compute various norms of
 :py:class:`Vectors <dolfin.cpp.Vector>` and :py:class:`Functions
 <dolfin.functions.function.Function>`, including the standard
@@ -20,8 +21,6 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
 
-__all__ = ["norm", "errornorm"]
-
 import ufl
 from ufl import inner, grad, div, curl, dx, FiniteElement, VectorElement, Coefficient
 from math import sqrt
@@ -32,7 +31,11 @@ from dolfin.cpp import GenericVector, GenericFunction, Function, Mesh, error, Ve
 from dolfin.fem.assembling import assemble
 from dolfin.fem.interpolation import interpolate
 from dolfin.functions.function import Function
-from dolfin.functions.functionspace import FunctionSpace, VectorFunctionSpace, TensorFunctionSpace
+from dolfin.functions.functionspace import FunctionSpace, \
+    VectorFunctionSpace, TensorFunctionSpace
+
+__all__ = ["norm", "errornorm"]
+
 
 def norm(v, norm_type="L2", mesh=None):
     """
@@ -91,14 +94,17 @@ def norm(v, norm_type="L2", mesh=None):
     """
 
     if not isinstance(v, (GenericVector, GenericFunction)):
-        raise TypeError("expected a GenericVector or GenericFunction")
+        cpp.dolfin_error("norms.py",
+                         "compute norm",
+                         "expected a GenericVector or GenericFunction")
 
     # Check arguments
     if not isinstance(norm_type, str):
         cpp.dolfin_error("norms.py",
                          "compute norm",
-                         "Norm type must be a string, not " + str(type(norm_type)))
-    if mesh is not None and not isinstance(mesh, Mesh):
+                         "Norm type must be a string, not " +
+                         str(type(norm_type)))
+    if mesh is not None and not isinstance(mesh, cpp.Mesh):
         cpp.dolfin_error("norms.py",
                          "compute norm",
                          "Expecting a Mesh, not " + str(type(mesh)))
@@ -132,7 +138,8 @@ def norm(v, norm_type="L2", mesh=None):
         else:
             cpp.dolfin_error("norms.py",
                              "compute norm",
-                             "Unknown norm type (\"%s\") for functions" % str(norm_type))
+                             "Unknown norm type (\"%s\") for functions"
+                             % str(norm_type))
     else:
         cpp.dolfin_error("norms.py",
                          "compute norm",
@@ -151,6 +158,7 @@ def norm(v, norm_type="L2", mesh=None):
     else:
         return sqrt(r)
 
+
 def errornorm(u, uh, norm_type="l2", degree_rise=3, mesh=None):
     """
     Compute and return the error :math:`e = u - u_h` in the given norm.
@@ -222,11 +230,11 @@ def errornorm(u, uh, norm_type="l2", degree_rise=3, mesh=None):
                          "Missing mesh")
 
     # Get rank
-    if not u.shape() == uh.shape():
+    if not u.ufl_shape == uh.ufl_shape:
         cpp.dolfin_error("norms.py",
                          "compute error norm",
                          "Value shapes don't match")
-    shape = u.shape()
+    shape = u.ufl_shape
     rank = len(shape)
 
     # Check that uh is associated with a finite element
@@ -247,9 +255,11 @@ def errornorm(u, uh, norm_type="l2", degree_rise=3, mesh=None):
     if rank == 0:
         V = FunctionSpace(mesh, "Discontinuous Lagrange", degree)
     elif rank == 1:
-        V = VectorFunctionSpace(mesh, "Discontinuous Lagrange", degree, dim=shape[0])
+        V = VectorFunctionSpace(mesh, "Discontinuous Lagrange", degree,
+                                dim=shape[0])
     elif rank > 1:
-        V = TensorFunctionSpace(mesh, "Discontinuous Lagrange", degree, shape=shape)
+        V = TensorFunctionSpace(mesh, "Discontinuous Lagrange", degree,
+                                shape=shape)
 
     # Interpolate functions into finite element space
     pi_u = interpolate(u, V)
