@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 """This module handles the Expression class in Python.
 
 The Expression class needs special handling and is not mapped directly
@@ -53,7 +54,7 @@ create_compiled_expression_class.
 # along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Anders Logg, 2008-2009.
-# Modified by Martin S. Alnaes 2013-2014
+# Modified by Martin Sandve Alnæs 2013-2014
 
 from __future__ import print_function
 
@@ -70,6 +71,7 @@ from six import add_metaclass # Requires newer six version than some buildbots h
 
 # Import UFL and SWIG-generated extension module (DOLFIN C++)
 import ufl
+from ufl import product
 from ufl.utils.indexflattening import flatten_multiindex, shape_to_strides
 import dolfin.cpp as cpp
 import numpy
@@ -81,13 +83,15 @@ from dolfin.compilemodules.expressions import compile_expressions
 from functools import reduce
 from six.moves import xrange as range
 
+
 # Local help class
 class UserDefinedParameters(dict):
     """
     Help class to set user defined parameters in Expressions
     """
+
     def __init__(self, expr, **kwargs):
-        assert(isinstance(expr, cpp.Expression))
+        assert isinstance(expr, cpp.Expression)
         self._expr = expr
         dict.__init__(self, **kwargs)
 
@@ -107,45 +111,70 @@ class UserDefinedParameters(dict):
             for name, value in other:
                 self[name] = value
 
+
 def create_compiled_expression_class(cpp_base):
     # Check the cpp_base
-    assert(isinstance(cpp_base, type))
+    assert isinstance(cpp_base, type)
 
-    def __init__(self, cppcode, element=None, cell=None, domain=None, \
-                 degree=None, name=None, label=None, **kwargs):
+    def __init__(self, cppcode, *args, **kwargs):
         """Initialize the Expression """
         # Initialize the cpp base class first and extract value_shape
         cpp_base.__init__(self)
-        value_shape = tuple(self.value_dimension(i) \
+
+        element = kwargs.pop("element", None)
+        degree = kwargs.pop("degree", None)
+        cell = kwargs.pop("cell", None)
+        domain = kwargs.pop("domain", None)
+        name = kwargs.pop("name", None)
+        label = kwargs.pop("label", None)
+        mpi_commm = kwargs.pop("mpi_comm", None)
+
+        value_shape = tuple(self.value_dimension(i)
                             for i in range(self.value_rank()))
 
         # Store the value_shape
         self._value_shape = value_shape
 
+        # Some messy cell/domain handling for compatibility, will be
+        # straightened out later
+        if domain is None:
+            ufl_domain = None
+        else:
+            if isinstance(domain, ufl.domain.AbstractDomain):
+                ufl_domain = domain
+            else:
+                # Probably getting a Mesh here, from existing dolfin
+                # tests. Will be the same later anyway.
+                ufl_domain = domain.ufl_domain()
+            if cell is None:
+                cell = ufl_domain.ufl_cell()
+
+
         # Select an appropriate element if not specified
         if element is None:
-            element = _auto_select_element_from_shape(value_shape, degree, cell, domain)
+            element = _auto_select_element_from_shape(value_shape, degree, cell)
         else:
             # Check that we have an element
             if not isinstance(element, ufl.FiniteElementBase):
                 raise TypeError("The 'element' argument must be a UFL"\
-                      " finite element.")
+                                " finite element.")
 
-            # Check same value shape of compiled expression and passed element
-            if not element.value_shape() == value_shape:
-                raise ValueError("The value shape of the passed 'element',"\
-                      " is not equal to the value shape of the compiled "\
-                      "expression.")
+        # Check that compiled expression and passed element have the
+        # same value shape
+        if not element.value_shape() == value_shape:
+            raise ValueError("The value shape of the passed 'element',"\
+                             " is not equal to the value shape of the compiled "\
+                             "expression.")
 
         # Initialize UFL base class
-        self._ufl_element = element
-        ufl.Coefficient.__init__(self, self._ufl_element, count=self.id())
+        ufl_function_space = ufl.FunctionSpace(ufl_domain, element)
+        ufl.Coefficient.__init__(self, ufl_function_space, count=self.id())
 
         # Set default variables
         for member, value in list(kwargs.items()):
             setattr(self, member, value)
 
-        name = name or "f_" + str(ufl.Coefficient.count(self))
+        name = name or "f_" + str(self.count())
         label = label or "User defined expression"
 
         self.rename(name, label)
@@ -154,16 +183,17 @@ def create_compiled_expression_class(cpp_base):
         self.user_parameters = UserDefinedParameters(self, **kwargs)
 
     # Create and return the class
-    return type("CompiledExpression", (Expression, ufl.Coefficient, cpp_base),\
-                {"__init__":__init__})
+    return type("CompiledExpression", (Expression, ufl.Coefficient, cpp_base),
+                {"__init__": __init__})
+
 
 def create_python_derived_expression_class(class_name, user_bases, user_dict):
     """Return Expression class
 
-    This function is used to create all the dynamically created Expression
-    classes. It takes a class_name, and a compiled cpp.Expression and returns
-    a class that inherits the compiled class together with dolfin.Expression
-    and ufl.Coefficient.
+    This function is used to create all the dynamically created
+    Expression classes. It takes a class_name, and a compiled
+    cpp.Expression and returns a class that inherits the compiled
+    class together with dolfin.Expression and ufl.Coefficient.
 
     *Arguments*
         class_name
@@ -172,15 +202,16 @@ def create_python_derived_expression_class(class_name, user_bases, user_dict):
             User defined bases
         user_dict
             Dict with user specified function or attributes
+
     """
 
     # Check args
-    assert(isinstance(class_name, str))
-    assert(isinstance(user_bases, list))
-    assert(isinstance(user_dict, dict))
+    assert isinstance(class_name, str)
+    assert isinstance(user_bases, list)
+    assert isinstance(user_dict, dict)
 
     # Define the bases
-    assert(all([isinstance(t, (type, type)) for t in user_bases]))
+    assert all([isinstance(t, (type, type)) for t in user_bases])
     bases = tuple([Expression, ufl.Coefficient, cpp.Expression] + user_bases)
 
     user_init = user_dict.pop("__init__", None)
@@ -199,38 +230,51 @@ def create_python_derived_expression_class(class_name, user_bases, user_dict):
         domain = kwargs.get("domain", None)
         name = kwargs.get("name", None)
         label = kwargs.get("label", None)
+        mpi_comm = kwargs.get("mpi_comm", None)
 
-        # Check if user has passed too many arguments if no
-        # user_init is provided
+        # Check if user has passed too many arguments if no user_init
+        # is provided
         if user_init is None:
-            from operator import add
             # First count how many valid kwargs is passed
-            num_valid_kwargs = reduce(add, [kwarg is not None \
-                                      for kwarg in [element, degree, cell, domain, name, label]])
+            num_valid_kwargs = sum(1 if kwarg is not None else 0
+                                   for kwarg in [element, degree, cell, domain,
+                                                 name, label, mpi_comm])
             if len(kwargs) != num_valid_kwargs:
                 raise TypeError("expected only 'kwargs' from one of "\
-                      "'element', 'degree', 'cell', 'domain', 'name' or 'label'")
+                                "'element', 'degree', 'cell', 'domain', 'name', 'label', or 'mpi_comm'")
             if len(args) != 0:
                 raise TypeError("expected no 'args'")
 
+
+        # Some messy cell/domain handling for compatibility, will be
+        # straightened out later
+        if domain is None:
+            ufl_domain = None
+        else:
+            if isinstance(domain, ufl.domain.AbstractDomain):
+                ufl_domain = domain
+            else:
+                # Probably getting a Mesh here, from existing dolfin
+                # tests. Will be the same later anyway.
+                ufl_domain = domain.ufl_domain()
+            if cell is None:
+                cell = ufl_domain.ufl_cell()
+
+
         # Select an appropriate element if not specified
         if element is None:
             element = _auto_select_element_from_shape(self.value_shape(),
-                                                      degree, cell, domain)
-        elif isinstance(element, ufl.FiniteElementBase):
-            pass
+                                                      degree, cell)
         else:
-            raise TypeError("The 'element' argument must be a UFL finite"\
-                  " element.")
+            if not isinstance(element, ufl.FiniteElementBase):
+                raise TypeError("The 'element' argument must be a UFL finite element.")
 
         # Initialize cpp_base class
         cpp.Expression.__init__(self, list(element.value_shape()))
 
         # Initialize UFL base class
-        ufl.Coefficient.__init__(self, element, count=self.id())
-
-        # Why do we store the ufl element here in addition to in ufl.Coefficient? It's available as self.element()...
-        self._ufl_element = element
+        ufl_function_space = ufl.FunctionSpace(ufl_domain, element)
+        ufl.Coefficient.__init__(self, ufl_function_space, count=self.id())
 
         # Calling the user defined_init
         if user_init is not None:
@@ -245,18 +289,18 @@ def create_python_derived_expression_class(class_name, user_bases, user_dict):
     # NOTE: Why not?
 
     ## Collect reserved attributes from both cpp.Function and ufl.Coefficient
-    #reserved_attr = dir(ufl.Coefficient)
-    #reserved_attr.extend(dir(cpp.Function))
+    # reserved_attr = dir(ufl.Coefficient)
+    # reserved_attr.extend(dir(cpp.Function))
     #
-    ## Remove attributes that will be set by python
-    #for attr in ["__module__"]:
+    # # Remove attributes that will be set by python
+    # for attr in ["__module__"]:
     #    while attr in reserved_attr:
     #        reserved_attr.remove(attr)
     #
     ## Check the dict_ for reserved attributes
-    #for attr in reserved_attr:
+    # for attr in reserved_attr:
     #    if attr in dict_:
-    #        raise TypeError("The Function attribute '%s' is reserved by PyDOLFIN."%attr)
+    #        raise TypeError("The Function attribute '%s' is reserved by PyDOLFIN." % attr)
 
     # Add __init__ to the user_dict
     user_dict["__init__"]  = __init__
@@ -264,14 +308,15 @@ def create_python_derived_expression_class(class_name, user_bases, user_dict):
     # Create the class and return it
     return type(class_name, bases, user_dict)
 
+
 class ExpressionMetaClass(type):
     """Meta Class for Expression"""
     def __new__(mcs, class_name, bases, dict_):
         """Returns a new Expression class"""
 
-        assert(isinstance(class_name, str)), "Expecting a 'str'"
-        assert(isinstance(bases, tuple)), "Expecting a 'tuple'"
-        assert(isinstance(dict_, dict)), "Expecting a 'dict'"
+        assert isinstance(class_name, str), "Expecting a 'str'"
+        assert isinstance(bases, tuple), "Expecting a 'tuple'"
+        assert isinstance(dict_, dict), "Expecting a 'dict'"
 
         # First check if we are creating the Expression class
         if class_name == "Expression":
@@ -282,18 +327,21 @@ class ExpressionMetaClass(type):
             #        ...
             if len(bases) > 1 and bases[0] != object:
                 raise TypeError("Cannot name a subclass of Expression:"\
-                      " 'Expression'")
+                                " 'Expression'")
 
-            # Return the new class, which just is the original Expression defined in
-            # this module
+            # Return the new class, which just is the original
+            # Expression defined in this module
             return type.__new__(mcs, class_name, bases, dict_)
 
-        # If creating a fullfledged derived expression class, i.e, inheriting
-        # dolfin.Expression, ufl.Coefficient and cpp.Expression (or a subclass)
-        # then just return the new class.
-        if len(bases) >= 3 and bases[0] == Expression and \
-               bases[1] == ufl.Coefficient and issubclass(bases[2], \
-                                                          cpp.Expression):
+        # If creating a fullfledged derived expression class, i.e,
+        # inheriting dolfin.Expression, ufl.Coefficient and
+        # cpp.Expression (or a subclass) then just return the new
+        # class.
+        if (len(bases) >= 3
+            and bases[0] == Expression
+            and bases[1] == ufl.Coefficient
+            and issubclass(bases[2], cpp.Expression)):
+
             # Return the instantiated class
             return type.__new__(mcs, class_name, bases, dict_)
 
@@ -303,7 +351,8 @@ class ExpressionMetaClass(type):
         # remove Expression, to be added later
         user_bases.remove(Expression)
 
-        # Check the user has provided either an eval or eval_cell method
+        # Check the user has provided either an eval or eval_cell
+        # method
         if not ('eval' in dict_ or 'eval_cell' in dict_):
             raise TypeError("expected an overload 'eval' or 'eval_cell' method")
 
@@ -314,53 +363,51 @@ class ExpressionMetaClass(type):
 
         # Check type and number of arguments of user_eval function
         if not isinstance(user_eval, types.FunctionType):
-            raise TypeError("'%s' attribute must be a 'function'"%eval_name)
-        if eval_name == "eval" and not user_eval.__code__.co_argcount == 3:
+            raise TypeError("'%s' attribute must be a 'function'" % eval_name)
+        if eval_name == "eval" and user_eval.__code__.co_argcount != 3:
             raise TypeError("The overloaded '%s' function must use "\
-                  "three arguments"%eval_name)
-        if eval_name == "eval_cell" and \
-               not user_eval.__code__.co_argcount == 4:
+                            "three arguments" % eval_name)
+        if eval_name == "eval_cell" and user_eval.__code__.co_argcount != 4:
             raise TypeError("The overloaded '%s' function must "\
-                  "use three arguments"%eval_name)
+                            "use three arguments" % eval_name)
 
         return create_python_derived_expression_class(class_name, user_bases, dict_)
 
-#--- The user interface ---
+
+# --- The user interface ---
 
 # Places here so it can be reused in Function
 @add_metaclass(ExpressionMetaClass)
 class Expression(object):
-    """
-    This class represents a user-defined expression.
+    """This class represents a user-defined expression.
 
     Expressions can be used as coefficients in variational forms or
-    interpolated into finite element spaces.
+    interpolated into finite element spaces. An element or degree must
+    be provided.
 
     *Arguments*
         cppcode
            C++ argument, see below
         element
-            Optional element argument
+            Element argument (required if degree is not specified)
         degree
-            Optional element degree when element is not given.
+            Element degree when element is not given.
         cell
             Optional cell argument to used in code generation
         domain
             Optional argument to determine the geometric dimension
-        degree
-            Optional argument to determine the degree of the element
         name
             Optional argument to set name of the Variable
         label
             Optional argument to set label of Variable
-            
+
         mpi_comm
-            Optional argument to allow JIT compilation on mpi groups. The 
+            Optional argument to allow JIT compilation on mpi groups. The
             Expression is only available at ranks in the same group.
             The Expression is JIT compiled on the first rank of the group.
 
-    *1. Simple user-defined JIT-compiled expressions*
-        One may alternatively specify a C++ code for evaluation of the
+    *1. Simple user-defined JIT-compiled expressions* One may
+        alternatively specify a C++ code for evaluation of the
         Expression as follows:
 
         .. code-block:: python
@@ -375,7 +422,10 @@ class Expression(object):
         .. code-block:: python
 
             f2 = Expression((('exp(x[0])','sin(x[1])'),
-                            ('sin(x[0])','tan(x[1])')))
+                            ('sin(x[0])','tan(x[1])')), degree=2)
+
+        Here, the Expression will be interpolated using Lagrange
+        polynomials of degree 2 when used in a form.
 
         In general, a single string expression will be interpreted as
         a scalar, a tuple of strings as a tensor of rank 1 (a vector)
@@ -395,13 +445,13 @@ class Expression(object):
 
         The parameters can be scalars and any scalar valued
         GenericFunction and are all initialized to the passed default
-        value. The user defined parameters can be accessed and set
-        as attributes or via the dict-like user_parameters attribute:
+        value. The user defined parameters can be accessed and set as
+        attributes or via the dict-like user_parameters attribute:
 
         .. code-block:: python
 
             f.A = 5.0
-            f.B = Expression("value", value=6.0)
+            f.B = Expression("value", value=6.0, degree=1)
             f.user_parameters["A"] = 1.0
             f.user_parameters["B"] = Constant(5.0)
 
@@ -409,10 +459,10 @@ class Expression(object):
         value-type. So if a parameter is a float, it can only be
         updated with float.
 
-    *2. Complex user-defined JIT-compiled Expressions*
-        One may also define an Expression using more complicated logic
-        with the 'cppcode' argument. This argument should be a string
-        of C++ code that implements a class that inherits from
+    *2. Complex user-defined JIT-compiled Expressions* One may also
+        define an Expression using more complicated logic with the
+        'cppcode' argument. This argument should be a string of C++
+        code that implements a class that inherits from
         dolfin::Expression.
 
         The following code illustrates how to define an Expression
@@ -434,8 +484,8 @@ class Expression(object):
               {
               }
 
-            void eval(Array<double>& values, const Array<double>& x,
-                      const ufc::cell& c) const
+              void eval(Array<double>& values, const Array<double>& x,
+                        const ufc::cell& c) const
               {
                 assert(cell_data);
                 const Cell cell(*cell_data->mesh(), c.index);
@@ -454,7 +504,7 @@ class Expression(object):
             };'''
 
             cell_data = CellFunction('uint', V.mesh())
-            f = Expression(code)
+            f = Expression(code, degree=1)
             f.cell_data = cell_data
 
         While JIT compiling an Expression the public interface is
@@ -500,13 +550,15 @@ class Expression(object):
               };
             }'''
 
-            e = Expression(code)
+            e = Expression(code, degree=2)
 
-    *3. User-defined expressions by subclassing*
-        The user can subclass Expression and overload the 'eval'
-        function. The value_shape of such an Expression will default
-        to 0. If a user wants a vector or tensor Expression, the
-        value_shape method needs to be overloaded.
+    *3. User-defined expressions by subclassing* The user can subclass
+        Expression and overload the 'eval' function. The value_shape
+        of such an Expression will default to 0. If a user wants a
+        vector or tensor Expression, the value_shape method needs to
+        be overloaded. The user must provide at least the polynomial
+        degree of the finite element basis used when using the
+        Expression in a finite element form.
 
         .. code-block:: python
 
@@ -518,10 +570,10 @@ class Expression(object):
                     value[1] = 250.0*exp(-(dx*dx + dy*dy)/0.01)
                 def value_shape(self):
                     return (2,)
-            f0 = MyExpression0()
+            f0 = MyExpression0(degree=2)
 
         If a user wants to use the Expression in a UFL form and have
-        more controll in which finite element should be used to
+        more control in which finite element should be used to
         interpolate the expression in, the user can pass this
         information using the element kwarg:
 
@@ -544,7 +596,7 @@ class Expression(object):
                     else:
                         value[0] = -1.0
 
-            f2 = MyExpression1()
+            f2 = MyExpression1(degree=1)
 
         The ufc_cell object can be queried for the following data:
 
@@ -558,9 +610,9 @@ class Expression(object):
             ufc_cell.mesh_identifier
 
         The user can customize initialization of derived Expressions.
-        However, because of magic behind the scenes, a user needs to pass
-        optional arguments to __init__ using ``**kwargs``, and _not_
-        calling the base class __init__:
+        However, because of magic behind the scenes, a user needs to
+        pass optional arguments to __init__ using ``**kwargs``, and
+        _not_ calling the base class __init__:
 
         .. code-block:: python
 
@@ -571,7 +623,7 @@ class Expression(object):
                 def eval(self, values, x):
                     ...
 
-            f3 = MyExpression1(mesh=mesh, domain=domain)
+            f3 = MyExpression1(degree=0, mesh=mesh, domain=domain)
 
         Note that subclassing may be significantly slower than using
         JIT-compiled expressions. This is because a callback from C++
@@ -580,12 +632,11 @@ class Expression(object):
 
     """
 
-    def __new__(cls, cppcode=None, element=None, cell=None, domain=None, degree=None, \
-                name=None, label=None, mpi_comm=None, **kwargs):
+    def __new__(cls, cppcode=None, *args, **kwargs):
 
-        # If the __new__ function is called because we are instantiating
-        # a python sub class of Expression, then just return a new instant
-        # of the passed class
+        # If the __new__ function is called because we are
+        # instantiating a python sub class of Expression, then just
+        # return a new instance of the passed class
         if cls.__name__ != "Expression":
             return object.__new__(cls)
 
@@ -598,14 +649,15 @@ class Expression(object):
 
         # Compile module and get the cpp.Expression class
         cpp_base, members = compile_expressions([cppcode],
-                                                [generic_function_members], \
-                                                mpi_comm=mpi_comm)
+                                                [generic_function_members],
+                                                mpi_comm=kwargs.get("mpi_comm"))
         cpp_base, members = cpp_base[0], members[0]
 
         # Check passed default arguments
-        not_allowed = [n for n in dir(cls) if n[0] !="_"]
+        not_allowed = [n for n in dir(cls) if n[0] != "_"]
         not_allowed += ["cppcode", "user_parameters"]
-        _check_kwargs(members, kwargs, not_allowed)
+        valid_kws = ["element", "cell", "domain", "degree", "name", "label", "mpi_comm"]
+        _check_kwargs(members, kwargs, not_allowed, valid_kws)
 
         # Store compile arguments for later use
         cpp_base.cppcode = cppcode
@@ -615,16 +667,14 @@ class Expression(object):
 
     # This method is only included so a user can check what arguments
     # one should use in IPython using tab completion
-    def __init__(self, cppcode=None, element=None, cell=None, domain=None, degree=None, \
-                 name=None, label=None, mpi_comm=None, **kwargs): pass
+    def __init__(self, cppcode=None, element=None, cell=None, domain=None,
+                 degree=None, name=None, label=None, mpi_comm=None, **kwargs):
+        cpp.dolfin_error("expression.py", "initialize Expression",
+                         "Calling 'Expression.__init__' has no effect")
 
     # Reuse the docstring from __new__
     __init__.__doc__ = __new__.__doc__
 
-    def ufl_element(self):
-        "Return the ufl FiniteElement."
-        return self._ufl_element
-
     def __str__(self):
         "x.__str__() <==> print(x)"
         return self.name()
@@ -651,9 +701,9 @@ class Expression(object):
         assert derivatives == () # TODO: Handle derivatives
 
         if component:
-            shape = self.shape()
+            shape = self.ufl_shape
             assert len(shape) == len(component)
-            value_size = ufl.common.product(shape)
+            value_size = product(shape)
             index = flatten_multiindex(component, shape_to_strides(shape))
             values = numpy.zeros(value_size)
             self(*x, values=values)
@@ -721,38 +771,39 @@ class Expression(object):
             raise TypeError("expected at least 1 argument")
 
         # Test for ufl restriction
-        if len(args) == 1 and args[0] in ('+','-'):
-            return ufl.Coefficient.__call__(self, *args)
+        if len(args) == 1 and isinstance(args[0], str):
+            if args[0] in ('+', '-'):
+                return ufl.Coefficient.__call__(self, *args)
 
         # Test for ufl mapping
         if len(args) == 2 and isinstance(args[1], dict) and self in args[1]:
             return ufl.Coefficient.__call__(self, *args)
 
         # Some help variables
-        value_size = ufl.common.product(self.ufl_element().value_shape())
+        value_size = product(self.ufl_element().value_shape())
 
         # If values (return argument) is passed, check the type and length
         values = kwargs.get("values", None)
         if values is not None:
             if not isinstance(values, numpy.ndarray):
                 raise TypeError("expected a NumPy array for 'values'")
-            if len(values) != value_size or \
-                   not numpy.issubdtype(values.dtype, 'd'):
+            if len(values) != value_size or not numpy.issubdtype(values.dtype, 'd'):
                 raise TypeError("expected a double NumPy array of length"\
-                      " %d for return values."%value_size)
+                                " %d for return values." % value_size)
             values_provided = True
         else:
             values_provided = False
             values = numpy.zeros(value_size, dtype='d')
 
         # Get dim if element has any domains
-        domains = self.ufl_element().domains()
-        dim = domains[0].geometric_dimension() if domains else None
+        cell = self.ufl_element().cell()
+        dim = None if cell is None else cell.geometric_dimension()
 
         # Assume all args are x argument
         x = args
 
-        # If only one x argument has been provided, unpack it if it's an iterable
+        # If only one x argument has been provided, unpack it if it's
+        # an iterable
         if len(x) == 1:
             if isinstance(x[0], cpp.Point):
                 if dim is not None:
@@ -773,7 +824,12 @@ class Expression(object):
             raise TypeError("coordinate argument too short")
 
         if dim is None:
-            warning("Evaluating an Expression without knowing the right geometric dimension, assuming %d is correct." % len(x))
+            # Disabled warning as it breaks py.test due to excessive
+            # output, and that code that is warned about is still
+            # officially supported. See
+            # https://bitbucket.org/fenics-project/dolfin/issues/355/
+            # warning("Evaluating an Expression without knowing the right geometric dimension, assuming %d is correct." % len(x))
+            pass
         else:
             if len(x) != dim:
                 raise TypeError("expected the geometry argument to be of "\
@@ -788,7 +844,8 @@ class Expression(object):
 
         return values
 
-#--- Utility functions ---
+
+# --- Utility functions ---
 
 def _check_cppcode(cppcode):
     "Check that cppcode makes sense"
@@ -796,39 +853,37 @@ def _check_cppcode(cppcode):
     # Check that we get a string expression or nested expression
     if not isinstance(cppcode, (str, tuple, list)):
         raise TypeError("Please provide a 'str', 'tuple' or 'list' for "\
-              "the 'cppcode' argument.")
+                        "the 'cppcode' argument.")
 
-def _auto_select_element_from_shape(shape, degree=None, cell=None, domain=None):
+
+def _auto_select_element_from_shape(shape, degree=None, cell=None):
     "Automatically select an appropriate element from cppcode."
 
-    # Cell to domain conversion TODO: deprecate cell argument
-    if domain is None:
-        if cell is None:
-            pass # No domain will result in undefined element
-        else:
-            domain = ufl.as_domain(cell)
-    else:
-        if cell is None:
-            domain = ufl.as_domain(domain) # Got only domain, good!
-        else:
-            raise ValueError("Got both domain and cell, use only domain.")
+    if degree is None:
+        cpp.deprecation("Automatic determination of degree for Expressions",
+                        "2016.1",
+                        "Specify the polynomial degree for interpolation of Expression, or provide a FiniteElement")
 
     # Default element, change to quadrature when working
-    Family = "Lagrange"
+    if degree == 0:
+        Family = "Discontinuous Lagrange"
+    else:
+        Family = "Lagrange"
 
     # Check if scalar, vector or tensor valued
     if len(shape) == 0:
-        element = ufl.FiniteElement(Family, domain, degree)
+        element = ufl.FiniteElement(Family, cell, degree)
     elif len(shape) == 1:
-        element = ufl.VectorElement(Family, domain, degree, dim=shape[0])
+        element = ufl.VectorElement(Family, cell, degree, dim=shape[0])
     else:
-        element = ufl.TensorElement(Family, domain, degree, shape=shape)
+        element = ufl.TensorElement(Family, cell, degree, shape=shape)
 
     cpp.debug("Automatic selection of expression element: " + str(element))
 
     return element
 
-def _check_kwargs(members, kwargs, not_allowed):
+
+def _check_kwargs(members, kwargs, not_allowed, valid):
     """
     Check that all kwargs passed is either scalars or scalar Constants,
     and that the name is allowed
@@ -844,13 +899,13 @@ def _check_kwargs(members, kwargs, not_allowed):
         raise RuntimeError("expected a default value to all member "\
                            "variables in the Expression. Missing: %s." % missing)
 
-    for name in list(kwargs.keys()):
+    for name, value in kwargs.items():
+        if name in valid:
+            continue
         if name in not_allowed:
             raise RuntimeError("Parameter name: '%s' is not allowed. It is "\
                                "part of the interface of Expression" % name)
-
-    if not (all(isinstance(value, (int, float)) or \
-                (isinstance(value, cpp.GenericFunction) and value.value_size()==1) \
-                for value in list(kwargs.values()))):
-        raise TypeError("expected default arguments for member variables "\
-              "to be scalars or a scalar GenericFunctions.")
+        if not (isinstance(value, (int, float)) or
+                (isinstance(value, cpp.GenericFunction) and value.value_size() == 1)):
+            raise TypeError("expected default arguments for member variables "\
+                            "to be scalars or a scalar GenericFunctions.")
