@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 """This module handles the Function class in Python.
 """
 # Copyright (C) 2009-2014 Johan Hake
@@ -17,7 +18,8 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
 #
-# Modified by Martin S. Alnaes 2013-2014
+# Modified by Martin Sandve Alnæs 2013-2014
+# Modified by Anders Logg 2015
 
 __all__ = ["Function", "TestFunction", "TrialFunction", "Argument",
            "TestFunctions", "TrialFunctions"]
@@ -26,11 +28,12 @@ import types
 
 # Import UFL and SWIG-generated extension module (DOLFIN C++)
 import ufl
+from ufl import product
 from ufl.utils.indexflattening import flatten_multiindex, shape_to_strides
 import dolfin.cpp as cpp
 import numpy
 
-from dolfin.functions.functionspace import FunctionSpaceBase
+from dolfin.functions.functionspace import FunctionSpace
 from dolfin.functions.constant import Constant
 from six.moves import xrange as range
 
@@ -48,28 +51,30 @@ def _check_mul_and_division(e, linear_comb, scalar_weight=1.0, multi_index=None)
     from ufl.constantvalue import ScalarValue
     from ufl.classes import ComponentTensor, MultiIndex, Indexed
     from ufl.algebra import Division, Product, Sum
-    #ops = e.operands()
+    #ops = e.ufl_operands
 
     # FIXME: What should be checked!?
-    same_multi_index = lambda x, y: len(x.free_indices()) == len(y.free_indices()) \
-                and list(x.index_dimensions().values()) == list(y.index_dimensions().values())
+    # martinal: This code has never done anything sensible,
+    #   but I don't know what it was supposed to do so I can't fix it.
+    #same_multi_index = lambda x, y: (x.ufl_free_indices == y.ufl_free_indices \
+    #                        and x.ufl_index_dimensions == y.ufl_index_dimensions)
 
-    assert(isinstance(scalar_weight, float))
+    assert isinstance(scalar_weight, float)
 
     # Split passed expression into scalar and expr
     if isinstance(e, Product):
-        for i, op in enumerate(e.operands()):
+        for i, op in enumerate(e.ufl_operands):
             if isinstance(op, ScalarValue) or \
                    (isinstance(op, Constant) and op.value_size()==1):
                 scalar = op
-                expr = e.operands()[1-i]
+                expr = e.ufl_operands[1-i]
                 break
         else:
             _assign_error()
 
         scalar_weight *= float(scalar)
     elif isinstance(e, Division):
-        expr, scalar = e.operands()
+        expr, scalar = e.ufl_operands
         if not (isinstance(scalar, ScalarValue) or \
                 isinstance(scalar, Constant) and scalar.value_rank()==1):
             _assign_error()
@@ -81,14 +86,14 @@ def _check_mul_and_division(e, linear_comb, scalar_weight=1.0, multi_index=None)
     # a Function or another ComponentTensor, where the latter wil
     # result in a recursive call
     if multi_index is not None:
-        assert(isinstance(multi_index, MultiIndex))
-        assert(isinstance(expr, Indexed))
+        assert isinstance(multi_index, MultiIndex)
+        assert isinstance(expr, Indexed)
 
         # Unpack Indexed and check equality with passed multi_index
-        expr, multi_index2 = expr.operands()
-        assert(isinstance(multi_index2, MultiIndex))
-        if not same_multi_index(multi_index, multi_index2):
-            _assign_error()
+        expr, multi_index2 = expr.ufl_operands
+        assert isinstance(multi_index2, MultiIndex)
+        #if not same_multi_index(multi_index, multi_index2):
+        #    _assign_error()
 
     if isinstance(expr, Function):
         linear_comb.append((expr, scalar_weight))
@@ -96,16 +101,14 @@ def _check_mul_and_division(e, linear_comb, scalar_weight=1.0, multi_index=None)
     elif isinstance(expr, (ComponentTensor, Product, Division, Sum)):
         # If componentTensor we need to unpack the MultiIndices
         if isinstance(expr, ComponentTensor):
-            expr, multi_index = expr.operands()
-            if not same_multi_index(multi_index, multi_index2):
-                _error()
+            expr, multi_index = expr.ufl_operands
+            #if not same_multi_index(multi_index, multi_index2):
+            #    _error()
 
         if isinstance(expr, (Product, Division)):
-            linear_comb = _check_mul_and_division(expr, linear_comb, \
-                                                  scalar_weight, multi_index)
+            linear_comb = _check_mul_and_division(expr, linear_comb, scalar_weight, multi_index)
         elif isinstance(expr, Sum):
-            linear_comb = _check_and_extract_functions(expr, linear_comb, \
-                                                       scalar_weight, multi_index)
+            linear_comb = _check_and_extract_functions(expr, linear_comb, scalar_weight, multi_index)
         else:
             _assign_error()
     else:
@@ -135,13 +138,13 @@ def _check_and_extract_functions(e, linear_comb=None, scalar_weight=1.0,
     # Third check a*u*b, u/a/b, a*u/b where a and b are scalars and u
     # is a Tensor
     elif isinstance(e, ComponentTensor):
-        e, multi_index = e.operands()
+        e, multi_index = e.ufl_operands
         linear_comb = _check_mul_and_division(e, linear_comb, scalar_weight, multi_index)
         return linear_comb
 
     # If not Product or Division we expect Sum
     elif isinstance(e, Sum):
-        for op in e.operands():
+        for op in e.ufl_operands:
             linear_comb = _check_and_extract_functions(op, linear_comb, \
                                                        scalar_weight, multi_index)
 
@@ -213,7 +216,10 @@ class Function(ufl.Coefficient, cpp.Function):
 
               f = Function(V)
 
-        - from another Function ``f``
+        - from cpp.Function ``f``
+
+          *Warning: this constructor is intended for internal libray use only.*
+          No copying is done - ``f`` is only wrapped as Function.
 
           .. code-block:: python
 
@@ -223,7 +229,7 @@ class Function(ufl.Coefficient, cpp.Function):
           <dolfin.functions.functionspace.FunctionSpace>` ``V`` and a
           :py:class:`GenericVector <dolfin.cpp.GenericVector>` ``v``
 
-          *Warning: this constructor is intended for internal libray use only*
+          *Warning: this constructor is intended for internal libray use only.*
 
           .. code-block:: python
 
@@ -242,7 +248,8 @@ class Function(ufl.Coefficient, cpp.Function):
 
     def __init__(self, *args, **kwargs):
         """Initialize Function."""
-        # Initial quick check for valid arguments (other checks sprinkled below)
+        # Initial quick check for valid arguments (other checks
+        # sprinkled below)
         if len(args) == 0:
             raise TypeError("expected 1 or more arguments")
 
@@ -250,6 +257,10 @@ class Function(ufl.Coefficient, cpp.Function):
         if isinstance(args[0], Function):
             other = args[0]
             if len(args) == 1:
+                # NOTE: Turn this into error when removing deprecation
+                # warning
+                cpp.deprecation("Function copy constructor", "2016.1",
+                                "Use 'Function.copy(deepcopy=True)' for copying.")
                 self.__init_copy_constructor(other)
             elif len(args) == 2:
                 i = args[1]
@@ -263,11 +274,11 @@ class Function(ufl.Coefficient, cpp.Function):
             other = args[0]
             if len(args) == 1:
                 # If creating a dolfin.Function from a cpp.Function
-                self.__init_cpp_copy_constructor(other)
+                self.__init_from_cpp_function(other)
             else:
                 raise TypeError("expected only one argument when passing cpp.Function"
                                 "to dolfin.Function constructor")
-        elif isinstance(args[0], FunctionSpaceBase):
+        elif isinstance(args[0], FunctionSpace):
             V = args[0]
             # If initialising from a FunctionSpace
             if len(args) == 1:
@@ -293,16 +304,15 @@ class Function(ufl.Coefficient, cpp.Function):
 
     def __init_copy_constructor(self, other):
         cpp.Function.__init__(self, other)
-        ufl.Coefficient.__init__(self, other._element, count=self.id())
+        ufl.Coefficient.__init__(self, other.ufl_function_space(), count=self.id())
 
-    def __init_cpp_copy_constructor(self, other):
-        # Lets be agressive in abusing dynamic typing shall we...
-        self.__class__ = Function
+    def __init_from_cpp_function(self, other):
+        # Assign all the members (including 'this' pointer to SWIG wraper)
+        # NOTE: This in fact performs assignment of C++ context
         self.__dict__ = other.__dict__
 
-        # (Not calling cpp.Function.__init__)
-        ufl.Coefficient.__init__(self, other.function_space().ufl_element(), \
-                                 count=self.id())
+        # Initialize the ufl.FunctionSpace (Not calling cpp.Function.__init__)
+        ufl.Coefficient.__init__(self, other.function_space().ufl_function_space(), count=self.id())
 
     def __init_subfunction_constructor(self, other, i):
         num_sub_spaces = other.function_space().num_sub_spaces()
@@ -311,26 +321,23 @@ class Function(ufl.Coefficient, cpp.Function):
         if not i < num_sub_spaces:
             raise RuntimeError("Can only extract subfunctions "
                                "with i = 0..%d"% num_sub_spaces)
-
         cpp.Function.__init__(self, other, i)
-        ufl.Coefficient.__init__(self, self.function_space().ufl_element(), \
-                                 count=self.id())
+        ufl.Coefficient.__init__(self, self.function_space().ufl_function_space(), count=self.id())
 
     def __init_from_function_space(self, V):
         cpp.Function.__init__(self, V)
-        ufl.Coefficient.__init__(self, V.ufl_element(), count=self.id())
+        ufl.Coefficient.__init__(self, V.ufl_function_space(), count=self.id())
 
     def __init_from_function_space_and_cpp_function(self, V, other):
         # Simple consistency checks on function spaces
         if other.function_space().dim() != V.dim():
             raise ValueError("non matching dimensions on passed FunctionSpaces")
-
         cpp.Function.__init__(self, other)
-        ufl.Coefficient.__init__(self, V.ufl_element(), count=self.id())
+        ufl.Coefficient.__init__(self, V.ufl_function_space(), count=self.id())
 
     def __init_from_function_space_and_function(self, V, other):
         cpp.Function.__init__(self, V, other)
-        ufl.Coefficient.__init__(self, V.ufl_element(), count=self.id())
+        ufl.Coefficient.__init__(self, V.ufl_function_space(), count=self.id())
 
     def sub(self, i, deepcopy = False):
         """
@@ -380,7 +387,7 @@ class Function(ufl.Coefficient, cpp.Function):
             self._assign(rhs)
         elif isinstance(rhs, (Sum, Product, Division, ComponentTensor)):
             if isinstance(rhs, ComponentTensor):
-                rhs, multi_index = rhs.operands()
+                rhs, multi_index = rhs.ufl_operands
             else:
                 multi_index = None
             linear_comb = _check_and_contract_linear_comb(rhs, self, \
@@ -421,10 +428,7 @@ class Function(ufl.Coefficient, cpp.Function):
         Extract any sub functions.
 
         A sub function can be extracted from a discrete function that
-        is in a :py:class:`MixedFunctionSpace
-        <dolfin.functions.functionspace.MixedFunctionSpace>` or in a
-        :py:class:`VectorFunctionSpace
-        <dolfin.functions.functionspace.VectorFunctionSpace>`. The sub
+        is in a mixed, vector, or tensor FunctionSpace. The sub
         function resides in the subspace of the mixed space.
 
         *Arguments*
@@ -438,10 +442,6 @@ class Function(ufl.Coefficient, cpp.Function):
             raise RuntimeError("No subfunctions to extract")
         return tuple(self.sub(i, deepcopy) for i in range(num_sub_spaces))
 
-    def ufl_element(self):
-        """Return ufl element"""
-        return self._element
-
     def __str__(self):
         """Return a pretty print representation of it self.
         """
@@ -465,9 +465,9 @@ class Function(ufl.Coefficient, cpp.Function):
         assert derivatives == () # TODO: Handle derivatives
 
         if component:
-            shape = self.shape()
+            shape = self.ufl_shape
             assert len(shape) == len(component)
-            value_size = ufl.common.product(shape)
+            value_size = product(shape)
             index = flatten_multiindex(component, shape_to_strides(shape))
             values = numpy.zeros(value_size)
             self(*x, values=values)
@@ -477,12 +477,14 @@ class Function(ufl.Coefficient, cpp.Function):
             return self(*x)
 
     def __float__(self):
-        if self.shape() != ():
+        if self.ufl_shape != ():
             raise RuntimeError("Cannot convert nonscalar function to float.")
         elm = self.ufl_element()
         if elm.family() != "Real":
             raise RuntimeError("Cannot convert spatially varying function to float.")
-        # Gather value directly from vector in a parallell safe way
+        # FIXME: This could be much simpler be exploiting that the
+        # vector is ghosted
+        # Gather value directly from vector in a parallel safe way
         vec = self.vector()
         indices = numpy.zeros(1, dtype=cpp.la_index_dtype())
         values = vec.gather(indices)
@@ -546,15 +548,16 @@ class Function(ufl.Coefficient, cpp.Function):
             raise TypeError("expected at least 1 argument")
 
         # Test for ufl restriction
-        if len(args) == 1 and args[0] in ('+','-'):
-            return ufl.Coefficient.__call__(self, *args)
+        if len(args) == 1 and isinstance(args[0], str):
+            if args[0] in ('+', '-'):
+                return ufl.Coefficient.__call__(self, *args)
 
         # Test for ufl mapping
         if len(args) == 2 and isinstance(args[1], dict) and self in args[1]:
             return ufl.Coefficient.__call__(self, *args)
 
         # Some help variables
-        value_size = ufl.common.product(self.ufl_element().value_shape())
+        value_size = product(self.ufl_element().value_shape())
 
         # If values (return argument) is passed, check the type and length
         values = kwargs.get("values", None)
@@ -571,7 +574,7 @@ class Function(ufl.Coefficient, cpp.Function):
             values = numpy.zeros(value_size, dtype='d')
 
         # Get the geometric dimension we live in
-        dim = self.geometric_dimension() # (UFL function)
+        dim = self.ufl_domain().geometric_dimension()
 
         # Assume all args are x argument
         x = args
@@ -608,6 +611,7 @@ class Function(ufl.Coefficient, cpp.Function):
 
 #--- Subclassing of ufl.{Basis, Trial, Test}Function ---
 
+# TODO: Update this message to clarify dolfin.FunctionSpace vs ufl.FunctionSpace
 _ufl_dolfin_difference_message = """\
 When constructing an Argument, TestFunction or TrialFunction,
 you must to provide a FunctionSpace and not a FiniteElement.
@@ -624,13 +628,23 @@ class Argument(ufl.Argument):
     This is the overloaded PyDOLFIN variant.
     """
     def __init__(self, V, number, part=None):
-        if not isinstance(V, FunctionSpaceBase):
-            if isinstance(V, ufl.FiniteElementBase):
+
+        # Check argument
+        if not isinstance(V, (FunctionSpace, cpp.MultiMeshFunctionSpace)):
+            if isinstance(V, (ufl.FiniteElementBase, ufl.FunctionSpace)):
                 raise TypeError(_ufl_dolfin_difference_message)
             else:
                 raise TypeError("Illegal argument for creation of Argument, not a FunctionSpace: " + str(V))
             raise TypeError("Illegal argument for creation of Argument, not a FunctionSpace: " + str(V))
-        ufl.Argument.__init__(self, V.ufl_element(), number, part)
+
+        # Handle MultiMesh
+        if isinstance(V, cpp.MultiMeshFunctionSpace):
+            self._V_multi = V
+            V = V._parts[0]
+
+        # Initialize UFL Argument
+        ufl.Argument.__init__(self, V.ufl_function_space(), number, part)
+
         self._V = V
 
     def function_space(self):
