@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 """ Module for converting various mesh formats."""
 
 # Copyright (C) 2006 Anders Logg
@@ -84,15 +85,13 @@ def mesh2xml(ifilename, ofilename):
     """Convert between .mesh and .xml, parser implemented as a
     state machine:
 
-        0 = read 'Dimension'
-        1 = read dimension
-        2 = read 'Vertices'
-        3 = read number of vertices
-        4 = read next vertex
-        5 = read 'Triangles' or 'Tetrahedra'
-        6 = read number of cells
-        7 = read next cell
-        8 = done
+        0 = read 'Vertices'
+        1 = read number of vertices
+        2 = read next vertex
+        3 = read 'Triangles' or 'Tetrahedra'
+        4 = read number of cells
+        5 = read next cell
+        6 = done
 
     """
 
@@ -112,11 +111,14 @@ def mesh2xml(ifilename, ofilename):
         if not line: break
 
         # Remove newline
-        line = line.rstrip("\n\r")
+        line = line.strip(" \n\r").split(" ")
 
-        # Read dimension
-        if  line == "Dimension" or line == " Dimension":
-            line = ifile.readline()
+        # Read dimension either on same line or following line
+        if  line[0] == "Dimension":
+            if (len(line) == 2):
+                line = line[1]
+            else:
+                line = ifile.readline()
             num_dims = int(line)
             if num_dims == 2:
                 cell_type = "triangle"
@@ -157,19 +159,13 @@ def mesh2xml(ifilename, ofilename):
         line = line.rstrip("\n\r")
 
         if state == 0:
-            if line == "Dimension" or line == " Dimension":
-                state += 1
-        elif state == 1:
-            num_dims = int(line)
-            state +=1
-        elif state == 2:
             if line == "Vertices" or line == " Vertices":
                 state += 1
-        elif state == 3:
+        elif state == 1:
             num_vertices = int(line)
             xml_writer.write_header_vertices(ofile, num_vertices)
             state +=1
-        elif state == 4:
+        elif state == 2:
             if num_dims == 2:
                 (x, y, tmp) = line.split()
                 x = float(x)
@@ -185,16 +181,16 @@ def mesh2xml(ifilename, ofilename):
             if num_vertices == num_vertices_read:
                 xml_writer.write_footer_vertices(ofile)
                 state += 1
-        elif state == 5:
+        elif state == 3:
             if (line == "Triangles"  or line == " Triangles") and num_dims == 2:
                 state += 1
             if line == "Tetrahedra" and num_dims == 3:
                 state += 1
-        elif state == 6:
+        elif state == 4:
             num_cells = int(line)
             xml_writer.write_header_cells(ofile, num_cells)
             state +=1
-        elif state == 7:
+        elif state == 5:
             if num_dims == 2:
                 (n0, n1, n2, tmp) = line.split()
                 n0 = int(n0) - 1
@@ -212,11 +208,11 @@ def mesh2xml(ifilename, ofilename):
             if num_cells == num_cells_read:
                 xml_writer.write_footer_cells(ofile)
                 state += 1
-        elif state == 8:
+        elif state == 6:
             break
 
     # Check that we got all data
-    if state == 8:
+    if state == 6:
         print("Conversion done")
     else:
         _error("Missing data, unable to convert")
