@@ -31,7 +31,7 @@ from dolfin_utils.test import * #cd_tempdir, pushpop_parameters, skip_in_paralle
 
 def create_data(A=None):
     "This function creates data used in the tests below"
-    mesh = UnitSquareMesh(3, 3)
+    mesh = UnitSquareMesh(4, 4)
     V = FunctionSpace(mesh, "Lagrange", 1)
     u = TrialFunction(V)
     v = TestFunction(V)
@@ -388,7 +388,7 @@ def test_p21_box_1():
     mesh = UnitSquareMesh(3, 3)
     V = VectorFunctionSpace(mesh, "Lagrange", 2)
     Q = FunctionSpace(mesh, "Lagrange", 1)
-    W = V*Q
+    #W = V*Q # deprecated!
 
 
 @use_gc_barrier
@@ -426,9 +426,9 @@ def test_p22_box_2():
 @use_gc_barrier
 def test_p23_box_1():
     mesh = UnitSquareMesh(3, 3)
-    V = VectorFunctionSpace(mesh, "Lagrange", 2)
-    Q = FunctionSpace(mesh, "Lagrange", 1)
-    W = V*Q
+    V = VectorElement("Lagrange", mesh.ufl_cell(), 2)
+    Q = FiniteElement("Lagrange", mesh.ufl_cell(), 1)
+    W = FunctionSpace(mesh, V*Q)
 
     w = Function(W)
     u, p = w.split()
@@ -441,7 +441,7 @@ def test_p24_box_1():
     class MyExpression(Expression):
         def eval(self, values, x):
             values[0] = sin(x[0])*cos(x[1])
-    f = MyExpression()
+    f = MyExpression(degree=1)
     print(f((0.5, 0.5)))
 
 
@@ -453,14 +453,14 @@ def test_p24_box_2():
             values[1] = cos(x[1])
         def value_shape(self):
             return (2,)
-    g = MyExpression()
+    g = MyExpression(degree=1)
     print(g((0.5, 0.5)))
 
 
 @use_gc_barrier
 def test_p24_box_3():
-    f = Expression("sin(x[0])*cos(x[1])")
-    g = Expression(("sin(x[0])", "cos(x[1])"))
+    f = Expression("sin(x[0])*cos(x[1])", degree=1)
+    g = Expression(("sin(x[0])", "cos(x[1])"), degree=1)
 
 
 @use_gc_barrier
@@ -469,7 +469,7 @@ def test_p25_box_1():
     T = 1.0
     dt = 0.5
 
-    h = Expression("t*sin(x[0])*cos(x[1])", t=0.0)
+    h = Expression("t*sin(x[0])*cos(x[1])", t=0.0, degree=1)
     while t < T:
         h.t = t
         t += dt
@@ -481,7 +481,7 @@ def test_p26_box_1():
     V = VectorFunctionSpace(mesh, "Lagrange", 1)
     u = TrialFunction(V)
     v = TestFunction(V)
-    f = Expression(("0.0", "0.0"))
+    f = Expression(("0.0", "0.0"), degree=0)
 
     E = 10.0
     nu = 0.3
@@ -537,7 +537,8 @@ def test_p30_box_1():
             return x[0] < 0.5 + DOLFIN_EPS and on_boundary
     neumann_boundary = NeumannBoundary()
 
-    dss = ds[neumann_boundary]
+    # Changed from notation dss = ds[neumann_boundary]
+    dss = ds(subdomain_data=neumann_boundary)
     a = g*v*dss(0) + h*v*dss(1)
 
 
@@ -551,7 +552,7 @@ def test_p30_box_2():
     class DirichletBoundary(SubDomain):
         def inside(self, x, on_boundary):
             return x[0] > 0.5 - DOLFIN_EPS and on_boundary
-    u_0 = DirichletValue()
+    u_0 = DirichletValue(degree=2)
     Gamma_D = DirichletBoundary()
     bc = DirichletBC(V, u_0, Gamma_D)
 
@@ -560,7 +561,7 @@ def test_p30_box_2():
 def test_p31_box_1():
     mesh = UnitSquareMesh(3, 3)
     V = FunctionSpace(mesh, "CG", 1)
-    u_0 = Expression("sin(x[0])")
+    u_0 = Expression("sin(x[0])", degree=1)
     bc = DirichletBC(V, u_0, "x[0] > 0.5 && on_boundary")
 
 
@@ -569,7 +570,7 @@ def test_p31_box_2():
     mesh = UnitSquareMesh(3, 3)
     V = FunctionSpace(mesh, "CG", 1)
     A, x, b = create_data()
-    u_0 = Expression("sin(x[0])")
+    u_0 = Expression("sin(x[0])", degree=1)
     bc = DirichletBC(V, u_0, "x[0] > 0.5 && on_boundary")
     u = Function(V)
 
@@ -581,7 +582,7 @@ def test_p31_box_2():
 def test_p32_box_1():
     mesh = UnitSquareMesh(3, 3)
     V = FunctionSpace(mesh, "CG", 1)
-    u_0 = Expression("sin(x[0])")
+    u_0 = Expression("sin(x[0])", degree=1)
     bc0 = DirichletBC(V, u_0, "x[0] > 0.5 && on_boundary")
     bc1 = DirichletBC(V, u_0, "x[0] > 0.5 && on_boundary")
     bc2 = DirichletBC(V, u_0, "x[0] > 0.5 && on_boundary")
@@ -599,8 +600,8 @@ def test_p32_box_1():
 def test_p32_box_2():
     mesh = UnitSquareMesh(3, 3)
     V = FunctionSpace(mesh, "CG", 1)
-    f = Expression("0.0")
-    u_0 = Expression("sin(x[0])")
+    f = Expression("0.0", degree=0)
+    u_0 = Expression("sin(x[0])", degree=1)
     bc0 = DirichletBC(V, u_0, "x[0] > 0.5 && on_boundary")
     bc1 = DirichletBC(V, u_0, "x[0] > 0.5 && on_boundary")
     bc2 = DirichletBC(V, u_0, "x[0] > 0.5 && on_boundary")
@@ -621,8 +622,8 @@ def test_p32_box_2():
 def test_p32_box_3():
     mesh = UnitSquareMesh(3, 3)
     V = FunctionSpace(mesh, "CG", 1)
-    f = Expression("0.0")
-    u_0 = Expression("sin(x[0])")
+    f = Expression("0.0", degree=0)
+    u_0 = Expression("sin(x[0])", degree=1)
     bc0 = DirichletBC(V, u_0, "x[0] > 0.5 && on_boundary")
     bc1 = DirichletBC(V, u_0, "x[0] > 0.5 && on_boundary")
     bc2 = DirichletBC(V, u_0, "x[0] > 0.5 && on_boundary")
@@ -646,8 +647,8 @@ def test_p32_box_3():
 def test_p33_box_1():
     mesh = UnitSquareMesh(3, 3)
     V = FunctionSpace(mesh, "CG", 1)
-    f = Expression("0.0")
-    u_0 = Expression("sin(x[0])")
+    f = Expression("0.0", degree=0)
+    u_0 = Expression("sin(x[0])", degree=1)
     bc0 = DirichletBC(V, u_0, "x[0] > 0.5 && on_boundary")
     bc1 = DirichletBC(V, u_0, "x[0] > 0.5 && on_boundary")
     bc2 = DirichletBC(V, u_0, "x[0] > 0.5 && on_boundary")
@@ -733,6 +734,7 @@ def test_p36_box_1(cd_tempdir, pushpop_parameters):
 
 
 @skip_in_parallel
+@skip_if_not_HDF5
 def test_p37_box_1(cd_tempdir):
     mesh = UnitSquareMesh(3, 3)
     V = FunctionSpace(mesh, "CG", 1)
@@ -749,6 +751,7 @@ def test_p37_box_1(cd_tempdir):
 
 
 @skip_in_parallel
+@skip_if_not_HDF5
 def test_p37_box_2():
     time_series = TimeSeries(mpi_comm_world(), "simulation_data")
     mesh = UnitSquareMesh(3, 3)
@@ -853,9 +856,9 @@ def test_p41_box_2():
     solver = KrylovSolver()
     solver.parameters["absolute_tolerance"] = 1e-6
     solver.parameters["report"] = True
-    solver.parameters["gmres"]["restart"] = 50
+    #solver.parameters["gmres"]["restart"] = 50
     #solver.parameters["preconditioner"]["reuse"] = True
-    solver.parameters["preconditioner"]["structure"] = "same"
+    #solver.parameters["preconditioner"]["structure"] = "same"
 
 
 @use_gc_barrier
@@ -867,7 +870,7 @@ def test_p42_box_1():
 
 @use_gc_barrier
 def test_p42_box_2(pushpop_parameters):
-    d = dict(num_threads=4, krylov_solver=dict(absolute_tolerance=1e-6))
+    d = dict(num_threads=4, krylov_solver=dict(absolute_tolerance=1.0e-6))
     parameters.update(d)
 
 
@@ -929,14 +932,14 @@ def test_p47_box_2():
         def eval(self, values, x):
             values[0] = sin(x[0])
     v = TestFunction(V)
-    f = Source()
+    f = Source(degree=1)
     L = f*v*dx
     assemble(L)
 
 
 @use_gc_barrier
 def test_p48_box_1():
-    e = Expression("sin(x[0])")
+    e = Expression("sin(x[0])", degree=2)
 
 
 @use_gc_barrier
@@ -945,14 +948,15 @@ def test_p49_box_1():
     V = FunctionSpace(mesh, "Lagrange", 1)
     u = TrialFunction(V)
     v = TestFunction(V)
-    c = Expression("sin(x[0])")
+    c = Expression("sin(x[0])", degree=2)
     a = c*dot(grad(u), grad(v))*dx
     A = assemble(a)
 
 
 @use_gc_barrier
 def test_p50_box_1(pushpop_parameters):
-    parameters["form_compiler"]["name"] = "sfc"
+    # This parameter has been removed, as no other form compilers exist today:
+    pass  # parameters["form_compiler"]["name"] = "sfc"
 
 
 @use_gc_barrier
